// collections.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

boolean Collection::memberP(Object* object) {
  // Return `true' iff `object' is a member of the collection `self'.
  { Collection* self = this;

    { Object* i = NULL;
      Iterator* iter_000 = ((Iterator*)(self->allocate_iterator()));

      while (iter_000->nextP()) {
        i = iter_000->value;
        if (eqlP(i, object)) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

boolean stella_collectionP(Object* self) {
  // Return `true' if `self' is a native collection.
  return (isaP(self, SGT_COLLECTIONS_STELLA_COLLECTION));
}

boolean Collection::no_duplicatesP() {
  // Return `true' if the collection `self' forbids duplicate values.
  { Collection* self = this;

    return (FALSE);
  }
}

boolean Set::no_duplicatesP() {
  { Set* self = this;

    return (TRUE);
  }
}

boolean Collection::orderedP() {
  // Return `true' if the collection `self' is ordered.
  { Collection* self = this;

    return (FALSE);
  }
}

boolean Sequence::orderedP() {
  { Sequence* self = this;

    return (TRUE);
  }
}

boolean Set::emptyP() {
  // Return `true' if the set `self' has no members.
  { Set* self = this;

    return (self->the_cons_list == NIL);
  }
}

boolean Set::non_emptyP() {
  // Return `true' if the set `self' has at least one member.
  { Set* self = this;

    return (!(self->the_cons_list == NIL));
  }
}

boolean terminate_setP(Set* self) {
  { Cons* cons = self->the_cons_list;
    Cons* rest = NULL;

    while (!(cons == NIL)) {
      rest = cons->rest;
      cons->free();
      cons = rest;
    }
    self->the_cons_list = NIL;
    return (TRUE);
  }
}

boolean Set::memberP(Object* object) {
  // Return `true' iff `object' is a member of the set `self'.
  // Uses an `eql?' test.
  { Set* self = this;

    { Object* i = NULL;
      Cons* iter_000 = self->the_cons_list;

      while (!(iter_000 == NIL)) {
        i = iter_000->value;
        iter_000 = iter_000->rest;
        if (eqlP(i, object)) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

int Set::length() {
  { Set* self = this;

    return (self->the_cons_list->length());
  }
}

void Set::insert(Object* value) {
  // Add `value' to the set `self'.  First checks for duplicate.
  { Set* self = this;

    if (!self->memberP(value)) {
      self->the_cons_list = cons(value, self->the_cons_list);
    }
  }
}

Abstract_Collection* Set::remove(Object* value) {
  // Remove all entries in `self' that match `value'.
  { Set* self = this;

    self->the_cons_list = self->the_cons_list->remove(value);
    return (self);
  }
}

Set* Set::reverse() {
  { Set* self = this;

    self->the_cons_list = self->the_cons_list->reverse();
    return (self);
  }
}

Set* Set::copy() {
  // Return a copy of the set `self'.  The conses in the copy are
  // freshly allocated.
  { Set* self = this;

    { Set* copy = ((Set*)(create_object(self->primary_type(), 0)));

      copy->the_cons_list = copy_cons_list(self->the_cons_list);
      return (copy);
    }
  }
}

void Set::clear() {
  // Make `self' an empty set.
  { Set* self = this;

    self->the_cons_list = NIL;
  }
}

Abstract_Iterator* Set::allocate_iterator() {
  { Set* self = this;

    { Set_Iterator* iterator = new_set_iterator();

      iterator->set_iterator_cursor = self->the_cons_list;
      iterator->set_iterator_collection = self;
      iterator->first_iterationP = TRUE;
      return (iterator);
    }
  }
}

boolean terminate_set_iteratorP(Set_Iterator* self) {
  self->set_iterator_cursor = NIL;
  self->set_iterator_collection = NULL;
  return (TRUE);
}

boolean Set_Iterator::nextP() {
  { Set_Iterator* self = this;

    if (self->first_iterationP) {
      {
        self->first_iterationP = FALSE;
        self->set_iterator_cursor = self->set_iterator_collection->the_cons_list;
      }
    }
    else {
      self->set_iterator_cursor = self->set_iterator_cursor->rest;
    }
    self->value = self->set_iterator_cursor->value;
    return (!(self->set_iterator_cursor == NIL));
  }
}

Object* Set_Iterator::value_setter(Object* value) {
  { Set_Iterator* self = this;

    self->set_iterator_cursor->value = value;
    self->value = value;
    return (value);
  }
}

void Vector::print_vector(ostream* stream) {
  { Vector* self = this;

    if (self->length() == 0) {
      *(stream) << "|i|[]";
    }
    else {
      { int i = 0;
        int limit = 9;

        *(stream) << "|i|[";
        { Object* element = NULL;
          Vector* vector_000 = self;
          int index_000 = 0;
          int length_000 = vector_000->length();

          while (index_000 < length_000) {
            element = (vector_000->the_array)[index_000];
            index_000 = index_000 + 1;
            *(stream) << element;
            i = i + 1;
            if (i > limit) {
              break;
            }
            if (i < self->length()) {
              *(stream) << " ";
            }
          }
        }
        if ((i <= limit) ||
            (i == self->length())) {
          *(stream) << "]";
        }
        else {
          *(stream) << " ...]";
        }
      }
    }
  }
}

boolean Vector::emptyP() {
  // Return `true' if `self' has length 0.
  { Vector* self = this;

    return (self->array_size == 0);
  }
}

boolean Vector::non_emptyP() {
  // Return `true' if `self' has length > 0.
  { Vector* self = this;

    return (self->array_size > 0);
  }
}

Object* Vector::first() {
  { Vector* self = this;

    return ((self->the_array)[0]);
  }
}

Object* Vector::second() {
  { Vector* self = this;

    return ((self->the_array)[1]);
  }
}

Object* Vector::third() {
  { Vector* self = this;

    return ((self->the_array)[2]);
  }
}

Object* Vector::fourth() {
  { Vector* self = this;

    return ((self->the_array)[3]);
  }
}

Object* Vector::fifth() {
  { Vector* self = this;

    return ((self->the_array)[4]);
  }
}

Object* Vector::nth(int position) {
  { Vector* self = this;

    return ((self->the_array)[position]);
  }
}

Object* Vector::last() {
  // Return the last item in the vector `self'.
  { Vector* self = this;

    return ((self->the_array)[(self->array_size - 1)]);
  }
}

Object* Vector::first_setter(Object* value) {
  { Vector* self = this;

    return ((self->the_array)[0] = value);
  }
}

Object* Vector::second_setter(Object* value) {
  { Vector* self = this;

    return ((self->the_array)[1] = value);
  }
}

Object* Vector::third_setter(Object* value) {
  { Vector* self = this;

    return ((self->the_array)[2] = value);
  }
}

Object* Vector::fourth_setter(Object* value) {
  { Vector* self = this;

    return ((self->the_array)[3] = value);
  }
}

Object* Vector::fifth_setter(Object* value) {
  { Vector* self = this;

    return ((self->the_array)[4] = value);
  }
}

Object* Vector::nth_setter(Object* value, int position) {
  { Vector* self = this;

    return ((self->the_array)[position] = value);
  }
}

Object* Vector::last_setter(Object* value) {
  { Vector* self = this;

    return ((self->the_array)[(self->array_size - 1)] = value);
  }
}

int Vector::length() {
  { Vector* self = this;

    return (self->array_size);
  }
}

boolean Vector::memberP(Object* object) {
  { Vector* self = this;

    { Object** array = self->the_array;

      { int i = NULL_INTEGER;
        int iter_000 = 0;
        int upper_bound_000 = self->length() - 1;

        while (iter_000 <= upper_bound_000) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          if (eqlP(array[i], object)) {
            return (TRUE);
          }
        }
      }
      return (FALSE);
    }
  }
}

int Vector::position(Object* object, int start) {
  // Return the position of `object' within the vector
  // `self' (counting from zero); or return `null' if `object' does not occur within 
  // `self' (uses an `eql?' test).  If `start' was supplied as non-`null', only 
  // consider the portion starting at `start', however, the returned position 
  // will always be relative to the entire vector.
  { Vector* self = this;

    if (start == NULL_INTEGER) {
      start = 0;
    }
    { int i = NULL_INTEGER;
      int iter_000 = start;
      int upper_bound_000 = self->length() - 1;

      while (iter_000 <= upper_bound_000) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        if (eqlP((self->the_array)[i], object)) {
          return (i);
        }
      }
    }
    return (NULL_INTEGER);
  }
}

void Vector::insert_at(int offset, Object* value) {
  { Vector* self = this;

    if (offset >= self->array_size) {
      throw *new_stella_exception("Out-of-bounds array access");
    }
    (self->the_array)[offset] = value;
  }
}

Vector* Vector::copy() {
  // Return a copy of the vector `self'.
  { Vector* self = this;

    { int size = self->array_size;
      Object** sourcearray = self->the_array;
      Vector* copy = new_vector(size);
      Object** copyarray = copy->the_array;

      { int i = NULL_INTEGER;
        int iter_000 = 0;
        int upper_bound_000 = size - 1;

        while (iter_000 <= upper_bound_000) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          copyarray[i] = (sourcearray[i]);
        }
      }
      return (copy);
    }
  }
}

void Vector::clear() {
  { Vector* self = this;

    { int size = self->array_size;
      Object** array = self->the_array;

      { int i = NULL_INTEGER;
        int iter_000 = 0;
        int upper_bound_000 = size - 1;

        while (iter_000 <= upper_bound_000) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          array[i] = NULL;
        }
      }
    }
  }
}

Vector* vector_(int values, ...) {
  // Return a vector containing `values', in order.
  { Vector* vector = new_vector(values);

    { Object* v = NULL;
      int i = NULL_INTEGER;
      va_list iter_000;
      int iter_000_count = values;
      int iter_001 = 0;

      va_start(iter_000, values);
      while ((iter_000_count--) > 0) {
        v = va_arg(iter_000, Object*);
        i = iter_001;
        iter_001 = iter_001 + 1;
        (vector->the_array)[i] = v;
      }
      va_end(iter_000);
    }
    return (vector);
  }
}

boolean Vector::object_equalP(Object* y) {
  // Return TRUE iff the vectors `x' and `y' are structurally
  // equivalent.  Uses `equal?' to test equality of elements.
  { Vector* x = this;

    if (subtype_ofP(safe_primary_type(y), SGT_COLLECTIONS_STELLA_VECTOR)) {
      { Object* y_000 = y;
        Vector* y = ((Vector*)(y_000));

        if (x->array_size == y->array_size) {
          { Object* eltx = NULL;
            Object* elty = NULL;
            Vector* vector_000 = x;
            int index_000 = 0;
            int length_000 = vector_000->length();
            Vector* vector_001 = y;
            int index_001 = 0;
            int length_001 = vector_001->length();

            while ((index_000 < length_000) &&
                (index_001 < length_001)) {
              eltx = (vector_000->the_array)[index_000];
              index_000 = index_000 + 1;
              elty = (vector_001->the_array)[index_001];
              index_001 = index_001 + 1;
              if (!equalP(eltx, elty)) {
                return (FALSE);
              }
            }
          }
          return (TRUE);
        }
      }
    }
    else {
    }
    return (FALSE);
  }
}

void Extensible_Vector::insert_at(int offset, Object* value) {
  { Extensible_Vector* self = this;

    if (offset >= self->array_size) {
      { int size = self->array_size;

        while (offset >= size) {
          size = 2 * size;
        }
        resize_vector(self, size);
      }
    }
    (self->the_array)[offset] = value;
  }
}

void Extensible_Symbol_Array::insert_at(int offset, Object* value) {
  { Extensible_Symbol_Array* self = this;

    if (offset >= self->array_size) {
      { int size = self->array_size;

        while (offset >= size) {
          size = 2 * size;
        }
        resize_vector(self, size);
      }
    }
    (self->the_array)[offset] = value;
    if (offset > self->top_symbol_offset) {
      self->top_symbol_offset = offset;
    }
  }
}

void Vector_Sequence::insert(Object* value) {
  // Append `value' to the END of the sequence `self'.  
  // Resize the array if necessary.
  { Vector_Sequence* self = this;

    { int oldlength = self->sequence_length;

      if (oldlength == self->array_size) {
        resize_vector(self, self->array_size + self->resize_increment_reader());
      }
      (self->the_array)[oldlength] = value;
      self->sequence_length = oldlength + 1;
    }
  }
}

Abstract_Collection* Vector_Sequence::remove(Object* value) {
  // Remove `value' from the sequence `self', and left shift
  // the values after it to close the gap.
  { Vector_Sequence* self = this;

    { Object** array = self->the_array;
      int firstshiftoffset = -1;
      int lastshiftoffset = self->sequence_length - 1;

      { int i = NULL_INTEGER;
        int iter_000 = 0;
        int upper_bound_000 = lastshiftoffset;
        boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

        while (unboundedP_000 ||
            (iter_000 <= upper_bound_000)) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          if (eqlP(array[i], value)) {
            firstshiftoffset = i + 1;
            break;
          }
        }
      }
      if (firstshiftoffset == -1) {
        return (self);
      }
      if (firstshiftoffset <= lastshiftoffset) {
        { int j = NULL_INTEGER;
          int iter_001 = firstshiftoffset;
          int upper_bound_001 = lastshiftoffset;
          boolean unboundedP_001 = upper_bound_001 == NULL_INTEGER;

          while (unboundedP_001 ||
              (iter_001 <= upper_bound_001)) {
            j = iter_001;
            iter_001 = iter_001 + 1;
            array[(j - 1)] = (array[j]);
          }
        }
      }
      array[lastshiftoffset] = NULL;
      self->sequence_length = lastshiftoffset;
      return (self);
    }
  }
}

int Vector_Sequence::length() {
  { Vector_Sequence* self = this;

    return (self->sequence_length);
  }
}

Abstract_Iterator* Vector::allocate_iterator() {
  { Vector* self = this;

    { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

      iterator->iterator_object = self;
      iterator->iterator_integer = 0;
      iterator->iterator_second_integer = self->array_size;
      iterator->iterator_next_code = ((cpp_function_code)(&vector_nextP));
      return (iterator);
    }
  }
}

boolean vector_nextP(All_Purpose_Iterator* self) {
  { int cursor = self->iterator_integer;

    if (cursor < self->iterator_second_integer) {
      self->value = (((Vector*)(self->iterator_object))->the_array)[cursor];
      self->iterator_integer = cursor + 1;
      return (TRUE);
    }
    return (FALSE);
  }
}

Iterator* Vector::but_last() {
  // Generate all but the last element of the vector `self'.
  { Vector* self = this;

    { All_Purpose_Iterator* iterator = new_all_purpose_iterator();

      iterator->iterator_object = self;
      iterator->iterator_integer = 0;
      iterator->iterator_second_integer = self->array_size - 1;
      iterator->iterator_next_code = ((cpp_function_code)(&vector_nextP));
      return (iterator);
    }
  }
}

boolean oUSE_STELLA_HASH_TABLESpo = FALSE;

Stella_Hash_Table* new_stella_hash_table() {
  { Stella_Hash_Table* self = NULL;

    self = new Stella_Hash_Table();
    self->the_stella_hash_table = NULL;
    self->the_hash_table = NULL;
    self->free_elements = NULL_INTEGER;
    self->initial_size = 50;
    self->size = NULL_INTEGER;
    self->the_table = NULL;
    self->initialize_hash_table();
    return (self);
  }
}

Surrogate* Stella_Hash_Table::primary_type() {
  { Stella_Hash_Table* self = this;

    return (SGT_COLLECTIONS_STELLA_STELLA_HASH_TABLE);
  }
}

Object* access_stella_hash_table_slot_value(Stella_Hash_Table* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_COLLECTIONS_STELLA_SIZE) {
    if (setvalueP) {
      self->size = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->size);
    }
  }
  else if (slotname == SYM_COLLECTIONS_STELLA_INITIAL_SIZE) {
    if (setvalueP) {
      self->initial_size = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->initial_size);
    }
  }
  else if (slotname == SYM_COLLECTIONS_STELLA_FREE_ELEMENTS) {
    if (setvalueP) {
      self->free_elements = ((Integer_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_integer(self->free_elements);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

double oSTELLA_HASH_TABLE_AVG_BUCKET_LENGTHo = 1.0;

void initialize_stella_hash_table(Stella_Hash_Table* self) {
  { int size = pick_hash_table_size_prime(floor_(self->initial_size / oSTELLA_HASH_TABLE_AVG_BUCKET_LENGTHo));
    Kv_Cons** table = new (GC)(Kv_Cons*[size]);

    { int i = NULL_INTEGER;
      int iter_000 = 0;
      int upper_bound_000 = size - 1;

      while (iter_000 <= upper_bound_000) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        table[i] = NULL;
      }
    }
    self->the_table = table;
    self->size = size;
    self->free_elements = floor_(size * oSTELLA_HASH_TABLE_AVG_BUCKET_LENGTHo);
  }
}

Stella_Hash_Table* make_stella_hash_table(int initialsize) {
  { Stella_Hash_Table* table = new Stella_Hash_Table();

    table->the_table = NULL;
    table->size = NULL_INTEGER;
    table->initial_size = initialsize;
    table->free_elements = NULL_INTEGER;
    return (table);
  }
}

void rehash_stella_hash_table(Stella_Hash_Table* self, int newsize) {
  if (self->the_table == NULL) {
    initialize_stella_hash_table(self);
    return;
  }
  { int size = self->size;
    Kv_Cons** table = self->the_table;
    Kv_Cons** newtable = new (GC)(Kv_Cons*[newsize]);
    int newbucketindex = 0;
    Kv_Cons* newbucket = NULL;
    Kv_Cons* cursor = NULL;
    Kv_Cons* current = NULL;

    { int i = NULL_INTEGER;
      int iter_000 = 0;
      int upper_bound_000 = newsize - 1;

      while (iter_000 <= upper_bound_000) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        newtable[i] = NULL;
      }
    }
    { int i = NULL_INTEGER;
      int iter_001 = 0;
      int upper_bound_001 = size - 1;

      while (iter_001 <= upper_bound_001) {
        i = iter_001;
        iter_001 = iter_001 + 1;
        cursor = table[i];
        while (((boolean)(cursor))) {
          newbucketindex = ((cursor->key->hash_code()) % newsize);
          newbucket = newtable[newbucketindex];
          current = cursor;
          cursor = cursor->rest;
          if (((boolean)(newbucket))) {
            current->rest = newbucket->rest;
            newbucket->rest = current;
          }
          else {
            newtable[newbucketindex] = current;
            current->rest = NULL;
          }
        }
      }
    }
    self->the_table = newtable;
    self->size = newsize;
    self->free_elements = max_(self->free_elements + floor_((newsize - size) * oSTELLA_HASH_TABLE_AVG_BUCKET_LENGTHo), 0);
  }
}

void stella_hash_table_insert_at(Stella_Hash_Table* self, Object* key, Object* value) {
  { Kv_Cons** table = self->the_table;
    int free = self->free_elements;
    int bucketindex = 0;
    Kv_Cons* bucket = NULL;

    if (table == NULL) {
      initialize_stella_hash_table(self);
      table = self->the_table;
      free = self->free_elements;
    }
    if (free == 0) {
      rehash_stella_hash_table(self, pick_hash_table_size_prime(self->size + 1));
      table = self->the_table;
      free = self->free_elements;
    }
    bucketindex = ((key->hash_code()) % (self->size));
    bucket = table[bucketindex];
    if (!((boolean)(bucket))) {
      { Kv_Cons* self_000 = new_kv_cons();

        self_000->key = key;
        self_000->value = value;
        table[bucketindex] = self_000;
      }
      self->free_elements = free - 1;
    }
    else {
      { Kv_Cons* cursor = bucket;

        while (((boolean)(cursor)) &&
            (!eqlP(cursor->key, key))) {
          cursor = cursor->rest;
        }
        if (((boolean)(cursor))) {
          cursor->value = value;
        }
        else {
          { Kv_Cons* self_001 = new_kv_cons();

            self_001->key = key;
            self_001->value = value;
            self_001->rest = bucket->rest;
            bucket->rest = self_001;
          }
          self->free_elements = free - 1;
        }
      }
    }
  }
}

void stella_hash_table_remove_at(Stella_Hash_Table* self, Object* key) {
  { Kv_Cons** table = self->the_table;
    int bucketindex = 0;
    Kv_Cons* bucket = NULL;

    if (table == NULL) {
      return;
    }
    bucketindex = ((key->hash_code()) % (self->size));
    bucket = table[bucketindex];
    if (!((boolean)(bucket))) {
      return;
    }
    else if (eqlP(bucket->key, key)) {
      table[bucketindex] = bucket->rest;
      self->free_elements = self->free_elements + 1;
      return;
    }
    else {
      { Kv_Cons* trailer = bucket;

        bucket = bucket->rest;
        while (((boolean)(bucket))) {
          if (eqlP(bucket->key, key)) {
            trailer->rest = bucket->rest;
            self->free_elements = self->free_elements + 1;
            return;
          }
          else {
            trailer = bucket;
            bucket = bucket->rest;
          }
        }
      }
    }
  }
}

Object* stella_hash_table_lookup(Stella_Hash_Table* self, Object* key) {
  { Kv_Cons** table = self->the_table;
    Kv_Cons* bucket = NULL;

    if (table == NULL) {
      return (NULL);
    }
    bucket = table[((key->hash_code()) % (self->size))];
    while (((boolean)(bucket))) {
      if (eqlP(bucket->key, key)) {
        return (bucket->value);
      }
      else {
        bucket = bucket->rest;
      }
    }
    return (NULL);
  }
}

Object* stella_string_hash_table_lookup(Stella_Hash_Table* self, char* key) {
  { Kv_Cons** table = self->the_table;
    Kv_Cons* bucket = NULL;

    if (table == NULL) {
      return (NULL);
    }
    bucket = table[((hash_string(key, 0)) % (self->size))];
    while (((boolean)(bucket))) {
      if (string_eqlP(((String_Wrapper*)(bucket->key))->wrapper_value, key)) {
        return (bucket->value);
      }
      else {
        bucket = bucket->rest;
      }
    }
    return (NULL);
  }
}

void help_startup_collections1() {
  {
    SGT_COLLECTIONS_STELLA_COLLECTION = ((Surrogate*)(intern_rigid_symbol_wrt_module("COLLECTION", NULL, 1)));
    SYM_COLLECTIONS_STELLA_VECTOR = ((Symbol*)(intern_rigid_symbol_wrt_module("VECTOR", NULL, 0)));
    KWD_COLLECTIONS_CPP = ((Keyword*)(intern_rigid_symbol_wrt_module("CPP", NULL, 2)));
    KWD_COLLECTIONS_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("FUNCTION", NULL, 2)));
    SGT_COLLECTIONS_STELLA_VECTOR = ((Surrogate*)(intern_rigid_symbol_wrt_module("VECTOR", NULL, 1)));
    SGT_COLLECTIONS_STELLA_STELLA_HASH_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("STELLA-HASH-TABLE", NULL, 1)));
    SYM_COLLECTIONS_STELLA_SIZE = ((Symbol*)(intern_rigid_symbol_wrt_module("SIZE", NULL, 0)));
    SYM_COLLECTIONS_STELLA_INITIAL_SIZE = ((Symbol*)(intern_rigid_symbol_wrt_module("INITIAL-SIZE", NULL, 0)));
    SYM_COLLECTIONS_STELLA_FREE_ELEMENTS = ((Symbol*)(intern_rigid_symbol_wrt_module("FREE-ELEMENTS", NULL, 0)));
    SYM_COLLECTIONS_STELLA_STARTUP_COLLECTIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-COLLECTIONS", NULL, 0)));
    SYM_COLLECTIONS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_collections2() {
  {
    define_method_object("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF COLLECTION) (OBJECT (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' iff `object' is a member of the collection `self'.\")", ((cpp_method_code)(&Collection::memberP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE-DUPLICATES (LIKE SELF)) ((SELF COLLECTION)) :DOCUMENTATION \"Return `self' with duplicates removed.  Preserves the\noriginal order of the remaining members.\" :PUBLIC? TRUE :ABSTRACT? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("STELLA-COLLECTION?", "(DEFUN (STELLA-COLLECTION? BOOLEAN) ((SELF OBJECT)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if `self' is a native collection.\")", ((cpp_function_code)(&stella_collectionP)), NULL);
    define_method_object("(DEFMETHOD (NO-DUPLICATES? BOOLEAN) ((SELF COLLECTION)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the collection `self' forbids duplicate values.\")", ((cpp_method_code)(&Collection::no_duplicatesP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NO-DUPLICATES? BOOLEAN) ((SELF SET)) :PUBLIC? TRUE)", ((cpp_method_code)(&Set::no_duplicatesP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ORDERED? BOOLEAN) ((SELF COLLECTION)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the collection `self' is ordered.\")", ((cpp_method_code)(&Collection::orderedP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ORDERED? BOOLEAN) ((SELF SEQUENCE)) :PUBLIC? TRUE)", ((cpp_method_code)(&Sequence::orderedP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the set `self' has no members.\")", ((cpp_method_code)(&Set::emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if the set `self' has at least one member.\")", ((cpp_method_code)(&Set::non_emptyP)), ((cpp_method_code)(NULL)));
    define_function_object("TERMINATE-SET?", "(DEFUN (TERMINATE-SET? BOOLEAN) ((SELF SET)))", ((cpp_function_code)(&terminate_setP)), NULL);
    define_method_object("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF SET) (OBJECT (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Return `true' iff `object' is a member of the set `self'.\nUses an `eql?' test.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Set::memberP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF SET)) :PUBLIC? TRUE)", ((cpp_method_code)(&Set::length)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT ((SELF SET) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Add `value' to the set `self'.  First checks for duplicate.\")", ((cpp_method_code)(&Set::insert)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE SET) ((SELF SET) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Remove all entries in `self' that match `value'.\")", ((cpp_method_code)(&Set::remove)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REVERSE SET) ((SELF SET)) :PUBLIC? TRUE)", ((cpp_method_code)(&Set::reverse)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY (SET OF (LIKE (ANY-VALUE SELF)))) ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Return a copy of the set `self'.  The conses in the copy are\nfreshly allocated.\")", ((cpp_method_code)(&Set::copy)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD CLEAR ((SELF SET)) :PUBLIC? TRUE :DOCUMENTATION \"Make `self' an empty set.\")", ((cpp_method_code)(&Set::clear)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ALLOCATE-ITERATOR (SET-ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF SET)))", ((cpp_method_code)(&Set::allocate_iterator)), ((cpp_method_code)(NULL)));
    define_function_object("TERMINATE-SET-ITERATOR?", "(DEFUN (TERMINATE-SET-ITERATOR? BOOLEAN) ((SELF SET-ITERATOR)))", ((cpp_function_code)(&terminate_set_iteratorP)), NULL);
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF SET-ITERATOR)))", ((cpp_method_code)(&Set_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (VALUE-SETTER OBJECT) ((SELF SET-ITERATOR) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&Set_Iterator::value_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD PRINT-VECTOR ((SELF VECTOR) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_method_code)(&Vector::print_vector)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF VECTOR)) :DOCUMENTATION \"Return `true' if `self' has length 0.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (EQL? (ARRAY-SIZE SELF) 0)))", ((cpp_method_code)(&Vector::emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF VECTOR)) :PUBLIC? TRUE :DOCUMENTATION \"Return `true' if `self' has length > 0.\" :GLOBALLY-INLINE? TRUE (RETURN (> (ARRAY-SIZE SELF) 0)))", ((cpp_method_code)(&Vector::non_emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIRST (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 0)))", ((cpp_method_code)(&Vector::first)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SECOND (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 1)))", ((cpp_method_code)(&Vector::second)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (THIRD (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 2)))", ((cpp_method_code)(&Vector::third)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FOURTH (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 3)))", ((cpp_method_code)(&Vector::fourth)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIFTH (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH SELF 4)))", ((cpp_method_code)(&Vector::fifth)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (POSITION INTEGER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (NTH (THE-ARRAY SELF) POSITION)))", ((cpp_method_code)(&Vector::nth)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LAST (LIKE (ANY-VALUE SELF))) ((SELF VECTOR)) :PUBLIC? TRUE :DOCUMENTATION \"Return the last item in the vector `self'.\")", ((cpp_method_code)(&Vector::last)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIRST-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 0) VALUE)))", ((cpp_method_code)(&Vector::first_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SECOND-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 1) VALUE)))", ((cpp_method_code)(&Vector::second_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (THIRD-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 2) VALUE)))", ((cpp_method_code)(&Vector::third_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FOURTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 3) VALUE)))", ((cpp_method_code)(&Vector::fourth_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIFTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH SELF 4) VALUE)))", ((cpp_method_code)(&Vector::fifth_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF))) (POSITION INTEGER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (SETF (NTH (THE-ARRAY SELF) POSITION) VALUE)))", ((cpp_method_code)(&Vector::nth_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LAST-SETTER (LIKE (ANY-VALUE SELF))) ((SELF VECTOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::last_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF VECTOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::length)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF VECTOR) (OBJECT OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::memberP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (POSITION INTEGER) ((SELF VECTOR) (OBJECT OBJECT) (START INTEGER)) :DOCUMENTATION \"Return the position of `object' within the vector\n`self' (counting from zero); or return `null' if `object' does not occur within \n`self' (uses an `eql?' test).  If `start' was supplied as non-`null', only \nconsider the portion starting at `start', however, the returned position \nwill always be relative to the entire vector.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::position)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-AT ((SELF VECTOR) (OFFSET INTEGER) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&Vector::insert_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY (VECTOR OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)) :DOCUMENTATION \"Return a copy of the vector `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::copy)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD CLEAR ((SELF VECTOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::clear)), ((cpp_method_code)(NULL)));
    define_function_object("VECTOR", "(DEFUN (VECTOR VECTOR) (|&REST| (VALUES OBJECT)) :DOCUMENTATION \"Return a vector containing `values', in order.\" :PUBLIC? TRUE)", ((cpp_function_code)(&vector_)), NULL);
    define_method_object("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X VECTOR) (Y OBJECT)) :DOCUMENTATION \"Return TRUE iff the vectors `x' and `y' are structurally\nequivalent.  Uses `equal?' to test equality of elements.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::object_equalP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-AT ((SELF EXTENSIBLE-VECTOR) (OFFSET INTEGER) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Extensible_Vector::insert_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-AT ((SELF EXTENSIBLE-SYMBOL-ARRAY) (OFFSET INTEGER) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Extensible_Symbol_Array::insert_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT ((SELF VECTOR-SEQUENCE) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Append `value' to the END of the sequence `self'.  \nResize the array if necessary.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector_Sequence::insert)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE VECTOR-SEQUENCE) ((SELF VECTOR-SEQUENCE) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Remove `value' from the sequence `self', and left shift\nthe values after it to close the gap.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector_Sequence::remove)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF VECTOR-SEQUENCE)) :PUBLIC? TRUE)", ((cpp_method_code)(&Vector_Sequence::length)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ALLOCATE-ITERATOR (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)))", ((cpp_method_code)(&Vector::allocate_iterator)), ((cpp_method_code)(NULL)));
    define_function_object("VECTOR-NEXT?", "(DEFUN (VECTOR-NEXT? BOOLEAN) ((SELF ALL-PURPOSE-ITERATOR)))", ((cpp_function_code)(&vector_nextP)), NULL);
    define_method_object("(DEFMETHOD (BUT-LAST (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)) :DOCUMENTATION \"Generate all but the last element of the vector `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::but_last)), ((cpp_method_code)(NULL)));
    define_function_object("INITIALIZE-STELLA-HASH-TABLE", "(DEFUN INITIALIZE-STELLA-HASH-TABLE ((SELF STELLA-HASH-TABLE)))", ((cpp_function_code)(&initialize_stella_hash_table)), NULL);
    define_function_object("MAKE-STELLA-HASH-TABLE", "(DEFUN (MAKE-STELLA-HASH-TABLE STELLA-HASH-TABLE) ((INITIALSIZE INTEGER)))", ((cpp_function_code)(&make_stella_hash_table)), NULL);
    define_function_object("REHASH-STELLA-HASH-TABLE", "(DEFUN REHASH-STELLA-HASH-TABLE ((SELF STELLA-HASH-TABLE) (NEWSIZE INTEGER)))", ((cpp_function_code)(&rehash_stella_hash_table)), NULL);
    define_function_object("STELLA-HASH-TABLE-INSERT-AT", "(DEFUN STELLA-HASH-TABLE-INSERT-AT ((SELF STELLA-HASH-TABLE) (KEY OBJECT) (VALUE OBJECT)))", ((cpp_function_code)(&stella_hash_table_insert_at)), NULL);
    define_function_object("STELLA-HASH-TABLE-REMOVE-AT", "(DEFUN STELLA-HASH-TABLE-REMOVE-AT ((SELF STELLA-HASH-TABLE) (KEY OBJECT)))", ((cpp_function_code)(&stella_hash_table_remove_at)), NULL);
    define_function_object("STELLA-HASH-TABLE-LOOKUP", "(DEFUN (STELLA-HASH-TABLE-LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF STELLA-HASH-TABLE) (KEY OBJECT)))", ((cpp_function_code)(&stella_hash_table_lookup)), NULL);
  }
}

void startup_collections() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_collections1();
    }
    if (current_startup_time_phaseP(5)) {
      { Class* renamed_Class = define_class_from_stringified_source("STELLA-HASH-TABLE", "(DEFCLASS STELLA-HASH-TABLE (ABSTRACT-HASH-TABLE) :PARAMETERS ((ANY-KEY :TYPE OBJECT) (ANY-VALUE :TYPE OBJECT)) :SLOTS ((THE-TABLE :TYPE (ARRAY () OF KV-CONS)) (SIZE :TYPE INTEGER) (INITIAL-SIZE :TYPE INTEGER :INITIALLY 50) (FREE-ELEMENTS :TYPE INTEGER)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_stella_hash_table));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_stella_hash_table_slot_value));
      }
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_collections2();
      define_function_object("STELLA-STRING-HASH-TABLE-LOOKUP", "(DEFUN (STELLA-STRING-HASH-TABLE-LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF STELLA-HASH-TABLE) (KEY STRING)))", ((cpp_function_code)(&stella_string_hash_table_lookup)), NULL);
      define_function_object("STARTUP-COLLECTIONS", "(DEFUN STARTUP-COLLECTIONS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_collections)), NULL);
      { Method_Slot* function = lookup_function(SYM_COLLECTIONS_STELLA_STARTUP_COLLECTIONS);

        set_dynamic_slot_value(function->dynamic_slots, SYM_COLLECTIONS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Collections"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      register_native_name(SYM_COLLECTIONS_STELLA_VECTOR, KWD_COLLECTIONS_CPP, KWD_COLLECTIONS_FUNCTION);
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *USE-STELLA-HASH-TABLES?* BOOLEAN FALSE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STELLA-HASH-TABLE-AVG-BUCKET-LENGTH* FLOAT 1.0)");
    }
  }
}

Surrogate* SGT_COLLECTIONS_STELLA_COLLECTION = NULL;

Symbol* SYM_COLLECTIONS_STELLA_VECTOR = NULL;

Keyword* KWD_COLLECTIONS_CPP = NULL;

Keyword* KWD_COLLECTIONS_FUNCTION = NULL;

Surrogate* SGT_COLLECTIONS_STELLA_VECTOR = NULL;

Surrogate* SGT_COLLECTIONS_STELLA_STELLA_HASH_TABLE = NULL;

Symbol* SYM_COLLECTIONS_STELLA_SIZE = NULL;

Symbol* SYM_COLLECTIONS_STELLA_INITIAL_SIZE = NULL;

Symbol* SYM_COLLECTIONS_STELLA_FREE_ELEMENTS = NULL;

Symbol* SYM_COLLECTIONS_STELLA_STARTUP_COLLECTIONS = NULL;

Symbol* SYM_COLLECTIONS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
