// demons.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

// Table containing all active and inactive demons, indexed
// on their names.
String_Hash_Table* oDEMONS_TABLEo = NULL;

// These demons trigger every time an active instance is created.
List* oCONSTRUCTOR_DEMONSo = NULL;

// These demons trigger every time an active instance is destroyed.
List* oDESTRUCTOR_DEMONSo = NULL;

// These demons trigger every time a slot value is modified.
List* oALL_SLOTS_DEMONSo = NULL;

// These demons trigger every time a slot value is modified, and
// cancel slot update when they return `false'.
List* oALL_SLOTS_GUARD_DEMONSo = NULL;

boolean validate_demon_referencesP(Demon* demon) {
  { Surrogate* classref = (((boolean)(demon->demon_class_refs)) ? ((Surrogate*)(demon->demon_class_refs->first())) : ((Surrogate*)(NULL)));
    Symbol* slotref = (((boolean)(demon->demon_slot_refs)) ? ((Symbol*)(demon->demon_slot_refs->first())) : ((Symbol*)(NULL)));
    Slot* slot = NULL;

    if ((!((boolean)(classref))) &&
        (!((boolean)(slotref)))) {
      return (TRUE);
    }
    if (!((boolean)(classref))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Missing :class option in demon " << "`" << demon << "'" << endl;
      return (FALSE);
    }
    if (!((boolean)(((Class*)(classref->surrogate_value))))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Class " << "`" << classref << "'" << " not yet defined." << endl;
      return (FALSE);
    }
    if (demon->demon_action == KWD_DEMONS_MODIFY) {
      if (!((boolean)(slotref))) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Missing :slot option in demon " << "`" << demon << "'" << endl;
        return (FALSE);
      }
      { boolean test_value_000 = FALSE;

        { 
          slot = lookup_slot(((Class*)(classref->surrogate_value)), slotref);
          test_value_000 = ((boolean)(slot));
        }
        test_value_000 = !test_value_000;
        if (test_value_000) {
          *(STANDARD_WARNING->native_stream) << "Warning: " << "No slot named " << "`" << slotref << "'" << " exists on class " << "`" << ((Class*)(classref->surrogate_value)) << "'" << "." << endl << "Can't create demon " << "`" << demon << "'" << endl;
          return (FALSE);
        }
      }
      if (!storage_slotP(slot)) {
        *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't install a demon on the method." << "`" << slot << "'" << endl;
        return (FALSE);
      }
    }
    return (TRUE);
  }
}

boolean incorporate_demon_optionsP(Demon* demon, Property_List* options) {
  if (((boolean)(options->lookup(KWD_DEMONS_CREATE)))) {
    demon->demon_action = KWD_DEMONS_CREATE;
  }
  else if (((boolean)(options->lookup(KWD_DEMONS_DESTROY)))) {
    demon->demon_action = KWD_DEMONS_DESTROY;
  }
  else {
    demon->demon_action = KWD_DEMONS_MODIFY;
  }
  { Object* k = NULL;
    Object* v = NULL;
    Cons* iter_000 = options->the_plist;

    while (!(iter_000 == NIL)) {
      k = iter_000->value;
      v = iter_000->rest->value;
      iter_000 = iter_000->rest->rest;
      { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(k));

        if ((test_value_000 == KWD_DEMONS_CREATE) ||
            (test_value_000 == KWD_DEMONS_DESTROY)) {
          demon->demon_action = ((Keyword*)(k));
          if (v == KWD_DEMONS_ALL) {
            demon->demon_allP = TRUE;
          }
          else {
            demon->demon_class_refs->insert(v->surrogatify());
          }
        }
        else if (test_value_000 == KWD_DEMONS_CLASS) {
          demon->demon_class_refs->insert(v->surrogatify());
        }
        else if (test_value_000 == KWD_DEMONS_SLOT) {
          if (v == KWD_DEMONS_ALL) {
            demon->demon_allP = TRUE;
          }
          else {
            demon->demon_slot_refs->insert(((Symbol*)(v->permanentify())));
          }
        }
        else if (test_value_000 == KWD_DEMONS_CODE) {
          demon->demon_code = ((Function_Code_Wrapper*)(v))->wrapper_value;
        }
        else if (test_value_000 == KWD_DEMONS_METHOD) {
          demon->demon_method = ((Method_Slot*)(v));
        }
        else if (test_value_000 == KWD_DEMONS_GUARDp) {
          demon->demon_guardP = TRUE;
        }
        else if (test_value_000 == KWD_DEMONS_INHERITEDp) {
          demon->demon_inheritP = TRUE;
          cout << "Inherit? not yet implemented for demons" << endl;
        }
        else if (test_value_000 == KWD_DEMONS_DOCUMENTATION) {
          demon->demon_documentation = ((String_Wrapper*)(v))->wrapper_value;
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
    }
  }
  if (((boolean)(demon->demon_method))) {
    demon->demon_code = demon->demon_method->function_code;
  }
  options->free();
  return (validate_demon_referencesP(demon));
}

Demon* lookup_demon(char* name) {
  // Return the demon named `name'.
  return (((Demon*)(oDEMONS_TABLEo->lookup(name))));
}

Demon* define_demon(char* name, int options, ...) {
  // Define a class or slot demon.  Options are :create,
  // :destroy, :class, :slot, :guard?, :code, :method, :inherit?, and :documentation.
  { Demon* demon = new_demon();
    Property_List* plist = new_property_list();
    Cons* conslist = NIL;
    Demon* olddemon = NULL;
    boolean activateP = FALSE;

    { Object* item = NULL;
      va_list iter_000;
      int iter_000_count = options;
      Cons* collect_000 = NULL;

      va_start(iter_000, options);
      while ((iter_000_count--) > 0) {
        item = va_arg(iter_000, Object*);
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(item, NIL);
            if (conslist == NIL) {
              conslist = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(conslist, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(item, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
      va_end(iter_000);
    }
    plist->the_plist = conslist;
    demon->demon_name = name;
    activateP = plist->remove_at(KWD_DEMONS_ACTIVATEp) == TRUE_WRAPPER;
    if (incorporate_demon_optionsP(demon, plist)) {
      { 
        olddemon = lookup_demon(name);
        if (((boolean)(olddemon))) {
          deactivate_demon(olddemon);
          olddemon->free();
        }
      }
      oDEMONS_TABLEo->insert_at(demon->demon_name, demon);
      if (activateP) {
        activate_demon(demon);
      }
      return (demon);
    }
    demon->free();
    return (NULL);
  }
}

List* inject_demon(List* demonlist, Demon* demon) {
  if (null_listP(demonlist)) {
    demonlist = new_list();
  }
  { List_Iterator* it = ((List_Iterator*)(demonlist->allocate_iterator()));

    while (it->nextP()) {
      if (string_eqlP(((Demon*)(it->value))->demon_name, demon->demon_name)) {
        it->value_setter(demon);
        return (demonlist);
      }
    }
  }
  demonlist->insert(demon);
  return (demonlist);
}

List* eject_demon(List* demonlist, Demon* demon) {
  if (!((boolean)(demonlist))) {
    return (NULL);
  }
  demonlist->remove(demon);
  if (demonlist->emptyP()) {
    demonlist->free();
    demonlist = NULL;
  }
  return (demonlist);
}

void activate_demon(Demon* demon) {
  // Install `demon' in the location(s) specified by its internal
  // structure.
  if (!((boolean)(demon))) {
    return;
  }
  { Keyword* test_value_000 = demon->demon_action;

    if (test_value_000 == KWD_DEMONS_MODIFY) {
      if (demon->demon_allP) {
        if (demon->demon_guardP) {
          oALL_SLOTS_GUARD_DEMONSo = inject_demon(oALL_SLOTS_GUARD_DEMONSo, demon);
        }
        else {
          oALL_SLOTS_DEMONSo = inject_demon(oALL_SLOTS_DEMONSo, demon);
        }
      }
      else {
        { Surrogate* type = NULL;
          Symbol* slotname = NULL;
          Cons* iter_000 = demon->demon_class_refs->the_cons_list;
          Cons* iter_001 = demon->demon_slot_refs->the_cons_list;

          while ((!(iter_000 == NIL)) &&
              (!(iter_001 == NIL))) {
            type = ((Surrogate*)(iter_000->value));
            iter_000 = iter_000->rest;
            slotname = ((Symbol*)(iter_001->value));
            iter_001 = iter_001->rest;
            { Storage_Slot* slot = ((Storage_Slot*)(lookup_slot(((Class*)(type->surrogate_value)), slotname)));

              if (!((boolean)(slot))) {
                *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't install demon on non-existant slot " << "`" << ((Class*)(type->surrogate_value)) << "'" << "." << "`" << slotname << "'" << endl;
                continue;
              }
              if (demon->demon_guardP) {
                set_dynamic_slot_value(slot->dynamic_slots, SYM_DEMONS_STELLA_SLOT_GUARD_DEMONS, inject_demon(slot->slot_guard_demons_reader(), demon), NULL);
              }
              else {
                set_dynamic_slot_value(slot->dynamic_slots, SYM_DEMONS_STELLA_SLOT_DEMONS, inject_demon(slot->slot_demons_reader(), demon), NULL);
              }
            }
          }
        }
      }
    }
    else if (test_value_000 == KWD_DEMONS_CREATE) {
      if (demon->demon_allP) {
        oCONSTRUCTOR_DEMONSo = inject_demon(oCONSTRUCTOR_DEMONSo, demon);
      }
      else {
        { Surrogate* classref = NULL;
          Cons* iter_002 = demon->demon_class_refs->the_cons_list;

          while (!(iter_002 == NIL)) {
            classref = ((Surrogate*)(iter_002->value));
            iter_002 = iter_002->rest;
            { Class* renamed_Class = ((Class*)(classref->surrogate_value));

              if (!((boolean)(renamed_Class))) {
                *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't install demon on non-existant class " << "`" << classref << "'" << endl;
                continue;
              }
              if (demon->demon_guardP) {
                set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEMONS_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS, inject_demon(renamed_Class->class_guard_constructor_demons_reader(), demon), NULL);
              }
              else {
                set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEMONS_STELLA_CLASS_CONSTRUCTOR_DEMONS, inject_demon(renamed_Class->class_constructor_demons_reader(), demon), NULL);
              }
            }
          }
        }
      }
    }
    else if (test_value_000 == KWD_DEMONS_DESTROY) {
      if (demon->demon_allP) {
        oDESTRUCTOR_DEMONSo = inject_demon(oDESTRUCTOR_DEMONSo, demon);
      }
      else {
        { Surrogate* classref = NULL;
          Cons* iter_003 = demon->demon_class_refs->the_cons_list;

          while (!(iter_003 == NIL)) {
            classref = ((Surrogate*)(iter_003->value));
            iter_003 = iter_003->rest;
            { Class* renamed_Class = ((Class*)(classref->surrogate_value));

              if (!((boolean)(renamed_Class))) {
                *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't install demon on non-existant class " << "`" << classref << "'" << endl;
                continue;
              }
              if (demon->demon_guardP) {
                set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEMONS_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS, inject_demon(renamed_Class->class_guard_destructor_demons_reader(), demon), NULL);
              }
              else {
                set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEMONS_STELLA_CLASS_DESTRUCTOR_DEMONS, inject_demon(renamed_Class->class_destructor_demons_reader(), demon), NULL);
              }
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

void deactivate_demon(Demon* demon) {
  // Detach `demon' from the location(s) specified by its internal
  // structure.
  if (!((boolean)(demon))) {
    return;
  }
  { Keyword* test_value_000 = demon->demon_action;

    if (test_value_000 == KWD_DEMONS_MODIFY) {
      if (demon->demon_allP) {
        if (demon->demon_guardP) {
          oALL_SLOTS_GUARD_DEMONSo = eject_demon(oALL_SLOTS_GUARD_DEMONSo, demon);
        }
        else {
          oALL_SLOTS_DEMONSo = eject_demon(oALL_SLOTS_DEMONSo, demon);
        }
      }
      else {
        { Surrogate* type = NULL;
          Symbol* slotname = NULL;
          Cons* iter_000 = demon->demon_class_refs->the_cons_list;
          Cons* iter_001 = demon->demon_slot_refs->the_cons_list;

          while ((!(iter_000 == NIL)) &&
              (!(iter_001 == NIL))) {
            type = ((Surrogate*)(iter_000->value));
            iter_000 = iter_000->rest;
            slotname = ((Symbol*)(iter_001->value));
            iter_001 = iter_001->rest;
            { Storage_Slot* slot = ((Storage_Slot*)(lookup_slot(((Class*)(type->surrogate_value)), slotname)));

              if (!((boolean)(slot))) {
                *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't remove demon from non-existant slot " << "`" << ((Class*)(type->surrogate_value)) << "'" << "." << "`" << slotname << "'" << endl;
                continue;
              }
              if (demon->demon_guardP) {
                set_dynamic_slot_value(slot->dynamic_slots, SYM_DEMONS_STELLA_SLOT_GUARD_DEMONS, eject_demon(slot->slot_guard_demons_reader(), demon), NULL);
              }
              else {
                set_dynamic_slot_value(slot->dynamic_slots, SYM_DEMONS_STELLA_SLOT_DEMONS, eject_demon(slot->slot_demons_reader(), demon), NULL);
              }
            }
          }
        }
      }
    }
    else if (test_value_000 == KWD_DEMONS_CREATE) {
      if (demon->demon_allP) {
        oCONSTRUCTOR_DEMONSo = eject_demon(oCONSTRUCTOR_DEMONSo, demon);
      }
      else {
        { Surrogate* classref = NULL;
          Cons* iter_002 = demon->demon_class_refs->the_cons_list;

          while (!(iter_002 == NIL)) {
            classref = ((Surrogate*)(iter_002->value));
            iter_002 = iter_002->rest;
            { Class* renamed_Class = ((Class*)(classref->surrogate_value));

              if (!((boolean)(renamed_Class))) {
                *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't install demon on non-existant class " << "`" << classref << "'" << endl;
                continue;
              }
              if (demon->demon_guardP) {
                set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEMONS_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS, eject_demon(renamed_Class->class_guard_constructor_demons_reader(), demon), NULL);
              }
              else {
                set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEMONS_STELLA_CLASS_CONSTRUCTOR_DEMONS, eject_demon(renamed_Class->class_constructor_demons_reader(), demon), NULL);
              }
            }
          }
        }
      }
    }
    else if (test_value_000 == KWD_DEMONS_DESTROY) {
      if (demon->demon_allP) {
        oDESTRUCTOR_DEMONSo = eject_demon(oDESTRUCTOR_DEMONSo, demon);
      }
      else {
        { Surrogate* classref = NULL;
          Cons* iter_003 = demon->demon_class_refs->the_cons_list;

          while (!(iter_003 == NIL)) {
            classref = ((Surrogate*)(iter_003->value));
            iter_003 = iter_003->rest;
            { Class* renamed_Class = ((Class*)(classref->surrogate_value));

              if (!((boolean)(renamed_Class))) {
                *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't install demon on non-existant class " << "`" << classref << "'" << endl;
                continue;
              }
              if (demon->demon_guardP) {
                set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEMONS_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS, eject_demon(renamed_Class->class_guard_destructor_demons_reader(), demon), NULL);
              }
              else {
                set_dynamic_slot_value(renamed_Class->dynamic_slots, SYM_DEMONS_STELLA_CLASS_DESTRUCTOR_DEMONS, eject_demon(renamed_Class->class_destructor_demons_reader(), demon), NULL);
              }
            }
          }
        }
      }
    }
    else {
      { Output_String_Stream* stream_000 = new_output_string_stream();

        *(stream_000->native_stream) << "`" << test_value_000 << "'" << " is not a valid case option";
        throw *new_stella_exception(stream_000->the_string_reader());
      }
    }
  }
}

Cons* destructure_defdemon_body(Cons* demontree, Cons*& _return1) {
  { Cons* optionstree = demontree;
    Cons* bodytree = demontree;
    Cons* trailingcursor = NULL;

    while (keywordP(bodytree->value)) {
      trailingcursor = bodytree;
      bodytree = bodytree->rest->rest;
    }
    if (((boolean)(trailingcursor))) {
      trailingcursor->rest->rest = NIL;
    }
    else {
      optionstree = NIL;
    }
    _return1 = bodytree;
    return (optionstree);
  }
}

void in_place_normalize_demon_options(Cons* optionstree) {
  { Cons_Iterator* it = optionstree->allocate_iterator();

    while (it->nextP()) {
      if (wrapperP(it->value)) {
        it->value_setter(listO(3, SYM_DEMONS_STELLA_WRAP_LITERAL, it->value, NIL));
      }
      else if (it->value == SYM_DEMONS_STELLA_TRUE) {
        it->value_setter(listO(3, SYM_DEMONS_STELLA_WRAP_BOOLEAN, SYM_DEMONS_STELLA_TRUE, NIL));
      }
      else if (it->value == SYM_DEMONS_STELLA_FALSE) {
        it->value_setter(listO(3, SYM_DEMONS_STELLA_WRAP_BOOLEAN, SYM_DEMONS_STELLA_FALSE, NIL));
      }
    }
  }
}

Object* defdemon_expander(String_Wrapper* name, Cons* parameterstree, Cons* optionsandbody) {
  // Define a demon `name' and attach it to a class or slot.
  { Object* firstparameter = parameterstree->value;
    Surrogate* type = NULL;
    Symbol* demonfunctionname = intern_symbol(name->wrapper_value);

    if (!consP(firstparameter)) {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Illegal first parameter in demon " << "`" << de_uglify_parse_tree(name) << "'" << "." << endl;
        }
      }
      return (NIL);
    }
    type = type_spec_to_base_type(safe_yield_type_specifier(((Cons*)(firstparameter))->rest->value));
    if (!((boolean)(((Class*)(type->surrogate_value))))) {
      { 
        BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
        signal_translation_error();
        if (!(suppress_warningsP())) {
          print_error_context(">> ERROR: ", STANDARD_ERROR);
          *(STANDARD_ERROR->native_stream) << endl << " " << "Demon defined on the non-existent class " << "`" << de_uglify_parse_tree(type) << "'" << "." << endl;
        }
      }
      return (NIL);
    }
    { Cons* optionstree = NULL;
      Cons* bodytree = NULL;

      optionstree = destructure_defdemon_body(optionsandbody, bodytree);
      in_place_normalize_demon_options(optionstree);
      return (listO(4, SYM_DEMONS_STELLA_PROGN, listO(3, SYM_DEMONS_STELLA_DEFUN, demonfunctionname, cons(parameterstree, bodytree->concatenate(NIL, 0))), listO(3, SYM_DEMONS_STELLA_STARTUP_TIME_PROGN, listO(3, SYM_DEMONS_STELLA_DEFINE_DEMON, name, optionstree->concatenate(listO(3, KWD_DEMONS_CODE, listO(3, SYM_DEMONS_STELLA_WRAP_LITERAL, listO(4, SYM_DEMONS_STELLA_THE_CODE, KWD_DEMONS_FUNCTION, demonfunctionname, NIL), NIL), NIL), 0)), NIL), NIL));
    }
  }
}

Object* read_slot_value(Standard_Object* self, Storage_Slot* slot) {
  { cpp_function_code code = ((Class*)(slot->slot_owner->surrogate_value))->class_slot_accessor_code;

    if (code != NULL) {
      return (((Object*  (*) (Standard_Object*, Symbol*, Object*, boolean))code)(self, slot->slot_name, ((Object*)(NULL)), FALSE));
    }
    if (subtype_ofP(self->primary_type(), slot->slot_owner)) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "read-slot-value: No generic slot reader available for " << "`" << slot << "'" << "." << endl;
    }
    else {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "read-slot-value: Slot " << "`" << slot << "'" << " does not exist on" << endl << " object " << "`" << self << "'" << " of type " << "`" << self->primary_type() << "'" << "." << endl;
    }
    return (NULL);
  }
}

Object* write_slot_value(Standard_Object* self, Storage_Slot* slot, Object* value) {
  { cpp_function_code code = ((Class*)(slot->slot_owner->surrogate_value))->class_slot_accessor_code;

    if (code != NULL) {
      return (((Object*  (*) (Standard_Object*, Symbol*, Object*, boolean))code)(self, slot->slot_name, value, TRUE));
    }
    if (subtype_ofP(self->primary_type(), slot->slot_owner)) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "write-slot-value: No generic slot writer available for " << "`" << slot << "'" << "." << endl;
    }
    else {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "write-slot-value: Slot " << "`" << slot << "'" << " does not exist on" << endl << " object " << "`" << self << "'" << " of type " << "`" << self->primary_type() << "'" << "." << endl;
    }
    return (NULL);
  }
}

Object* put_slot_value(Standard_Object* self, Storage_Slot* slot, Object* value) {
  if (slot->collection_valuedP()) {
    { List* oldvalues = ((List*)(read_slot_value(self, slot)));

      oldvalues->insert(value);
      return (value);
    }
  }
  else {
    return (write_slot_value(self, slot, value));
  }
}

void drop_slot_value(Standard_Object* self, Storage_Slot* slot, Object* value) {
  if (slot->collection_valuedP()) {
    { List* oldvalues = ((List*)(read_slot_value(self, slot)));

      oldvalues->remove(value);
    }
  }
  else {
    { Object* oldvalue = read_slot_value(self, slot);

      if (eqlP(value, oldvalue)) {
        write_slot_value(self, slot, NULL);
      }
    }
  }
}

boolean check_for_slot_demonsP(Storage_Slot* slot) {
  return (((boolean)(oALL_SLOTS_DEMONSo)) ||
      (((boolean)(oALL_SLOTS_GUARD_DEMONSo)) ||
       (((boolean)(slot->slot_guard_demons_reader())) ||
        ((boolean)(slot->slot_demons_reader())))));
}

boolean run_slot_guard_demonsP(Standard_Object* self, Storage_Slot* slot, Object* oldvalue, Object* newvalue) {
  if (((boolean)(oALL_SLOTS_GUARD_DEMONSo))) {
    { Demon* demon = NULL;
      Cons* iter_000 = oALL_SLOTS_GUARD_DEMONSo->the_cons_list;

      while (!(iter_000 == NIL)) {
        demon = ((Demon*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((boolean  (*) (Standard_Object*, Storage_Slot*, Object*, Object*))demon->demon_code)(self, slot, oldvalue, newvalue)) {
          return (FALSE);
        }
      }
    }
  }
  if (((boolean)(slot->slot_guard_demons_reader()))) {
    { Demon* demon = NULL;
      Cons* iter_001 = slot->slot_guard_demons_reader()->the_cons_list;

      while (!(iter_001 == NIL)) {
        demon = ((Demon*)(iter_001->value));
        iter_001 = iter_001->rest;
        if (!((boolean  (*) (Standard_Object*, Storage_Slot*, Object*, Object*))demon->demon_code)(self, slot, oldvalue, newvalue)) {
          return (FALSE);
        }
      }
    }
  }
  return (TRUE);
}

void run_slot_demons(Standard_Object* self, Storage_Slot* slot, Object* oldvalue, Object* newvalue) {
  if (((boolean)(oALL_SLOTS_DEMONSo))) {
    { Demon* demon = NULL;
      Cons* iter_000 = oALL_SLOTS_DEMONSo->the_cons_list;

      while (!(iter_000 == NIL)) {
        demon = ((Demon*)(iter_000->value));
        iter_000 = iter_000->rest;
        ((void  (*) (Standard_Object*, Storage_Slot*, Object*, Object*))demon->demon_code)(self, slot, oldvalue, newvalue);
      }
    }
  }
  if (((boolean)(slot->slot_demons_reader()))) {
    { Demon* demon = NULL;
      Cons* iter_001 = slot->slot_demons_reader()->the_cons_list;

      while (!(iter_001 == NIL)) {
        demon = ((Demon*)(iter_001->value));
        iter_001 = iter_001->rest;
        ((void  (*) (Standard_Object*, Storage_Slot*, Object*, Object*))demon->demon_code)(self, slot, oldvalue, newvalue);
      }
    }
  }
}

void run_constructor_demons(Standard_Object* self) {
  { Class* renamed_Class = self->primary_class();

    if (((boolean)(renamed_Class))) {
      if (((boolean)(oCONSTRUCTOR_DEMONSo))) {
        { Demon* demon = NULL;
          Cons* iter_000 = oCONSTRUCTOR_DEMONSo->the_cons_list;

          while (!(iter_000 == NIL)) {
            demon = ((Demon*)(iter_000->value));
            iter_000 = iter_000->rest;
            ((void  (*) (Standard_Object*, Class*))demon->demon_code)(self, renamed_Class);
          }
        }
      }
      if (((boolean)(renamed_Class->class_constructor_demons_reader()))) {
        { Demon* demon = NULL;
          Cons* iter_001 = renamed_Class->class_constructor_demons_reader()->the_cons_list;

          while (!(iter_001 == NIL)) {
            demon = ((Demon*)(iter_001->value));
            iter_001 = iter_001->rest;
            ((void  (*) (Standard_Object*, Class*))demon->demon_code)(self, renamed_Class);
          }
        }
      }
    }
  }
}

void run_destructor_demons(Standard_Object* self) {
  { Class* renamed_Class = self->primary_class();

    if (((boolean)(renamed_Class))) {
      if (((boolean)(oDESTRUCTOR_DEMONSo))) {
        { Demon* demon = NULL;
          Cons* iter_000 = oDESTRUCTOR_DEMONSo->the_cons_list;

          while (!(iter_000 == NIL)) {
            demon = ((Demon*)(iter_000->value));
            iter_000 = iter_000->rest;
            ((void  (*) (Standard_Object*, Class*))demon->demon_code)(self, renamed_Class);
          }
        }
      }
      if (((boolean)(renamed_Class->class_destructor_demons_reader()))) {
        { Demon* demon = NULL;
          Cons* iter_001 = renamed_Class->class_destructor_demons_reader()->the_cons_list;

          while (!(iter_001 == NIL)) {
            demon = ((Demon*)(iter_001->value));
            iter_001 = iter_001->rest;
            ((void  (*) (Standard_Object*, Class*))demon->demon_code)(self, renamed_Class);
          }
        }
      }
    }
  }
}

Cons* yield_active_slot_reader_body(Storage_Slot* slot, Object* defaultexpression) {
  { Symbol* slotname = slot->slot_name;

    if (!slot->collection_valuedP()) {
      return (wrap_with_default_value_and_return_code(slot, defaultexpression));
    }
    else {
      {
        if (((boolean)(defaultexpression))) {
          { 
            BIND_STELLA_SPECIAL(oPRINTREADABLYpo, boolean, TRUE);
            signal_translation_error();
            if (!(suppress_warningsP())) {
              print_error_context(">> ERROR: ", STANDARD_ERROR);
              *(STANDARD_ERROR->native_stream) << endl << " " << "Can't define a default value for the active " << "collection-valued slot named." << "`" << slot->slot_name << "'" << "Ignoring the default value." << "." << endl;
            }
          }
        }
        return (listO(9, SYM_DEMONS_STELLA_LET, cons(listO(3, SYM_DEMONS_STELLA_VALUE, listO(4, SYM_DEMONS_STELLA_SLOT_VALUE, SYM_DEMONS_STELLA_SELF, slotname, NIL), NIL), NIL), listO(4, SYM_DEMONS_STELLA_WHEN, listO(3, SYM_DEMONS_STELLA_DEFINEDp, SYM_DEMONS_STELLA_VALUE, NIL), listO(3, SYM_DEMONS_STELLA_RETURN, SYM_DEMONS_STELLA_VALUE, NIL), NIL), listO(4, SYM_DEMONS_STELLA_SETQ, SYM_DEMONS_STELLA_VALUE, listO(3, SYM_DEMONS_STELLA_NEW, type_to_symbol(slot->type()), NIL), NIL), listO(4, SYM_DEMONS_STELLA_SETF, listO(3, SYM_DEMONS_STELLA_OWNER_INSTANCE, SYM_DEMONS_STELLA_VALUE, NIL), SYM_DEMONS_STELLA_SELF, NIL), listO(4, SYM_DEMONS_STELLA_SETF, listO(3, SYM_DEMONS_STELLA_ACTIVE_SLOT, SYM_DEMONS_STELLA_VALUE, NIL), listO(4, SYM_DEMONS_STELLA_GET_SLOT, SYM_DEMONS_STELLA_SELF, listO(3, SYM_DEMONS_STELLA_BQUOTE, slot->slot_name, NIL), NIL), NIL), listO(4, SYM_DEMONS_STELLA_SETF, listO(4, SYM_DEMONS_STELLA_SLOT_VALUE, SYM_DEMONS_STELLA_SELF, slotname, NIL), SYM_DEMONS_STELLA_VALUE, NIL), listO(3, SYM_DEMONS_STELLA_RETURN, SYM_DEMONS_STELLA_VALUE, NIL), NIL));
      }
    }
  }
}

void cant_overwrite_active_collection_slot(Standard_Object* self, Symbol* slotname) {
  *(STANDARD_WARNING->native_stream) << "Warning: " << "Can't replace the value of the active slot " << "`" << slotname << "'" << " on the object " << endl << "`" << self << "'" << ".  Call `insert' or `remove' to add/remove slot values." << endl;
}

Cons* yield_active_slot_writer_body(Storage_Slot* slot) {
  { Symbol* slotname = slot->slot_name;
    Symbol* wrappingfunction = ((slot->slot_base_type == SGT_DEMONS_STELLA_BOOLEAN) ? SYM_DEMONS_STELLA_WRAP_BOOLEAN : SYM_DEMONS_STELLA_WRAP_LITERAL);

    if (slot->collection_valuedP()) {
      return (listO(5, SYM_DEMONS_STELLA_PROGN, SYM_DEMONS_STELLA_VALUE, listO(4, SYM_DEMONS_STELLA_CANT_OVERWRITE_ACTIVE_COLLECTION_SLOT, SYM_DEMONS_STELLA_SELF, listO(3, SYM_DEMONS_STELLA_BQUOTE, slotname, NIL), NIL), listO(3, SYM_DEMONS_STELLA_RETURN, listO(4, SYM_DEMONS_STELLA_SLOT_VALUE, SYM_DEMONS_STELLA_SELF, slot->slot_name, NIL), NIL), NIL));
    }
    else if (subtype_ofP(slot->type(), SGT_DEMONS_STELLA_LITERAL)) {
      return (listO(4, SYM_DEMONS_STELLA_LET, listO(3, listO(3, SYM_DEMONS_STELLA_SLOT, listO(4, SYM_DEMONS_STELLA_GET_SLOT, SYM_DEMONS_STELLA_SELF, listO(3, SYM_DEMONS_STELLA_BQUOTE, slotname, NIL), NIL), NIL), listO(3, SYM_DEMONS_STELLA_OLDVALUE, listO(4, SYM_DEMONS_STELLA_SLOT_VALUE, SYM_DEMONS_STELLA_SELF, slotname, NIL), NIL), NIL), listO(4, SYM_DEMONS_STELLA_WHEN, listO(3, SYM_DEMONS_STELLA_NOT, listO(4, SYM_DEMONS_STELLA_EQLp, SYM_DEMONS_STELLA_OLDVALUE, SYM_DEMONS_STELLA_VALUE, NIL), NIL), listO(5, SYM_DEMONS_STELLA_IF, listO(3, SYM_DEMONS_STELLA_CHECK_FOR_SLOT_DEMONSp, SYM_DEMONS_STELLA_SLOT, NIL), listO(5, SYM_DEMONS_STELLA_LET, listO(3, listO(3, SYM_DEMONS_STELLA_WRAPPEDOLDVALUE, listO(5, SYM_DEMONS_STELLA_CHOOSE, listO(3, SYM_DEMONS_STELLA_DEFINEDp, SYM_DEMONS_STELLA_OLDVALUE, NIL), cons(wrappingfunction, cons(SYM_DEMONS_STELLA_OLDVALUE, NIL)), SYM_DEMONS_STELLA_NULL, NIL), NIL), listO(3, SYM_DEMONS_STELLA_WRAPPEDNEWVALUE, listO(5, SYM_DEMONS_STELLA_CHOOSE, listO(3, SYM_DEMONS_STELLA_DEFINEDp, SYM_DEMONS_STELLA_VALUE, NIL), cons(wrappingfunction, cons(SYM_DEMONS_STELLA_VALUE, NIL)), SYM_DEMONS_STELLA_NULL, NIL), NIL), NIL), listO(8, SYM_DEMONS_STELLA_WHEN, listO(6, SYM_DEMONS_STELLA_RUN_SLOT_GUARD_DEMONSp, SYM_DEMONS_STELLA_SELF, SYM_DEMONS_STELLA_SLOT, SYM_DEMONS_STELLA_WRAPPEDOLDVALUE, SYM_DEMONS_STELLA_WRAPPEDNEWVALUE, NIL), listO(4, SYM_DEMONS_STELLA_SETF, listO(4, SYM_DEMONS_STELLA_SLOT_VALUE, SYM_DEMONS_STELLA_SELF, slotname, NIL), SYM_DEMONS_STELLA_VALUE, NIL), listO(6, SYM_DEMONS_STELLA_RUN_SLOT_DEMONS, SYM_DEMONS_STELLA_SELF, SYM_DEMONS_STELLA_SLOT, SYM_DEMONS_STELLA_WRAPPEDOLDVALUE, SYM_DEMONS_STELLA_WRAPPEDNEWVALUE, NIL), listO(4, SYM_DEMONS_STELLA_WHEN, listO(3, SYM_DEMONS_STELLA_DEFINEDp, SYM_DEMONS_STELLA_WRAPPEDOLDVALUE, NIL), listO(3, SYM_DEMONS_STELLA_FREE, SYM_DEMONS_STELLA_WRAPPEDOLDVALUE, NIL), NIL), listO(4, SYM_DEMONS_STELLA_WHEN, listO(3, SYM_DEMONS_STELLA_DEFINEDp, SYM_DEMONS_STELLA_WRAPPEDNEWVALUE, NIL), listO(3, SYM_DEMONS_STELLA_FREE, SYM_DEMONS_STELLA_WRAPPEDNEWVALUE, NIL), NIL), listO(3, SYM_DEMONS_STELLA_RETURN, SYM_DEMONS_STELLA_VALUE, NIL), NIL), listO(3, SYM_DEMONS_STELLA_RETURN, SYM_DEMONS_STELLA_OLDVALUE, NIL), NIL), listO(3, SYM_DEMONS_STELLA_RETURN, listO(4, SYM_DEMONS_STELLA_SETF, listO(4, SYM_DEMONS_STELLA_SLOT_VALUE, SYM_DEMONS_STELLA_SELF, slotname, NIL), SYM_DEMONS_STELLA_VALUE, NIL), NIL), NIL), NIL), NIL));
    }
    else {
      return (listO(5, SYM_DEMONS_STELLA_LET, listO(3, listO(3, SYM_DEMONS_STELLA_SLOT, listO(4, SYM_DEMONS_STELLA_GET_SLOT, SYM_DEMONS_STELLA_SELF, listO(3, SYM_DEMONS_STELLA_BQUOTE, slotname, NIL), NIL), NIL), listO(3, SYM_DEMONS_STELLA_OLDVALUE, listO(4, SYM_DEMONS_STELLA_SLOT_VALUE, SYM_DEMONS_STELLA_SELF, slotname, NIL), NIL), NIL), listO(6, SYM_DEMONS_STELLA_WHEN, listO(4, SYM_DEMONS_STELLA_AND, listO(3, SYM_DEMONS_STELLA_NOT, listO(4, SYM_DEMONS_STELLA_EQp, SYM_DEMONS_STELLA_OLDVALUE, SYM_DEMONS_STELLA_VALUE, NIL), NIL), listO(6, SYM_DEMONS_STELLA_RUN_SLOT_GUARD_DEMONSp, SYM_DEMONS_STELLA_SELF, SYM_DEMONS_STELLA_SLOT, SYM_DEMONS_STELLA_OLDVALUE, SYM_DEMONS_STELLA_VALUE, NIL), NIL), listO(4, SYM_DEMONS_STELLA_SETF, listO(4, SYM_DEMONS_STELLA_SLOT_VALUE, SYM_DEMONS_STELLA_SELF, slotname, NIL), SYM_DEMONS_STELLA_VALUE, NIL), listO(6, SYM_DEMONS_STELLA_RUN_SLOT_DEMONS, SYM_DEMONS_STELLA_SELF, SYM_DEMONS_STELLA_SLOT, SYM_DEMONS_STELLA_OLDVALUE, SYM_DEMONS_STELLA_VALUE, NIL), listO(3, SYM_DEMONS_STELLA_RETURN, SYM_DEMONS_STELLA_VALUE, NIL), NIL), listO(3, SYM_DEMONS_STELLA_RETURN, SYM_DEMONS_STELLA_OLDVALUE, NIL), NIL));
    }
  }
}

void Active_List::insert(Object* value) {
  { Active_List* self = this;

    { Storage_Slot* slot = self->active_slot;
      Standard_Object* owner = self->owner_instance;

      if (run_slot_guard_demonsP(owner, slot, self, value)) {
        self->the_cons_list = cons(value, self->the_cons_list);
        run_slot_demons(owner, slot, NULL, value);
      }
    }
  }
}

void Active_Set::insert(Object* value) {
  { Active_Set* self = this;

    { Storage_Slot* slot = self->active_slot;
      Standard_Object* owner = self->owner_instance;

      if (!(self->the_cons_list->memberP(value))) {
        if (run_slot_guard_demonsP(owner, slot, self, value)) {
          self->the_cons_list = cons(value, self->the_cons_list);
          run_slot_demons(owner, slot, NULL, value);
        }
      }
    }
  }
}

Abstract_Collection* Active_List::remove(Object* value) {
  { Active_List* self = this;

    { Storage_Slot* slot = self->active_slot;
      Standard_Object* owner = self->owner_instance;

      if (self->the_cons_list->memberP(value)) {
        if (run_slot_guard_demonsP(owner, slot, value, NULL)) {
          self->the_cons_list = self->the_cons_list->remove(value);
          run_slot_demons(owner, slot, value, NULL);
        }
      }
      return (self);
    }
  }
}

Abstract_Collection* Active_Set::remove(Object* value) {
  { Active_Set* self = this;

    { Storage_Slot* slot = self->active_slot;
      Standard_Object* owner = self->owner_instance;

      if (self->the_cons_list->memberP(value)) {
        if (run_slot_guard_demonsP(owner, slot, value, NULL)) {
          self->the_cons_list = self->the_cons_list->remove(value);
          run_slot_demons(owner, slot, value, NULL);
        }
      }
      return (self);
    }
  }
}

Hook_List* new_hook_list() {
  { Hook_List* self = NULL;

    self = new Hook_List();
    self->the_kv_list = NULL;
    self->documentation = NULL;
    self->signature = NULL;
    return (self);
  }
}

Surrogate* Hook_List::primary_type() {
  { Hook_List* self = this;

    return (SGT_DEMONS_STELLA_HOOK_LIST);
  }
}

Object* access_hook_list_slot_value(Hook_List* self, Symbol* slotname, Object* value, boolean setvalueP) {
  if (slotname == SYM_DEMONS_STELLA_SIGNATURE) {
    if (setvalueP) {
      self->signature = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->signature);
    }
  }
  else if (slotname == SYM_DEMONS_STELLA_DOCUMENTATION) {
    if (setvalueP) {
      self->documentation = ((String_Wrapper*)(value))->wrapper_value;
    }
    else {
      value = wrap_string(self->documentation);
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << slotname << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (value);
}

void add_hook(Hook_List* hooklist, Symbol* hookfunction) {
  // Insert the function named `hookFunction' into `hookList'.
  { Method_Slot* function = lookup_function(hookfunction);

    if (!((boolean)(function))) {
      *(STANDARD_WARNING->native_stream) << "Warning: " << "Reference to non-existent hook function: `hookFunction'." << endl;
      return;
    }
    hooklist->insert_at(hookfunction, wrap_function_code(function->function_code));
  }
}

void drop_hook(Hook_List* hooklist, Symbol* hookfunction) {
  // Remove the function named `hookFunction' from `hookList'.
  hooklist->remove_at(hookfunction);
}

void run_hooks(Hook_List* hooklist, Object* argument) {
  // Run all hook functions in `hooklist', applying
  // each one to `argument'.
  { Kv_List_Iterator* it = ((Kv_List_Iterator*)(hooklist->allocate_iterator()));

    while (it->nextP()) {
      ((void  (*) (Object*))((Function_Code_Wrapper*)(((Function_Code_Wrapper*)(it->value))))->wrapper_value)(argument);
    }
  }
}

// HOOK-LIST called by `in-module', applied to a
// `module' argument.
Hook_List* oCHANGE_MODULE_HOOKSo = NULL;

// HOOK-LIST called by `clear-module', applied to a
// `module' argument.
Hook_List* oCLEAR_MODULE_HOOKSo = NULL;

// HOOK-LIST called by `clear-module', applied to a
// `module' argument.
Hook_List* oDESTROY_CONTEXT_HOOKSo = NULL;

// HOOK-LIST is called by `undefine-old-class'.  Each
// hook function is passed a LIST of two RELATION objects; the first is the
// old relation object and the second is the new (redefined) relation object.
// The hook functions are expected to copy/transfer information from the
// old relation to the new relation so that it won't be lost.
Hook_List* oREDEFINE_RELATION_HOOKSo = NULL;

// HOOK-LIST is called by `finalize-one-class'
// and `finalize-local-slot', applied to a RELATION argument.
Hook_List* oFINALIZE_RELATION_HOOKSo = NULL;

// HOOK-LIST called by `create-native-class', applied to
// a STELLA class.  Return value of TRUE blocks creation.
Hook_List* oBLOCK_NATIVE_CLASS_CREATION_HOOKSo = NULL;

void help_startup_demons1() {
  {
    KWD_DEMONS_MODIFY = ((Keyword*)(intern_rigid_symbol_wrt_module("MODIFY", NULL, 2)));
    KWD_DEMONS_CREATE = ((Keyword*)(intern_rigid_symbol_wrt_module("CREATE", NULL, 2)));
    KWD_DEMONS_DESTROY = ((Keyword*)(intern_rigid_symbol_wrt_module("DESTROY", NULL, 2)));
    KWD_DEMONS_ALL = ((Keyword*)(intern_rigid_symbol_wrt_module("ALL", NULL, 2)));
    KWD_DEMONS_CLASS = ((Keyword*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 2)));
    KWD_DEMONS_SLOT = ((Keyword*)(intern_rigid_symbol_wrt_module("SLOT", NULL, 2)));
    KWD_DEMONS_CODE = ((Keyword*)(intern_rigid_symbol_wrt_module("CODE", NULL, 2)));
    KWD_DEMONS_METHOD = ((Keyword*)(intern_rigid_symbol_wrt_module("METHOD", NULL, 2)));
    KWD_DEMONS_GUARDp = ((Keyword*)(intern_rigid_symbol_wrt_module("GUARD?", NULL, 2)));
    KWD_DEMONS_INHERITEDp = ((Keyword*)(intern_rigid_symbol_wrt_module("INHERITED?", NULL, 2)));
    KWD_DEMONS_DOCUMENTATION = ((Keyword*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 2)));
    KWD_DEMONS_ACTIVATEp = ((Keyword*)(intern_rigid_symbol_wrt_module("ACTIVATE?", NULL, 2)));
    SYM_DEMONS_STELLA_SLOT_GUARD_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-GUARD-DEMONS", NULL, 0)));
    SYM_DEMONS_STELLA_SLOT_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-DEMONS", NULL, 0)));
    SYM_DEMONS_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-GUARD-CONSTRUCTOR-DEMONS", NULL, 0)));
    SYM_DEMONS_STELLA_CLASS_CONSTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-CONSTRUCTOR-DEMONS", NULL, 0)));
    SYM_DEMONS_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-GUARD-DESTRUCTOR-DEMONS", NULL, 0)));
    SYM_DEMONS_STELLA_CLASS_DESTRUCTOR_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("CLASS-DESTRUCTOR-DEMONS", NULL, 0)));
    SYM_DEMONS_STELLA_WRAP_LITERAL = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-LITERAL", NULL, 0)));
    SYM_DEMONS_STELLA_TRUE = ((Symbol*)(intern_rigid_symbol_wrt_module("TRUE", NULL, 0)));
    SYM_DEMONS_STELLA_WRAP_BOOLEAN = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAP-BOOLEAN", NULL, 0)));
    SYM_DEMONS_STELLA_FALSE = ((Symbol*)(intern_rigid_symbol_wrt_module("FALSE", NULL, 0)));
    SYM_DEMONS_STELLA_PROGN = ((Symbol*)(intern_rigid_symbol_wrt_module("PROGN", NULL, 0)));
    SYM_DEMONS_STELLA_DEFUN = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFUN", NULL, 0)));
    SYM_DEMONS_STELLA_STARTUP_TIME_PROGN = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-TIME-PROGN", NULL, 0)));
    SYM_DEMONS_STELLA_DEFINE_DEMON = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINE-DEMON", NULL, 0)));
    SYM_DEMONS_STELLA_THE_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("THE-CODE", NULL, 0)));
    KWD_DEMONS_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("FUNCTION", NULL, 2)));
    SYM_DEMONS_STELLA_DEFDEMON = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFDEMON", NULL, 0)));
    SYM_DEMONS_STELLA_DEFDEMON_EXPANDER = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFDEMON-EXPANDER", NULL, 0)));
    SYM_DEMONS_STELLA_LET = ((Symbol*)(intern_rigid_symbol_wrt_module("LET", NULL, 0)));
    SYM_DEMONS_STELLA_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("VALUE", NULL, 0)));
    SYM_DEMONS_STELLA_SLOT_VALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT-VALUE", NULL, 0)));
    SYM_DEMONS_STELLA_SELF = ((Symbol*)(intern_rigid_symbol_wrt_module("SELF", NULL, 0)));
    SYM_DEMONS_STELLA_WHEN = ((Symbol*)(intern_rigid_symbol_wrt_module("WHEN", NULL, 0)));
    SYM_DEMONS_STELLA_DEFINEDp = ((Symbol*)(intern_rigid_symbol_wrt_module("DEFINED?", NULL, 0)));
    SYM_DEMONS_STELLA_RETURN = ((Symbol*)(intern_rigid_symbol_wrt_module("RETURN", NULL, 0)));
    SYM_DEMONS_STELLA_SETQ = ((Symbol*)(intern_rigid_symbol_wrt_module("SETQ", NULL, 0)));
    SYM_DEMONS_STELLA_NEW = ((Symbol*)(intern_rigid_symbol_wrt_module("NEW", NULL, 0)));
    SYM_DEMONS_STELLA_SETF = ((Symbol*)(intern_rigid_symbol_wrt_module("SETF", NULL, 0)));
    SYM_DEMONS_STELLA_OWNER_INSTANCE = ((Symbol*)(intern_rigid_symbol_wrt_module("OWNER-INSTANCE", NULL, 0)));
    SYM_DEMONS_STELLA_ACTIVE_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("ACTIVE-SLOT", NULL, 0)));
    SYM_DEMONS_STELLA_GET_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("GET-SLOT", NULL, 0)));
    SYM_DEMONS_STELLA_BQUOTE = ((Symbol*)(intern_rigid_symbol_wrt_module("BQUOTE", NULL, 0)));
    SGT_DEMONS_STELLA_BOOLEAN = ((Surrogate*)(intern_rigid_symbol_wrt_module("BOOLEAN", NULL, 1)));
    SYM_DEMONS_STELLA_CANT_OVERWRITE_ACTIVE_COLLECTION_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("CANT-OVERWRITE-ACTIVE-COLLECTION-SLOT", NULL, 0)));
    SGT_DEMONS_STELLA_LITERAL = ((Surrogate*)(intern_rigid_symbol_wrt_module("LITERAL", NULL, 1)));
    SYM_DEMONS_STELLA_SLOT = ((Symbol*)(intern_rigid_symbol_wrt_module("SLOT", NULL, 0)));
    SYM_DEMONS_STELLA_OLDVALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("OLDVALUE", NULL, 0)));
    SYM_DEMONS_STELLA_NOT = ((Symbol*)(intern_rigid_symbol_wrt_module("NOT", NULL, 0)));
    SYM_DEMONS_STELLA_EQLp = ((Symbol*)(intern_rigid_symbol_wrt_module("EQL?", NULL, 0)));
    SYM_DEMONS_STELLA_IF = ((Symbol*)(intern_rigid_symbol_wrt_module("IF", NULL, 0)));
    SYM_DEMONS_STELLA_CHECK_FOR_SLOT_DEMONSp = ((Symbol*)(intern_rigid_symbol_wrt_module("CHECK-FOR-SLOT-DEMONS?", NULL, 0)));
    SYM_DEMONS_STELLA_WRAPPEDOLDVALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAPPEDOLDVALUE", NULL, 0)));
    SYM_DEMONS_STELLA_CHOOSE = ((Symbol*)(intern_rigid_symbol_wrt_module("CHOOSE", NULL, 0)));
    SYM_DEMONS_STELLA_NULL = ((Symbol*)(intern_rigid_symbol_wrt_module("NULL", NULL, 0)));
    SYM_DEMONS_STELLA_WRAPPEDNEWVALUE = ((Symbol*)(intern_rigid_symbol_wrt_module("WRAPPEDNEWVALUE", NULL, 0)));
    SYM_DEMONS_STELLA_RUN_SLOT_GUARD_DEMONSp = ((Symbol*)(intern_rigid_symbol_wrt_module("RUN-SLOT-GUARD-DEMONS?", NULL, 0)));
    SYM_DEMONS_STELLA_RUN_SLOT_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("RUN-SLOT-DEMONS", NULL, 0)));
    SYM_DEMONS_STELLA_FREE = ((Symbol*)(intern_rigid_symbol_wrt_module("FREE", NULL, 0)));
  }
}

void help_startup_demons2() {
  {
    SYM_DEMONS_STELLA_AND = ((Symbol*)(intern_rigid_symbol_wrt_module("AND", NULL, 0)));
    SYM_DEMONS_STELLA_EQp = ((Symbol*)(intern_rigid_symbol_wrt_module("EQ?", NULL, 0)));
    SGT_DEMONS_STELLA_HOOK_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("HOOK-LIST", NULL, 1)));
    SYM_DEMONS_STELLA_SIGNATURE = ((Symbol*)(intern_rigid_symbol_wrt_module("SIGNATURE", NULL, 0)));
    SYM_DEMONS_STELLA_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 0)));
    SYM_DEMONS_STELLA_STARTUP_DEMONS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-DEMONS", NULL, 0)));
    SYM_DEMONS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void startup_demons() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_demons1();
      help_startup_demons2();
    }
    if (current_startup_time_phaseP(4)) {
      oDEMONS_TABLEo = new_string_hash_table();
      oCHANGE_MODULE_HOOKSo = new_hook_list();
      oCLEAR_MODULE_HOOKSo = new_hook_list();
      oDESTROY_CONTEXT_HOOKSo = new_hook_list();
      oREDEFINE_RELATION_HOOKSo = new_hook_list();
      oFINALIZE_RELATION_HOOKSo = new_hook_list();
      oBLOCK_NATIVE_CLASS_CREATION_HOOKSo = new_hook_list();
    }
    if (current_startup_time_phaseP(5)) {
      { Class* renamed_Class = define_class_from_stringified_source("HOOK-LIST", "(DEFCLASS HOOK-LIST (KEY-VALUE-LIST) :DOCUMENTATION \"Each HOOK-LIST object contains a list of zero or\nmore function codes that get funcall'd some procedure.\" :PARAMETERS ((ANY-KEY :TYPE SYMBOL) (ANY-VALUE :TYPE FUNCTION-CODE-WRAPPER)) :SLOTS ((SIGNATURE :TYPE STRING) (DOCUMENTATION :TYPE STRING)))");

        renamed_Class->class_constructor_code = ((cpp_function_code)(&new_hook_list));
        renamed_Class->class_slot_accessor_code = ((cpp_function_code)(&access_hook_list_slot_value));
      }
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("VALIDATE-DEMON-REFERENCES?", "(DEFUN (VALIDATE-DEMON-REFERENCES? BOOLEAN) ((DEMON DEMON)))", ((cpp_function_code)(&validate_demon_referencesP)), NULL);
      define_function_object("INCORPORATE-DEMON-OPTIONS?", "(DEFUN (INCORPORATE-DEMON-OPTIONS? BOOLEAN) ((DEMON DEMON) (OPTIONS PROPERTY-LIST)))", ((cpp_function_code)(&incorporate_demon_optionsP)), NULL);
      define_function_object("LOOKUP-DEMON", "(DEFUN (LOOKUP-DEMON DEMON) ((NAME STRING)) :DOCUMENTATION \"Return the demon named `name'.\")", ((cpp_function_code)(&lookup_demon)), NULL);
      define_function_object("DEFINE-DEMON", "(DEFUN (DEFINE-DEMON DEMON) ((NAME STRING) |&REST| (OPTIONS OBJECT)) :DOCUMENTATION \"Define a class or slot demon.  Options are :create,\n:destroy, :class, :slot, :guard?, :code, :method, :inherit?, and :documentation.\")", ((cpp_function_code)(&define_demon)), NULL);
      define_function_object("INJECT-DEMON", "(DEFUN (INJECT-DEMON (LIST OF DEMON)) ((DEMONLIST (LIST OF DEMON)) (DEMON DEMON)))", ((cpp_function_code)(&inject_demon)), NULL);
      define_function_object("EJECT-DEMON", "(DEFUN (EJECT-DEMON (LIST OF DEMON)) ((DEMONLIST (LIST OF DEMON)) (DEMON DEMON)))", ((cpp_function_code)(&eject_demon)), NULL);
      define_function_object("ACTIVATE-DEMON", "(DEFUN ACTIVATE-DEMON ((DEMON DEMON)) :DOCUMENTATION \"Install `demon' in the location(s) specified by its internal\nstructure.\")", ((cpp_function_code)(&activate_demon)), NULL);
      define_function_object("DEACTIVATE-DEMON", "(DEFUN DEACTIVATE-DEMON ((DEMON DEMON)) :DOCUMENTATION \"Detach `demon' from the location(s) specified by its internal\nstructure.\")", ((cpp_function_code)(&deactivate_demon)), NULL);
      define_function_object("DESTRUCTURE-DEFDEMON-BODY", "(DEFUN (DESTRUCTURE-DEFDEMON-BODY CONS CONS) ((DEMONTREE CONS)))", ((cpp_function_code)(&destructure_defdemon_body)), NULL);
      define_function_object("IN-PLACE-NORMALIZE-DEMON-OPTIONS", "(DEFUN IN-PLACE-NORMALIZE-DEMON-OPTIONS ((OPTIONSTREE CONS)))", ((cpp_function_code)(&in_place_normalize_demon_options)), NULL);
      define_function_object("DEFDEMON-EXPANDER", "(DEFUN (DEFDEMON-EXPANDER OBJECT) ((NAME STRING-WRAPPER) (PARAMETERSTREE CONS) |&BODY| (OPTIONSANDBODY CONS)) :DOCUMENTATION \"Define a demon `name' and attach it to a class or slot.\")", ((cpp_function_code)(&defdemon_expander)), NULL);
      define_function_object("READ-SLOT-VALUE", "(DEFUN (READ-SLOT-VALUE OBJECT) ((SELF STANDARD-OBJECT) (SLOT STORAGE-SLOT)) :PUBLIC? TRUE)", ((cpp_function_code)(&read_slot_value)), NULL);
      define_function_object("WRITE-SLOT-VALUE", "(DEFUN (WRITE-SLOT-VALUE OBJECT) ((SELF STANDARD-OBJECT) (SLOT STORAGE-SLOT) (VALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_function_code)(&write_slot_value)), NULL);
      define_function_object("PUT-SLOT-VALUE", "(DEFUN (PUT-SLOT-VALUE OBJECT) ((SELF STANDARD-OBJECT) (SLOT STORAGE-SLOT) (VALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_function_code)(&put_slot_value)), NULL);
      define_function_object("DROP-SLOT-VALUE", "(DEFUN DROP-SLOT-VALUE ((SELF STANDARD-OBJECT) (SLOT STORAGE-SLOT) (VALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_function_code)(&drop_slot_value)), NULL);
      define_function_object("CHECK-FOR-SLOT-DEMONS?", "(DEFUN (CHECK-FOR-SLOT-DEMONS? BOOLEAN) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&check_for_slot_demonsP)), NULL);
      define_function_object("RUN-SLOT-GUARD-DEMONS?", "(DEFUN (RUN-SLOT-GUARD-DEMONS? BOOLEAN) ((SELF STANDARD-OBJECT) (SLOT STORAGE-SLOT) (OLDVALUE OBJECT) (NEWVALUE OBJECT)))", ((cpp_function_code)(&run_slot_guard_demonsP)), NULL);
      define_function_object("RUN-SLOT-DEMONS", "(DEFUN RUN-SLOT-DEMONS ((SELF STANDARD-OBJECT) (SLOT STORAGE-SLOT) (OLDVALUE OBJECT) (NEWVALUE OBJECT)))", ((cpp_function_code)(&run_slot_demons)), NULL);
      define_function_object("RUN-CONSTRUCTOR-DEMONS", "(DEFUN RUN-CONSTRUCTOR-DEMONS ((SELF STANDARD-OBJECT)))", ((cpp_function_code)(&run_constructor_demons)), NULL);
      define_function_object("RUN-DESTRUCTOR-DEMONS", "(DEFUN RUN-DESTRUCTOR-DEMONS ((SELF STANDARD-OBJECT)))", ((cpp_function_code)(&run_destructor_demons)), NULL);
      define_function_object("YIELD-ACTIVE-SLOT-READER-BODY", "(DEFUN (YIELD-ACTIVE-SLOT-READER-BODY CONS) ((SLOT STORAGE-SLOT) (DEFAULTEXPRESSION OBJECT)))", ((cpp_function_code)(&yield_active_slot_reader_body)), NULL);
      define_function_object("CANT-OVERWRITE-ACTIVE-COLLECTION-SLOT", "(DEFUN CANT-OVERWRITE-ACTIVE-COLLECTION-SLOT ((SELF STANDARD-OBJECT) (SLOTNAME SYMBOL)))", ((cpp_function_code)(&cant_overwrite_active_collection_slot)), NULL);
      define_function_object("YIELD-ACTIVE-SLOT-WRITER-BODY", "(DEFUN (YIELD-ACTIVE-SLOT-WRITER-BODY CONS) ((SLOT STORAGE-SLOT)))", ((cpp_function_code)(&yield_active_slot_writer_body)), NULL);
      define_method_object("(DEFMETHOD INSERT ((SELF ACTIVE-LIST) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&Active_List::insert)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD INSERT ((SELF ACTIVE-SET) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&Active_Set::insert)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (REMOVE ACTIVE-LIST) ((SELF ACTIVE-LIST) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&Active_List::remove)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (REMOVE ACTIVE-SET) ((SELF ACTIVE-SET) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&Active_Set::remove)), ((cpp_method_code)(NULL)));
      define_function_object("ADD-HOOK", "(DEFUN ADD-HOOK ((HOOKLIST HOOK-LIST) (HOOKFUNCTION SYMBOL)) :PUBLIC? TRUE :DOCUMENTATION \"Insert the function named `hookFunction' into `hookList'.\")", ((cpp_function_code)(&add_hook)), NULL);
      define_function_object("DROP-HOOK", "(DEFUN DROP-HOOK ((HOOKLIST HOOK-LIST) (HOOKFUNCTION SYMBOL)) :PUBLIC? TRUE :DOCUMENTATION \"Remove the function named `hookFunction' from `hookList'.\")", ((cpp_function_code)(&drop_hook)), NULL);
      define_function_object("RUN-HOOKS", "(DEFUN RUN-HOOKS ((HOOKLIST HOOK-LIST) (ARGUMENT OBJECT)) :DOCUMENTATION \"Run all hook functions in `hooklist', applying\neach one to `argument'.\")", ((cpp_function_code)(&run_hooks)), NULL);
      define_function_object("STARTUP-DEMONS", "(DEFUN STARTUP-DEMONS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_demons)), NULL);
      { Method_Slot* function = lookup_function(SYM_DEMONS_STELLA_STARTUP_DEMONS);

        set_dynamic_slot_value(function->dynamic_slots, SYM_DEMONS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Demons"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *DEMONS-TABLE* STRING-HASH-TABLE (NEW STRING-HASH-TABLE) :DOCUMENTATION \"Table containing all active and inactive demons, indexed\non their names.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CONSTRUCTOR-DEMONS* (LIST OF DEMON) NULL :DOCUMENTATION \"These demons trigger every time an active instance is created.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *DESTRUCTOR-DEMONS* (LIST OF DEMON) NULL :DOCUMENTATION \"These demons trigger every time an active instance is destroyed.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *ALL-SLOTS-DEMONS* (LIST OF DEMON) NULL :DOCUMENTATION \"These demons trigger every time a slot value is modified.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *ALL-SLOTS-GUARD-DEMONS* (LIST OF DEMON) NULL :DOCUMENTATION \"These demons trigger every time a slot value is modified, and\ncancel slot update when they return `false'.\")");
      store_macro(SYM_DEMONS_STELLA_DEFDEMON, SYM_DEMONS_STELLA_DEFDEMON_EXPANDER, ((cpp_function_code)(&defdemon_expander)));
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CHANGE-MODULE-HOOKS* HOOK-LIST (NEW HOOK-LIST) :DOCUMENTATION \"HOOK-LIST called by `in-module', applied to a\n`module' argument.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *CLEAR-MODULE-HOOKS* HOOK-LIST (NEW HOOK-LIST) :DOCUMENTATION \"HOOK-LIST called by `clear-module', applied to a\n`module' argument.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *DESTROY-CONTEXT-HOOKS* HOOK-LIST (NEW HOOK-LIST) :DOCUMENTATION \"HOOK-LIST called by `clear-module', applied to a\n`module' argument.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *REDEFINE-RELATION-HOOKS* HOOK-LIST (NEW HOOK-LIST) :DOCUMENTATION \"HOOK-LIST is called by `undefine-old-class'.  Each\nhook function is passed a LIST of two RELATION objects; the first is the\nold relation object and the second is the new (redefined) relation object.\nThe hook functions are expected to copy/transfer information from the\nold relation to the new relation so that it won't be lost.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *FINALIZE-RELATION-HOOKS* HOOK-LIST (NEW HOOK-LIST) :DOCUMENTATION \"HOOK-LIST is called by `finalize-one-class'\nand `finalize-local-slot', applied to a RELATION argument.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *BLOCK-NATIVE-CLASS-CREATION-HOOKS* HOOK-LIST (NEW HOOK-LIST) :DOCUMENTATION \"HOOK-LIST called by `create-native-class', applied to\na STELLA class.  Return value of TRUE blocks creation.\")");
    }
  }
}

Keyword* KWD_DEMONS_MODIFY = NULL;

Keyword* KWD_DEMONS_CREATE = NULL;

Keyword* KWD_DEMONS_DESTROY = NULL;

Keyword* KWD_DEMONS_ALL = NULL;

Keyword* KWD_DEMONS_CLASS = NULL;

Keyword* KWD_DEMONS_SLOT = NULL;

Keyword* KWD_DEMONS_CODE = NULL;

Keyword* KWD_DEMONS_METHOD = NULL;

Keyword* KWD_DEMONS_GUARDp = NULL;

Keyword* KWD_DEMONS_INHERITEDp = NULL;

Keyword* KWD_DEMONS_DOCUMENTATION = NULL;

Keyword* KWD_DEMONS_ACTIVATEp = NULL;

Symbol* SYM_DEMONS_STELLA_SLOT_GUARD_DEMONS = NULL;

Symbol* SYM_DEMONS_STELLA_SLOT_DEMONS = NULL;

Symbol* SYM_DEMONS_STELLA_CLASS_GUARD_CONSTRUCTOR_DEMONS = NULL;

Symbol* SYM_DEMONS_STELLA_CLASS_CONSTRUCTOR_DEMONS = NULL;

Symbol* SYM_DEMONS_STELLA_CLASS_GUARD_DESTRUCTOR_DEMONS = NULL;

Symbol* SYM_DEMONS_STELLA_CLASS_DESTRUCTOR_DEMONS = NULL;

Symbol* SYM_DEMONS_STELLA_WRAP_LITERAL = NULL;

Symbol* SYM_DEMONS_STELLA_TRUE = NULL;

Symbol* SYM_DEMONS_STELLA_WRAP_BOOLEAN = NULL;

Symbol* SYM_DEMONS_STELLA_FALSE = NULL;

Symbol* SYM_DEMONS_STELLA_PROGN = NULL;

Symbol* SYM_DEMONS_STELLA_DEFUN = NULL;

Symbol* SYM_DEMONS_STELLA_STARTUP_TIME_PROGN = NULL;

Symbol* SYM_DEMONS_STELLA_DEFINE_DEMON = NULL;

Symbol* SYM_DEMONS_STELLA_THE_CODE = NULL;

Keyword* KWD_DEMONS_FUNCTION = NULL;

Symbol* SYM_DEMONS_STELLA_DEFDEMON = NULL;

Symbol* SYM_DEMONS_STELLA_DEFDEMON_EXPANDER = NULL;

Symbol* SYM_DEMONS_STELLA_LET = NULL;

Symbol* SYM_DEMONS_STELLA_VALUE = NULL;

Symbol* SYM_DEMONS_STELLA_SLOT_VALUE = NULL;

Symbol* SYM_DEMONS_STELLA_SELF = NULL;

Symbol* SYM_DEMONS_STELLA_WHEN = NULL;

Symbol* SYM_DEMONS_STELLA_DEFINEDp = NULL;

Symbol* SYM_DEMONS_STELLA_RETURN = NULL;

Symbol* SYM_DEMONS_STELLA_SETQ = NULL;

Symbol* SYM_DEMONS_STELLA_NEW = NULL;

Symbol* SYM_DEMONS_STELLA_SETF = NULL;

Symbol* SYM_DEMONS_STELLA_OWNER_INSTANCE = NULL;

Symbol* SYM_DEMONS_STELLA_ACTIVE_SLOT = NULL;

Symbol* SYM_DEMONS_STELLA_GET_SLOT = NULL;

Symbol* SYM_DEMONS_STELLA_BQUOTE = NULL;

Surrogate* SGT_DEMONS_STELLA_BOOLEAN = NULL;

Symbol* SYM_DEMONS_STELLA_CANT_OVERWRITE_ACTIVE_COLLECTION_SLOT = NULL;

Surrogate* SGT_DEMONS_STELLA_LITERAL = NULL;

Symbol* SYM_DEMONS_STELLA_SLOT = NULL;

Symbol* SYM_DEMONS_STELLA_OLDVALUE = NULL;

Symbol* SYM_DEMONS_STELLA_NOT = NULL;

Symbol* SYM_DEMONS_STELLA_EQLp = NULL;

Symbol* SYM_DEMONS_STELLA_IF = NULL;

Symbol* SYM_DEMONS_STELLA_CHECK_FOR_SLOT_DEMONSp = NULL;

Symbol* SYM_DEMONS_STELLA_WRAPPEDOLDVALUE = NULL;

Symbol* SYM_DEMONS_STELLA_CHOOSE = NULL;

Symbol* SYM_DEMONS_STELLA_NULL = NULL;

Symbol* SYM_DEMONS_STELLA_WRAPPEDNEWVALUE = NULL;

Symbol* SYM_DEMONS_STELLA_RUN_SLOT_GUARD_DEMONSp = NULL;

Symbol* SYM_DEMONS_STELLA_RUN_SLOT_DEMONS = NULL;

Symbol* SYM_DEMONS_STELLA_FREE = NULL;

Symbol* SYM_DEMONS_STELLA_AND = NULL;

Symbol* SYM_DEMONS_STELLA_EQp = NULL;

Surrogate* SGT_DEMONS_STELLA_HOOK_LIST = NULL;

Symbol* SYM_DEMONS_STELLA_SIGNATURE = NULL;

Symbol* SYM_DEMONS_STELLA_DOCUMENTATION = NULL;

Symbol* SYM_DEMONS_STELLA_STARTUP_DEMONS = NULL;

Symbol* SYM_DEMONS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
