// idl-class-out.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Cons* idl_yield_initial_null_value() {
  return (((Cons*)(idl_translate_a_tree(NULL))));
}

Cons* idl_yield_attribute_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Symbol* slotname = true_slot_name(slot->slot_name, renamed_Class->class_type);
    Object* initialvalueform = slot->initially();
    Cons* facetform = NIL;
    Slot* mostgeneralslot = NULL;

    if (idl_class_category(renamed_Class) == SYM_IDL_CLASS_OUT_STELLA_IDL_INTERFACE) {
      if (slot->slot_read_onlyP) {
        facetform = listO(3, wrap_string("readonly"), wrap_string("attribute"), NIL);
      }
      else {
        facetform = cons(wrap_string("attribute"), NIL);
      }
    }
    if (!(((boolean)(initialvalueform)))) {
      initialvalueform = idl_yield_initial_null_value();
    }
    mostgeneralslot = slot;
    return (listO(3, facetform->concatenate(cons(idl_translate_type_spec(mostgeneralslot->compute_return_type_spec(renamed_Class->class_type)), NIL), 0), idl_translate_name(slotname), cons(initialvalueform, NIL)));
  }
}

Cons* idl_yield_parameterized_attribute_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Symbol* slotname = true_slot_name(slot->slot_name, renamed_Class->class_type);
    Object* initialvalueform = slot->initially();
    Keyword* allocation = slot->allocation();
    Cons* allocationform = NIL;

    if (!(((boolean)(initialvalueform)))) {
      initialvalueform = idl_yield_initial_null_value();
    }
    if (allocation == KWD_IDL_CLASS_OUT_EMBEDDED) {
      allocationform = NIL;
    }
    else if (allocation == KWD_IDL_CLASS_OUT_INSTANCE) {
      allocationform = NIL;
    }
    else if (allocation == KWD_IDL_CLASS_OUT_CLASS) {
      allocationform = cons_list(1, wrap_string("static"));
    }
    else {
      allocationform = NIL;
    }
    return (listO(3, allocationform->concatenate(cons(idl_translate_class_parameter_name(get_slot_class_parameter_name(slot)), NIL), 0), idl_translate_name(slotname), cons(initialvalueform, NIL)));
  }
}

Cons* idl_yield_static_member_allocation_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Symbol* slotname = slot->slot_name;
    Object* initialvalueform = slot->initially();
    Cons* allocationform = NIL;
    Slot* mostgeneralslot = NULL;

    if (!(((boolean)(initialvalueform)))) {
      initialvalueform = idl_yield_initial_null_value();
    }
    mostgeneralslot = slot;
    for (;;) {
      if (((boolean)(mostgeneralslot->slot_direct_equivalent))) {
        mostgeneralslot = mostgeneralslot->slot_direct_equivalent;
      }
      else {
        break;
      }
    }
    return (listO(3, allocationform->concatenate(cons(idl_translate_type_spec(mostgeneralslot->compute_return_type_spec(renamed_Class->class_type)), NIL), 0), idl_translate_name(slotname), cons(initialvalueform, NIL)));
  }
}

Cons* idl_yield_operation_signature_tree(Method_Slot* method) {
  { Method_Slot* mostgeneralmethod = method;
    Standard_Object* returntype = NULL;

    oMETHODBEINGWALKEDo.set(method);
    while (((boolean)(mostgeneralmethod->slot_direct_equivalent)) &&
        (isaP(mostgeneralmethod->slot_direct_equivalent, SGT_IDL_CLASS_OUT_STELLA_METHOD_SLOT) &&
         (!method_contains_unknown_typeP(((Method_Slot*)(mostgeneralmethod->slot_direct_equivalent)))))) {
      mostgeneralmethod = ((Method_Slot*)(mostgeneralmethod->slot_direct_equivalent));
    }
    returntype = mostgeneralmethod->slot_base_type;
    return (cons((method->method_functionP ? NULL : idl_translate_class_name(intern_symbol_in_module(method->slot_owner->symbol_name, ((Module*)(method->slot_owner->home_context)), TRUE))), listO(3, cons(idl_translate_type_spec(returntype), NIL), idl_translate_function_name(method), cons(idl_translate_method_parameters(method), NIL))));
  }
}

Cons* idl_yield_constructor_signature_tree(Method_Slot* method) {
  oMETHODBEINGWALKEDo.set(method);
  return (cons(idl_translate_class_name(intern_symbol_in_module(method->slot_owner->symbol_name, ((Module*)(method->slot_owner->home_context)), TRUE)), listO(3, NIL, idl_translate_constructor_name(method), cons(idl_translate_method_parameters(method), NIL))));
}

boolean illegal_idl_methodP(Method_Slot* methodslot) {
  return (get_quoted_tree("((DEFINED? NULL?) \"/STELLA\")", "/STELLA")->memberP(methodslot->slot_name));
}

Cons* idl_get_parameterized_attribute_definitions(Class* renamed_Class) {
  { Cons* attributedefs = NIL;

    { Slot* slot = NULL;
      Cons* iter_000 = renamed_Class->class_local_slots->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        slot = ((Slot*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (storage_slotP(slot) &&
            (native_slotP(((Storage_Slot*)(slot))) &&
             slot_has_class_parameter_typeP(((Storage_Slot*)(slot)), renamed_Class))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(idl_yield_parameterized_attribute_tree(((Storage_Slot*)(slot)), renamed_Class), NIL);
              if (attributedefs == NIL) {
                attributedefs = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(attributedefs, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(idl_yield_parameterized_attribute_tree(((Storage_Slot*)(slot)), renamed_Class), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    return (attributedefs);
  }
}

Cons* idl_get_attribute_definitions(Class* renamed_Class) {
  { Cons* attributedefs = NIL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (storage_slotP(slot) &&
            (native_slotP(((Storage_Slot*)(slot))) &&
             (native_slot_home(slot, renamed_Class) == renamed_Class))) {
          attributedefs = cons(idl_yield_attribute_tree(((Storage_Slot*)(slot)), renamed_Class), attributedefs);
        }
      }
    }
    return (attributedefs->reverse());
  }
}

Cons* idl_get_static_variable_definitions(Class* renamed_Class) {
  { Cons* staticattributedefs = NIL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (storage_slotP(slot) &&
            (native_slotP(((Storage_Slot*)(slot))) &&
             ((((Storage_Slot*)(slot))->slot_allocation_reader() == KWD_IDL_CLASS_OUT_CLASS) &&
              ((!slot_has_unknown_typeP(((Storage_Slot*)(slot)), renamed_Class)) &&
               ((!((Storage_Slot*)(slot))->slot_hardwiredP) &&
                (!slot->slot_markedP)))))) {
          staticattributedefs = cons(idl_yield_static_member_allocation_tree(((Storage_Slot*)(slot)), renamed_Class), staticattributedefs);
        }
      }
    }
    return (staticattributedefs);
  }
}

boolean idl_constructorP(Method_Slot* method) {
  return ((method->type() == SGT_IDL_CLASS_OUT_STELLA_VOID) &&
      ((method->method_parameter_names_reader()->length() == 1) &&
       (string_eqlP(method->slot_owner->symbol_name, method->slot_name->symbol_name) &&
        (((Module*)(method->slot_owner->home_context)) == ((Module*)(method->slot_name->home_context))))));
}

Cons* idl_get_constructor_definitions(Class* renamed_Class) {
  { Cons* constructordefs = NIL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();
      Cons* collect_000 = NULL;

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if ((!slot->slot_markedP) &&
            ((slot->primary_type() == SGT_IDL_CLASS_OUT_STELLA_METHOD_SLOT) &&
             (local_slotP(slot, renamed_Class) &&
              idl_constructorP(((Method_Slot*)(slot)))))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(idl_yield_constructor_signature_tree(((Method_Slot*)(slot))), NIL);
              if (constructordefs == NIL) {
                constructordefs = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(constructordefs, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(idl_yield_constructor_signature_tree(((Method_Slot*)(slot))), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    return (constructordefs);
  }
}

Cons* idl_get_operation_definitions(Class* renamed_Class) {
  { Cons* operationdefs = NIL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();
      Cons* collect_000 = NULL;

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if ((!slot->slot_markedP) &&
            ((slot->primary_type() == SGT_IDL_CLASS_OUT_STELLA_METHOD_SLOT) &&
             (local_slotP(slot, renamed_Class) &&
              ((!idl_constructorP(((Method_Slot*)(slot)))) &&
               ((!((Method_Slot*)(slot))->method_functionP) &&
                ((!illegal_idl_methodP(((Method_Slot*)(slot)))) &&
                 (!method_contains_unknown_typeP(((Method_Slot*)(slot)))))))))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(idl_yield_operation_signature_tree(((Method_Slot*)(slot))), NIL);
              if (operationdefs == NIL) {
                operationdefs = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(operationdefs, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(idl_yield_operation_signature_tree(((Method_Slot*)(slot))), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    { Method_Slot* auxiliary = NULL;
      Cons* iter_001 = renamed_Class->class_auxiliary_methods_reader()->the_cons_list;
      Cons* collect_001 = NULL;

      while (!(iter_001 == NIL)) {
        auxiliary = ((Method_Slot*)(iter_001->value));
        iter_001 = iter_001->rest;
        if (!((boolean)(collect_001))) {
          {
            collect_001 = cons(idl_yield_operation_signature_tree(auxiliary), NIL);
            if (operationdefs == NIL) {
              operationdefs = collect_001;
            }
            else {
              add_cons_to_end_of_cons_list(operationdefs, collect_001);
            }
          }
        }
        else {
          {
            collect_001->rest = cons(idl_yield_operation_signature_tree(auxiliary), NIL);
            collect_001 = collect_001->rest;
          }
        }
      }
    }
    return (operationdefs);
  }
}

Symbol* idl_class_category(Class* renamed_Class) {
  if (!(((boolean)(renamed_Class->properties_reader())))) {
    return (SYM_IDL_CLASS_OUT_STELLA_IDL_INTERFACE);
  }
  { Generalized_Symbol* test_value_000 = ((Generalized_Symbol*)(renamed_Class->properties_reader()->first()));

    if (test_value_000 == SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsINTERFACE) {
      return (SYM_IDL_CLASS_OUT_STELLA_IDL_INTERFACE);
    }
    else if (test_value_000 == SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsSTRUCT) {
      return (SYM_IDL_CLASS_OUT_STELLA_IDL_STRUCT);
    }
    else if (test_value_000 == SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsENUM) {
      return (SYM_IDL_CLASS_OUT_STELLA_IDL_ENUM);
    }
    else if (test_value_000 == SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsTYPEDEF) {
      return (SYM_IDL_CLASS_OUT_STELLA_IDL_TYPEDEF);
    }
    else {
      return (SYM_IDL_CLASS_OUT_STELLA_IDL_INTERFACE);
    }
  }
}

Cons* idl_translate_define_native_class_unit(Translation_Unit* unit) {
  { Class* renamed_Class = ((Class*)(unit->the_object));
    Cons* derivedclasses = NIL;
    Cons* parameterizedattributedefs = NIL;
    Cons* attributedefs = NIL;
    Cons* operationdefs = NIL;
    Cons* constructordefs = NIL;
    Cons* parameters = NIL;
    Cons* staticattributedefs = NIL;

    { Surrogate* super = NULL;
      Cons* iter_000 = renamed_Class->class_direct_supers->the_cons_list;
      Cons* collect_000 = NULL;

      while (!(iter_000 == NIL)) {
        super = ((Surrogate*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(idl_translate_class_name(intern_symbol_in_module(super->symbol_name, ((Module*)(super->home_context)), TRUE)), NIL);
            if (derivedclasses == NIL) {
              derivedclasses = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(derivedclasses, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(idl_translate_class_name(intern_symbol_in_module(super->symbol_name, ((Module*)(super->home_context)), TRUE)), NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
    }
    { Symbol* parameter = NULL;
      Cons* iter_001 = renamed_Class->class_parameters_reader()->the_cons_list;
      Cons* collect_001 = NULL;

      while (!(iter_001 == NIL)) {
        parameter = ((Symbol*)(iter_001->value));
        iter_001 = iter_001->rest;
        if (!((boolean)(collect_001))) {
          {
            collect_001 = cons(idl_translate_class_parameter_name(parameter), NIL);
            if (parameters == NIL) {
              parameters = collect_001;
            }
            else {
              add_cons_to_end_of_cons_list(parameters, collect_001);
            }
          }
        }
        else {
          {
            collect_001->rest = cons(idl_translate_class_parameter_name(parameter), NIL);
            collect_001 = collect_001->rest;
          }
        }
      }
    }
    { Slot* slot = NULL;
      Iterator* iter_002 = renamed_Class->class_slots();

      while (iter_002->nextP()) {
        slot = ((Slot*)(iter_002->value));
        slot->slot_markedP = FALSE;
      }
    }
    { Slot* slot = NULL;
      Iterator* iter_003 = renamed_Class->class_slots();

      while (iter_003->nextP()) {
        slot = ((Slot*)(iter_003->value));
        if (!slot->slot_markedP) {
          if (((boolean)(slot->slot_direct_equivalent)) &&
              (!slot->slot_direct_equivalent->slot_renamedP)) {
            mark_direct_equivalent_slot(slot);
            slot->slot_markedP = FALSE;
          }
        }
      }
    }
    constructordefs = idl_get_constructor_definitions(renamed_Class);
    parameterizedattributedefs = idl_get_parameterized_attribute_definitions(renamed_Class);
    attributedefs = idl_get_attribute_definitions(renamed_Class);
    operationdefs = idl_get_operation_definitions(renamed_Class);
    staticattributedefs = idl_get_static_variable_definitions(renamed_Class);
    return (cons(idl_class_category(renamed_Class), cons(idl_translate_class_name(intern_symbol_in_module(renamed_Class->class_type->symbol_name, ((Module*)(renamed_Class->class_type->home_context)), TRUE)), cons(parameters, cons(derivedclasses, cons(staticattributedefs, (((!(parameterizedattributedefs == NIL)) ? cons(listO(3, SYM_IDL_CLASS_OUT_STELLA_IDL_DECLARATIONS, parameterizedattributedefs, NIL), NIL) : NIL))->concatenate((((!(attributedefs == NIL)) ? cons(listO(3, SYM_IDL_CLASS_OUT_STELLA_IDL_DECLARATIONS, attributedefs, NIL), NIL) : NIL))->concatenate((((!(constructordefs == NIL)) ? cons(listO(3, SYM_IDL_CLASS_OUT_STELLA_IDL_SIGNATURES, constructordefs, NIL), NIL) : NIL))->concatenate((((!(operationdefs == NIL)) ? cons(listO(3, SYM_IDL_CLASS_OUT_STELLA_IDL_SIGNATURES, operationdefs, NIL), NIL) : NIL))->concatenate(NIL, 0), 0), 0), 0)))))));
  }
}

void startup_idl_class_out() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      SYM_IDL_CLASS_OUT_STELLA_IDL_INTERFACE = ((Symbol*)(intern_rigid_symbol_wrt_module("IDL_INTERFACE", NULL, 0)));
      KWD_IDL_CLASS_OUT_EMBEDDED = ((Keyword*)(intern_rigid_symbol_wrt_module("EMBEDDED", NULL, 2)));
      KWD_IDL_CLASS_OUT_INSTANCE = ((Keyword*)(intern_rigid_symbol_wrt_module("INSTANCE", NULL, 2)));
      KWD_IDL_CLASS_OUT_CLASS = ((Keyword*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 2)));
      SGT_IDL_CLASS_OUT_STELLA_METHOD_SLOT = ((Surrogate*)(intern_rigid_symbol_wrt_module("METHOD-SLOT", NULL, 1)));
      SGT_IDL_CLASS_OUT_STELLA_VOID = ((Surrogate*)(intern_rigid_symbol_wrt_module("VOID", NULL, 1)));
      SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsINTERFACE = ((Symbol*)(intern_rigid_symbol_wrt_module("/IDL-THEORY/interface", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsSTRUCT = ((Symbol*)(intern_rigid_symbol_wrt_module("/IDL-THEORY/struct", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsENUM = ((Symbol*)(intern_rigid_symbol_wrt_module("/IDL-THEORY/enum", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsTYPEDEF = ((Symbol*)(intern_rigid_symbol_wrt_module("/IDL-THEORY/typedef", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_IDL_STRUCT = ((Symbol*)(intern_rigid_symbol_wrt_module("IDL_STRUCT", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_IDL_ENUM = ((Symbol*)(intern_rigid_symbol_wrt_module("IDL_ENUM", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_IDL_TYPEDEF = ((Symbol*)(intern_rigid_symbol_wrt_module("IDL_TYPEDEF", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_IDL_DECLARATIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("IDL_DECLARATIONS", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_IDL_SIGNATURES = ((Symbol*)(intern_rigid_symbol_wrt_module("IDL_SIGNATURES", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_STARTUP_IDL_CLASS_OUT = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-IDL-CLASS-OUT", NULL, 0)));
      SYM_IDL_CLASS_OUT_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("IDL-YIELD-INITIAL-NULL-VALUE", "(DEFUN (IDL-YIELD-INITIAL-NULL-VALUE CONS) ())", ((cpp_function_code)(&idl_yield_initial_null_value)), NULL);
      define_function_object("IDL-YIELD-ATTRIBUTE-TREE", "(DEFUN (IDL-YIELD-ATTRIBUTE-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&idl_yield_attribute_tree)), NULL);
      define_function_object("IDL-YIELD-PARAMETERIZED-ATTRIBUTE-TREE", "(DEFUN (IDL-YIELD-PARAMETERIZED-ATTRIBUTE-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&idl_yield_parameterized_attribute_tree)), NULL);
      define_function_object("IDL-YIELD-STATIC-MEMBER-ALLOCATION-TREE", "(DEFUN (IDL-YIELD-STATIC-MEMBER-ALLOCATION-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&idl_yield_static_member_allocation_tree)), NULL);
      define_function_object("IDL-YIELD-OPERATION-SIGNATURE-TREE", "(DEFUN (IDL-YIELD-OPERATION-SIGNATURE-TREE CONS) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&idl_yield_operation_signature_tree)), NULL);
      define_function_object("IDL-YIELD-CONSTRUCTOR-SIGNATURE-TREE", "(DEFUN (IDL-YIELD-CONSTRUCTOR-SIGNATURE-TREE CONS) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&idl_yield_constructor_signature_tree)), NULL);
      define_function_object("ILLEGAL-IDL-METHOD?", "(DEFUN (ILLEGAL-IDL-METHOD? BOOLEAN) ((METHODSLOT METHOD-SLOT)))", ((cpp_function_code)(&illegal_idl_methodP)), NULL);
      define_function_object("IDL-GET-PARAMETERIZED-ATTRIBUTE-DEFINITIONS", "(DEFUN (IDL-GET-PARAMETERIZED-ATTRIBUTE-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&idl_get_parameterized_attribute_definitions)), NULL);
      define_function_object("IDL-GET-ATTRIBUTE-DEFINITIONS", "(DEFUN (IDL-GET-ATTRIBUTE-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&idl_get_attribute_definitions)), NULL);
      define_function_object("IDL-GET-STATIC-VARIABLE-DEFINITIONS", "(DEFUN (IDL-GET-STATIC-VARIABLE-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&idl_get_static_variable_definitions)), NULL);
      define_function_object("IDL-CONSTRUCTOR?", "(DEFUN (IDL-CONSTRUCTOR? BOOLEAN) ((METHOD METHOD-SLOT)))", ((cpp_function_code)(&idl_constructorP)), NULL);
      define_function_object("IDL-GET-CONSTRUCTOR-DEFINITIONS", "(DEFUN (IDL-GET-CONSTRUCTOR-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&idl_get_constructor_definitions)), NULL);
      define_function_object("IDL-GET-OPERATION-DEFINITIONS", "(DEFUN (IDL-GET-OPERATION-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&idl_get_operation_definitions)), NULL);
      define_function_object("IDL-CLASS-CATEGORY", "(DEFUN (IDL-CLASS-CATEGORY SYMBOL) ((CLASS CLASS)))", ((cpp_function_code)(&idl_class_category)), NULL);
      define_function_object("IDL-TRANSLATE-DEFINE-NATIVE-CLASS-UNIT", "(DEFUN (IDL-TRANSLATE-DEFINE-NATIVE-CLASS-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&idl_translate_define_native_class_unit)), NULL);
      define_function_object("STARTUP-IDL-CLASS-OUT", "(DEFUN STARTUP-IDL-CLASS-OUT () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_idl_class_out)), NULL);
      { Method_Slot* function = lookup_function(SYM_IDL_CLASS_OUT_STELLA_STARTUP_IDL_CLASS_OUT);

        set_dynamic_slot_value(function->dynamic_slots, SYM_IDL_CLASS_OUT_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Idl-Class-Out"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
  }
}

Symbol* SYM_IDL_CLASS_OUT_STELLA_IDL_INTERFACE = NULL;

Keyword* KWD_IDL_CLASS_OUT_EMBEDDED = NULL;

Keyword* KWD_IDL_CLASS_OUT_INSTANCE = NULL;

Keyword* KWD_IDL_CLASS_OUT_CLASS = NULL;

Surrogate* SGT_IDL_CLASS_OUT_STELLA_METHOD_SLOT = NULL;

Surrogate* SGT_IDL_CLASS_OUT_STELLA_VOID = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsINTERFACE = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsSTRUCT = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsENUM = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_sIDL_THEORYsTYPEDEF = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_IDL_STRUCT = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_IDL_ENUM = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_IDL_TYPEDEF = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_IDL_DECLARATIONS = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_IDL_SIGNATURES = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_STARTUP_IDL_CLASS_OUT = NULL;

Symbol* SYM_IDL_CLASS_OUT_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
