// java-class-out.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

Cons* java_yield_instance_var_tree(Storage_Slot* slot, Class* renamed_Class) {
  { Symbol* slotname = true_slot_name(slot->slot_name, renamed_Class->class_type);
    Standard_Object* slottype = (slot->slot_context_sensitiveP ? SGT_JAVA_CLASS_OUT_STELLA_OBJECT : ((Standard_Object*)(slot->compute_return_type_spec(renamed_Class->class_type))));
    Keyword* allocation = slot->allocation();
    Cons* allocationform = NULL;
    Cons* accessform = NULL;

    if (slot->slot_publicP) {
      accessform = cons(wrap_string("public"), NIL);
    }
    else {
      accessform = cons(wrap_string("protected"), NIL);
    }
    if (allocation == KWD_JAVA_CLASS_OUT_EMBEDDED) {
      allocationform = NIL;
    }
    else if (allocation == KWD_JAVA_CLASS_OUT_INSTANCE) {
      allocationform = NIL;
    }
    else if (allocation == KWD_JAVA_CLASS_OUT_CLASS) {
      allocationform = cons(wrap_string("static"), NIL);
    }
    else {
      allocationform = NIL;
    }
    return (listO(3, accessform->concatenate(allocationform->concatenate(cons(java_translate_type_spec(slottype), NIL), 0), 0), java_translate_name(slotname), NIL));
  }
}

Cons* java_get_instance_variable_definitions(Class* renamed_Class) {
  if (renamed_Class->mixinP) {
    return (NIL);
  }
  { Cons* instancevardefs = NIL;
    Class* superclass = NULL;

    { Slot* slot = NULL;
      Iterator* iter_000 = renamed_Class->class_slots();
      Cons* collect_000 = NULL;

      while (iter_000->nextP()) {
        slot = ((Slot*)(iter_000->value));
        if (storage_slotP(slot) &&
            (native_slotP(((Storage_Slot*)(slot))) &&
             (native_slot_home(slot, renamed_Class) == renamed_Class))) {
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(java_yield_instance_var_tree(((Storage_Slot*)(slot)), renamed_Class), NIL);
              if (instancevardefs == NIL) {
                instancevardefs = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(instancevardefs, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(java_yield_instance_var_tree(((Storage_Slot*)(slot)), renamed_Class), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
    }
    { Surrogate* super = NULL;
      Cons* iter_001 = renamed_Class->class_direct_supers->the_cons_list;

      while (!(iter_001 == NIL)) {
        super = ((Surrogate*)(iter_001->value));
        iter_001 = iter_001->rest;
        superclass = super->get_stella_class(TRUE);
        if (superclass->mixinP) {
          { Slot* slot = NULL;
            Iterator* iter_002 = superclass->class_slots();
            Cons* collect_001 = NULL;

            while (iter_002->nextP()) {
              slot = ((Slot*)(iter_002->value));
              if (storage_slotP(slot) &&
                  (native_slotP(((Storage_Slot*)(slot))) &&
                   (native_slot_home(slot, superclass) == superclass))) {
                if (!((boolean)(collect_001))) {
                  {
                    collect_001 = cons(java_yield_instance_var_tree(((Storage_Slot*)(slot)), superclass), NIL);
                    if (instancevardefs == NIL) {
                      instancevardefs = collect_001;
                    }
                    else {
                      add_cons_to_end_of_cons_list(instancevardefs, collect_001);
                    }
                  }
                }
                else {
                  {
                    collect_001->rest = cons(java_yield_instance_var_tree(((Storage_Slot*)(slot)), superclass), NIL);
                    collect_001 = collect_001->rest;
                  }
                }
              }
            }
          }
        }
      }
    }
    return (instancevardefs);
  }
}

Cons* java_get_constructor_and_method_definitions(Class* renamed_Class, Cons*& _return1) {
  { Cons* constructordefinitions = NIL;
    Cons* methoddefinitions = NIL;
    Method_Slot* method = NULL;

    { Translation_Unit* tu = NULL;
      Cons* iter_000 = oTRANSLATIONUNITSo.get()->the_cons_list;

      while (!(iter_000 == NIL)) {
        tu = ((Translation_Unit*)(iter_000->value));
        iter_000 = iter_000->rest;
        if ((tu->category == SYM_JAVA_CLASS_OUT_STELLA_METHOD) ||
            ((tu->category == SYM_JAVA_CLASS_OUT_STELLA_MACRO) ||
             (tu->category == SYM_JAVA_CLASS_OUT_STELLA_PRINT_METHOD))) {
          if (tu->category == SYM_JAVA_CLASS_OUT_STELLA_PRINT_METHOD) {
            if (canonical_type(((Class*)(tu->the_object))->class_type) == renamed_Class->class_type) {
              methoddefinitions = cons(tu, methoddefinitions);
            }
          }
          else {
            {
              method = ((Method_Slot*)(tu->the_object));
              if (java_flotsam_functionP(method) ||
                  method->method_startup_functionP()) {
              }
              else if (((Boolean_Wrapper*)(dynamic_slot_value(method->dynamic_slots, SYM_JAVA_CLASS_OUT_STELLA_METHOD_CONSTRUCTORp, FALSE_WRAPPER)))->wrapper_value) {
                if (canonical_type(method->slot_base_type) == renamed_Class->class_type) {
                  constructordefinitions = cons(tu, constructordefinitions);
                }
              }
              else if (canonical_type(method->slot_owner) == renamed_Class->class_type) {
                methoddefinitions = cons(tu, methoddefinitions);
              }
            }
          }
        }
      }
    }
    { Cons* _return_temp = constructordefinitions->reverse();

      _return1 = methoddefinitions->reverse();
      return (_return_temp);
    }
  }
}

Cons* java_translate_define_native_class_unit(Translation_Unit* unit) {
  return (java_translate_define_native_class(((Class*)(unit->the_object))));
}

Cons* java_translate_define_native_class(Class* renamed_Class) {
  { Object* parentclass = NULL;
    Cons* interfaceclasses = NIL;
    Cons* instancevardefs = NIL;
    Cons* constructordefs = NIL;
    Cons* modifiers = NIL;
    Cons* methoddefs = NIL;

    { 
      BIND_STELLA_SPECIAL(oCONTEXTo, Context*, renamed_Class->home_module());
      BIND_STELLA_SPECIAL(oMODULEo, Module*, oCONTEXTo.get()->base_module);
      if (renamed_Class->mixinP) {
        return (NIL);
      }
      if (renamed_Class->abstractP) {
        modifiers = cons(wrap_string("abstract"), modifiers);
      }
      if (renamed_Class->publicP()) {
        modifiers = cons(wrap_string("public"), modifiers);
      }
      if (((boolean)(((Surrogate*)(renamed_Class->class_direct_supers->first()))))) {
        parentclass = java_translate_class_name(type_to_symbol(((Surrogate*)(renamed_Class->class_direct_supers->first()))));
      }
      instancevardefs = java_get_instance_variable_definitions(renamed_Class);
      constructordefs = java_get_constructor_and_method_definitions(renamed_Class, methoddefs);
      return (cons((exception_classP(renamed_Class) ? SYM_JAVA_CLASS_OUT_STELLA_JAVA_EXCEPTION_CLASS : SYM_JAVA_CLASS_OUT_STELLA_JAVA_CLASS), cons(((((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_JAVA_CLASS_OUT_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value != NULL) ? wrap_string(((String_Wrapper*)(dynamic_slot_value(renamed_Class->dynamic_slots, SYM_JAVA_CLASS_OUT_STELLA_DOCUMENTATION, NULL_STRING_WRAPPER)))->wrapper_value) : NULL), cons(modifiers, cons((renamed_Class->mixinP ? wrap_string("interface") : wrap_string("class")), cons(java_translate_class_name(intern_symbol_in_module(renamed_Class->class_type->symbol_name, ((Module*)(renamed_Class->class_type->home_context)), TRUE)), cons(parentclass, cons(interfaceclasses, cons(((!(instancevardefs == NIL)) ? cons(cons(SYM_JAVA_CLASS_OUT_STELLA_JAVA_DECLARATIONS, instancevardefs->concatenate(NIL, 0)), NIL) : NIL), cons(constructordefs, cons(methoddefs, NIL)))))))))));
    }
  }
}

void startup_java_class_out() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      SGT_JAVA_CLASS_OUT_STELLA_OBJECT = ((Surrogate*)(intern_rigid_symbol_wrt_module("OBJECT", NULL, 1)));
      KWD_JAVA_CLASS_OUT_EMBEDDED = ((Keyword*)(intern_rigid_symbol_wrt_module("EMBEDDED", NULL, 2)));
      KWD_JAVA_CLASS_OUT_INSTANCE = ((Keyword*)(intern_rigid_symbol_wrt_module("INSTANCE", NULL, 2)));
      KWD_JAVA_CLASS_OUT_CLASS = ((Keyword*)(intern_rigid_symbol_wrt_module("CLASS", NULL, 2)));
      SYM_JAVA_CLASS_OUT_STELLA_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD", NULL, 0)));
      SYM_JAVA_CLASS_OUT_STELLA_MACRO = ((Symbol*)(intern_rigid_symbol_wrt_module("MACRO", NULL, 0)));
      SYM_JAVA_CLASS_OUT_STELLA_PRINT_METHOD = ((Symbol*)(intern_rigid_symbol_wrt_module("PRINT-METHOD", NULL, 0)));
      SYM_JAVA_CLASS_OUT_STELLA_METHOD_CONSTRUCTORp = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-CONSTRUCTOR?", NULL, 0)));
      SYM_JAVA_CLASS_OUT_STELLA_JAVA_EXCEPTION_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_EXCEPTION_CLASS", NULL, 0)));
      SYM_JAVA_CLASS_OUT_STELLA_JAVA_CLASS = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_CLASS", NULL, 0)));
      SYM_JAVA_CLASS_OUT_STELLA_DOCUMENTATION = ((Symbol*)(intern_rigid_symbol_wrt_module("DOCUMENTATION", NULL, 0)));
      SYM_JAVA_CLASS_OUT_STELLA_JAVA_DECLARATIONS = ((Symbol*)(intern_rigid_symbol_wrt_module("JAVA_DECLARATIONS", NULL, 0)));
      SYM_JAVA_CLASS_OUT_STELLA_STARTUP_JAVA_CLASS_OUT = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-JAVA-CLASS-OUT", NULL, 0)));
      SYM_JAVA_CLASS_OUT_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("JAVA-YIELD-INSTANCE-VAR-TREE", "(DEFUN (JAVA-YIELD-INSTANCE-VAR-TREE CONS) ((SLOT STORAGE-SLOT) (CLASS CLASS)))", ((cpp_function_code)(&java_yield_instance_var_tree)), NULL);
      define_function_object("JAVA-GET-INSTANCE-VARIABLE-DEFINITIONS", "(DEFUN (JAVA-GET-INSTANCE-VARIABLE-DEFINITIONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&java_get_instance_variable_definitions)), NULL);
      define_function_object("JAVA-GET-CONSTRUCTOR-AND-METHOD-DEFINITIONS", "(DEFUN (JAVA-GET-CONSTRUCTOR-AND-METHOD-DEFINITIONS CONS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&java_get_constructor_and_method_definitions)), NULL);
      define_function_object("JAVA-TRANSLATE-DEFINE-NATIVE-CLASS-UNIT", "(DEFUN (JAVA-TRANSLATE-DEFINE-NATIVE-CLASS-UNIT CONS) ((UNIT TRANSLATION-UNIT)))", ((cpp_function_code)(&java_translate_define_native_class_unit)), NULL);
      define_function_object("JAVA-TRANSLATE-DEFINE-NATIVE-CLASS", "(DEFUN (JAVA-TRANSLATE-DEFINE-NATIVE-CLASS CONS) ((CLASS CLASS)))", ((cpp_function_code)(&java_translate_define_native_class)), NULL);
      define_function_object("STARTUP-JAVA-CLASS-OUT", "(DEFUN STARTUP-JAVA-CLASS-OUT () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_java_class_out)), NULL);
      { Method_Slot* function = lookup_function(SYM_JAVA_CLASS_OUT_STELLA_STARTUP_JAVA_CLASS_OUT);

        set_dynamic_slot_value(function->dynamic_slots, SYM_JAVA_CLASS_OUT_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Java-Class-Out"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
  }
}

Surrogate* SGT_JAVA_CLASS_OUT_STELLA_OBJECT = NULL;

Keyword* KWD_JAVA_CLASS_OUT_EMBEDDED = NULL;

Keyword* KWD_JAVA_CLASS_OUT_INSTANCE = NULL;

Keyword* KWD_JAVA_CLASS_OUT_CLASS = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_METHOD = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_MACRO = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_PRINT_METHOD = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_METHOD_CONSTRUCTORp = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_JAVA_EXCEPTION_CLASS = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_JAVA_CLASS = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_DOCUMENTATION = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_JAVA_DECLARATIONS = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_STARTUP_JAVA_CLASS_OUT = NULL;

Symbol* SYM_JAVA_CLASS_OUT_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
