// lists.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

List* NIL_LIST = NULL;

boolean defined_listP(List* self) {
  // Return TRUE unless `self' is NULL or the `NIL-LIST'.
  return ((!(self == NIL_LIST)) &&
      ((boolean)(self)));
}

boolean null_listP(List* self) {
  // Return TRUE iff `self' is NULL or the `NIL-LIST'.
  return ((self == NIL_LIST) ||
      (!((boolean)(self))));
}

boolean List::emptyP() {
  // Return TRUE if the list `self' has no members.
  { List* self = this;

    return (self->the_cons_list == NIL);
  }
}

boolean List::non_emptyP() {
  // Return TRUE if the list `self' has at least one member.
  { List* self = this;

    return (!(self->the_cons_list == NIL));
  }
}

boolean terminate_listP(List* self) {
  if (self == NIL_LIST) {
    if (!(!(self == NIL_LIST))) {
      cerr << "Safety violation: " << "Attempt to free NIL-LIST.";
    }
    return (FALSE);
  }
  { Cons* cons = self->the_cons_list;
    Cons* rest = NULL;

    while (!(cons == NIL)) {
      rest = cons->rest;
      cons->free();
      cons = rest;
    }
    self->the_cons_list = NIL;
    return (TRUE);
  }
}

boolean List::memberP(Object* object) {
  // Return TRUE iff `object' is a member of the list
  // `self' (uses an `eql?' test).
  { List* self = this;

    { Object* i = NULL;
      Cons* iter_000 = self->the_cons_list;

      while (!(iter_000 == NIL)) {
        i = iter_000->value;
        iter_000 = iter_000->rest;
        if (eqlP(i, object)) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

boolean List::membP(Object* object) {
  // Return TRUE iff `object' is a member of the cons list
  // `self' (uses an 'eq?' test).
  { List* self = this;

    { Object* i = NULL;
      Cons* iter_000 = self->the_cons_list;

      while (!(iter_000 == NIL)) {
        i = iter_000->value;
        iter_000 = iter_000->rest;
        if (i == object) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

Object* List::first() {
  // Return the first item in the list `self', or NULL if empty.
  { List* self = this;

    { Cons* cons = self->the_cons_list;

      return (((!(cons == NIL)) ? cons->value : NULL));
    }
  }
}

Object* List::second() {
  // Return the second item in the list `self', or NULL if empty.
  { List* self = this;

    return (self->the_cons_list->rest->value);
  }
}

Object* List::third() {
  // Return the third item in the list `self', or NULL if empty.
  { List* self = this;

    return (self->the_cons_list->rest->rest->value);
  }
}

Object* List::fourth() {
  // Return the fourth item in the list `self', or NULL if empty.
  { List* self = this;

    return (self->the_cons_list->fourth());
  }
}

Object* List::fifth() {
  // Return the fifth item in the list `self', or NULL if empty.
  { List* self = this;

    return (self->the_cons_list->fifth());
  }
}

Object* List::nth(int position) {
  // Return the nth item in the list `self', or NULL if empty.
  { List* self = this;

    return (self->the_cons_list->nth(position));
  }
}

Object* List::last() {
  // Return the last element of `self'.
  { List* self = this;

    return (self->the_cons_list->last());
  }
}

Cons* List::rest() {
  // Return a cons list of all but the first item in the list `self'.
  { List* self = this;

    return (self->the_cons_list->rest);
  }
}

Object* List::first_setter(Object* value) {
  { List* self = this;

    return (self->the_cons_list->first_setter(value));
  }
}

Object* List::second_setter(Object* value) {
  { List* self = this;

    return (self->the_cons_list->second_setter(value));
  }
}

Object* List::third_setter(Object* value) {
  { List* self = this;

    return (self->the_cons_list->third_setter(value));
  }
}

Object* List::fourth_setter(Object* value) {
  { List* self = this;

    return (self->the_cons_list->fourth_setter(value));
  }
}

Object* List::fifth_setter(Object* value) {
  { List* self = this;

    return (self->the_cons_list->fifth_setter(value));
  }
}

Object* List::nth_setter(Object* value, int position) {
  { List* self = this;

    return (self->the_cons_list->nth_setter(value, position));
  }
}

int List::length() {
  { List* self = this;

    // inline method
    return (self->the_cons_list->length());
  }
}

int List::position(Object* object, int start) {
  // Return the position of `object' within the list
  // `self' (counting from zero); or return NULL if `object' does not occur within 
  // `self' (uses an `eql?' test).  If `start' was supplied as non-NULL, only 
  // consider the sublist starting at `start', however, the returned position 
  // will always be relative to the entire list.
  { List* self = this;

    return (self->the_cons_list->position(object, start));
  }
}

void List::insert(Object* value) {
  // Add `value' to the front of the list `self'.
  { List* self = this;

    if (!(!(self == NIL_LIST))) {
      cerr << "Safety violation: " << "Attempt to insert into NIL-LIST.";
    }
    self->the_cons_list = cons(value, self->the_cons_list);
  }
}

void List::push(Object* value) {
  // Add `value' to the front of the list `self'.
  { List* self = this;

    if (!(!(self == NIL_LIST))) {
      cerr << "Safety violation: " << "Attempt to insert into NIL-LIST.";
    }
    self->the_cons_list = cons(value, self->the_cons_list);
  }
}

void List::insert_new(Object* value) {
  // Add `value' to the front of the list `self' unless its 
  // already a member.
  { List* self = this;

    if (!(!(self == NIL_LIST))) {
      cerr << "Safety violation: " << "Attempt to insert into NIL-LIST.";
    }
    if (!self->the_cons_list->memberP(value)) {
      self->the_cons_list = cons(value, self->the_cons_list);
    }
  }
}

void List::insert_last(Object* value) {
  // Insert `value' as the last entry in the list `self'.
  { List* self = this;

    if (!(!(self == NIL_LIST))) {
      cerr << "Safety violation: " << "Attempt to insert into NIL-LIST.";
    }
    { Cons* cursor = self->the_cons_list;
      Cons* lastcons = cons(value, NIL);

      if (cursor == NIL) {
        self->the_cons_list = lastcons;
      }
      else {
        {
          while (!(cursor->rest == NIL)) {
            cursor = cursor->rest;
          }
          cursor->rest = lastcons;
        }
      }
    }
  }
}

Abstract_Collection* List::remove(Object* value) {
  // Destructively remove all entries in `self' that match `value'.
  { List* self = this;

    self->the_cons_list = self->the_cons_list->remove(value);
    return (self);
  }
}

List* List::remove_if(cpp_function_code testP) {
  // Destructively remove all members of the list `self' for which
  // 'test?' evaluates to TRUE.  `test' takes a single argument of type OBJECT and
  // returns TRUE or FALSE.  Returns `self'.
  { List* self = this;

    self->the_cons_list = self->the_cons_list->remove_if(testP);
    return (self);
  }
}

Collection* List::remove_duplicates() {
  // Destructively remove duplicates from `self' and return the result.
  // Preserves the original order of the remaining members.
  { List* self = this;

    if (!(self->the_cons_list == NIL)) {
      self->the_cons_list = self->the_cons_list->remove_duplicates();
    }
    return (self);
  }
}

Object* List::pop() {
  // Remove and return the first element in the list `self'.
  // Return NULL if the list is empty.
  { List* self = this;

    { Cons* cons = self->the_cons_list;
      Object* value = NULL;

      if (cons == NIL) {
        return (NULL);
      }
      value = cons->value;
      self->the_cons_list = cons->rest;
      return (value);
    }
  }
}

List* List::reverse() {
  // Reverse the members of `self' (in place).
  { List* self = this;

    self->the_cons_list = self->the_cons_list->reverse();
    return (self);
  }
}

List* List::substitute(Object* invalue, Object* outvalue) {
  // Destructively replace each appearance of `outValue' by
  // `inValue' in the list `self'.
  { List* self = this;

    self->the_cons_list = self->the_cons_list->substitute(invalue, outvalue);
    return (self);
  }
}

List* List::concatenate(List* list2, int otherlists, ...) {
  // Copy `list2' and all `otherLists' onto the end of `list1'.
  // The operation is destructive wrt `list1', but leaves all other lists intact.
  // The two mandatory parameters allow us to optimize the common binary case by
  // not relying on the somewhat less efficient variable arguments mechanism.
  { List* list1 = this;

    { Cons* firstlist = list1->the_cons_list;
      Cons* lastlist = list2->the_cons_list;
      Cons* copy = NIL;

      if (firstlist == NIL) {
        firstlist = copy_cons_list(lastlist);
        lastlist = firstlist;
      }
      else if (lastlist == NIL) {
        lastlist = firstlist;
      }
      else {
        lastlist = copy_cons_list(lastlist);
        last_cons(firstlist)->rest = lastlist;
      }
      if (otherlists > 0) {
        { List* list = NULL;
          va_list iter_000;
          int iter_000_count = otherlists;

          va_start(iter_000, otherlists);
          while ((iter_000_count--) > 0) {
            list = va_arg(iter_000, List*);
            if (list->non_emptyP()) {
              copy = copy_cons_list(list->the_cons_list);
              if (!(lastlist == NIL)) {
                last_cons(lastlist)->rest = copy;
              }
              else {
                firstlist = copy;
              }
              lastlist = copy;
            }
          }
          va_end(iter_000);
        }
      }
      list1->the_cons_list = firstlist;
      return (list1);
    }
  }
}

List* List::prepend(List* list2) {
  // Copy `list2' onto the front of the list `self'.
  // The operation is destructive wrt `self', but leaves `list2' intact.
  { List* self = this;

    if (list2->emptyP()) {
      return (self);
    }
    self->the_cons_list = copy_cons_list(list2->the_cons_list)->concatenate(self->the_cons_list, 0);
    return (self);
  }
}

List* List::copy() {
  // Return a copy of the list `self'.  The conses in the copy are
  // freshly allocated.
  { List* self = this;

    { List* copy = ((List*)(create_object(self->primary_type(), 0)));

      copy->the_cons_list = copy_cons_list(self->the_cons_list);
      return (copy);
    }
  }
}

void List::clear() {
  // Make `self' an empty list.
  { List* self = this;

    self->the_cons_list = NIL;
  }
}

List* List::sort(cpp_function_code predicate) {
  // Perform a stable, destructive sort of `self' according to
  // `predicate', and return the result.  If `predicate' has a '<' semantics, the
  // result will be in ascending order.  If `predicate' is NULL, a suitable
  // '<' predicate is chosen depending on the first element of `self', and it 
  // is assumed that all elements of `self' have the same type (supported
  // element types are GENERALIZED-SYMBOL, STRING, INTEGER, and FLOAT).
  { List* self = this;

    self->the_cons_list = self->the_cons_list->sort(predicate);
    return (self);
  }
}

List* map_null_to_nil_list(List* self) {
  // Return NIL-LIST iff `self' is NULL or `self' otherwise.
  return (((!((boolean)(self))) ? NIL_LIST : self));
}

boolean List::object_equalP(Object* y) {
  // Return TRUE iff the lists `x' and `y' are structurally
  // equivalent.  Uses `equal?' to test equality of elements.
  { List* x = this;

    if (subtype_ofP(safe_primary_type(y), SGT_LISTS_STELLA_LIST)) {
      { Object* y_000 = y;
        List* y = ((List*)(y_000));

        return (equalP(x->the_cons_list, y->the_cons_list));
      }
    }
    else {
    }
    return (FALSE);
  }
}

Abstract_Iterator* List::allocate_iterator() {
  { List* self = this;

    { List_Iterator* iterator = new_list_iterator();

      iterator->list_iterator_cursor = self->the_cons_list;
      iterator->list_iterator_collection = self;
      iterator->first_iterationP = TRUE;
      return (iterator);
    }
  }
}

boolean terminate_list_iteratorP(List_Iterator* self) {
  self->list_iterator_cursor = NIL;
  self->list_iterator_collection = NULL;
  return (TRUE);
}

boolean List_Iterator::nextP() {
  { List_Iterator* self = this;

    if (self->first_iterationP) {
      {
        self->first_iterationP = FALSE;
        self->list_iterator_cursor = self->list_iterator_collection->the_cons_list;
      }
    }
    else {
      self->list_iterator_cursor = self->list_iterator_cursor->rest;
    }
    self->value = self->list_iterator_cursor->value;
    return (!(self->list_iterator_cursor == NIL));
  }
}

boolean List_Iterator::emptyP() {
  { List_Iterator* self = this;

    return ((self->first_iterationP ? (self->list_iterator_collection->the_cons_list == NIL) : (self->list_iterator_cursor->rest == NIL)));
  }
}

Object* List_Iterator::value_setter(Object* value) {
  { List_Iterator* self = this;

    self->list_iterator_cursor->value = value;
    self->value = value;
    return (value);
  }
}

Iterator* List::but_last() {
  // Generate all but the last element of the list `self'.
  { List* self = this;

    return (self->the_cons_list->but_last());
  }
}

List* List::remove_deleted_members() {
  { List* self = this;

    self->the_cons_list = self->the_cons_list->remove_deleted_members();
    return (self);
  }
}

void Property_List::initialize_object() {
  { Property_List* self = this;

    self->the_plist = NIL;
  }
}

Object* search_plist(Cons* plist, Object* key) {
  while (!(plist == NIL)) {
    if (eqlP(plist->value, key)) {
      return (plist->rest->value);
    }
    plist = plist->rest->rest;
  }
  return (NULL);
}

Object* Property_List::lookup(Object* key) {
  { Property_List* self = this;

    // inline method
    return (search_plist(self->the_plist, key));
  }
}

int Property_List::length() {
  { Property_List* self = this;

    { Cons* cursor = self->the_plist;
      int length = 0;

      while (!(cursor == NIL)) {
        length = length + 1;
        cursor = cursor->rest->rest;
      }
      return (length);
    }
  }
}

void Property_List::insert_at(Object* key, Object* value) {
  // Insert the entry <`key', `value'> into the property list `self'.
  // If a previous entry existed with key `key', that entry is replaced.
  { Property_List* self = this;

    { Cons* cursor = self->the_plist;

      while (!(cursor == NIL)) {
        if (eqlP(cursor->value, key)) {
          cursor = cursor->rest;
          cursor->value = value;
          return;
        }
        cursor = cursor->rest->rest;
      }
      self->the_plist = cons(key, cons(value, self->the_plist));
    }
  }
}

Object* Property_List::remove_at(Object* key) {
  // Remove the entry that matches the key `key'.  Return the
  // value of the matching entry, or NULL if there is no matching entry.  Assumes that at
  // most one entry matches `key'.
  { Property_List* self = this;

    { Cons* cursor = self->the_plist;
      Cons* previouscons = NULL;
      Object* value = NULL;

      if (!(cursor == NIL)) {
        if (eqlP(cursor->value, key)) {
          self->the_plist = cursor->rest->rest;
          value = cursor->rest->value;
          return (value);
        }
        previouscons = cursor->rest;
        cursor = previouscons->rest;
        while (!(cursor == NIL)) {
          if (eqlP(cursor->value, key)) {
            previouscons->rest = cursor->rest->rest;
            value = cursor->rest->value;
            return (value);
          }
          previouscons = cursor->rest;
          cursor = previouscons->rest;
        }
      }
      return (NULL);
    }
  }
}

boolean Property_List::emptyP() {
  { Property_List* self = this;

    return (self->the_plist == NIL);
  }
}

boolean Property_List::non_emptyP() {
  { Property_List* self = this;

    return (!(self->the_plist == NIL));
  }
}

Property_List* Property_List::copy() {
  // Return a copy of the list `self'.  The conses in the copy are
  // freshly allocated.
  { Property_List* self = this;

    { Property_List* copy = ((Property_List*)(create_object(self->primary_type(), 0)));

      copy->the_plist = copy_cons_list(self->the_plist);
      return (copy);
    }
  }
}

void Property_List::clear() {
  // Make `self' an empty property list.
  { Property_List* self = this;

    self->the_plist = NIL;
  }
}

boolean Property_List::object_equalP(Object* y) {
  // Return TRUE if `x' and `y' represent the same set of key/value pairs..
  { Property_List* x = this;

    if (subtype_ofP(safe_primary_type(y), SGT_LISTS_STELLA_PROPERTY_LIST)) {
      { Object* y_000 = y;
        Property_List* y = ((Property_List*)(y_000));

        if (x->length() == y->length()) {
          { Object* key = NULL;
            Object* value = NULL;
            Cons* iter_000 = x->the_plist;

            while (!(iter_000 == NIL)) {
              key = iter_000->value;
              value = iter_000->rest->value;
              iter_000 = iter_000->rest->rest;
              if (!equalP(value, y->lookup(key))) {
                return (FALSE);
              }
            }
          }
          return (TRUE);
        }
      }
    }
    else {
    }
    return (FALSE);
  }
}

boolean List::equivalent_setsP(List* otherlist) {
  { List* self = this;

    { boolean test_value_000 = FALSE;

      { boolean alwaysP_000 = TRUE;

        { Object* item = NULL;
          Cons* iter_000 = self->the_cons_list;

          while (!(iter_000 == NIL)) {
            item = iter_000->value;
            iter_000 = iter_000->rest;
            if (!otherlist->memberP(item)) {
              alwaysP_000 = FALSE;
              break;
            }
          }
        }
        test_value_000 = alwaysP_000;
      }
      if (test_value_000) {
        test_value_000 = self->length() == otherlist->length();
      }
      { boolean value_000 = test_value_000;

        return (value_000);
      }
    }
  }
}

List* List::intersection(List* otherlist) {
  { List* self = this;

    { List* list = new_list();

      { Object* i = NULL;
        Cons* iter_000 = self->the_cons_list;
        Cons* collect_000 = NULL;

        while (!(iter_000 == NIL)) {
          i = iter_000->value;
          iter_000 = iter_000->rest;
          if (otherlist->memberP(i)) {
            if (!((boolean)(collect_000))) {
              {
                collect_000 = cons(i, NIL);
                if (list->the_cons_list == NIL) {
                  list->the_cons_list = collect_000;
                }
                else {
                  add_cons_to_end_of_cons_list(list->the_cons_list, collect_000);
                }
              }
            }
            else {
              {
                collect_000->rest = cons(i, NIL);
                collect_000 = collect_000->rest;
              }
            }
          }
        }
      }
      return (list);
    }
  }
}

List* List::renamed_Union(List* otherlist) {
  { List* self = this;

    { List* list = self->copy();
      List* othersurvivors = new_list();

      { Object* i = NULL;
        Cons* iter_000 = otherlist->the_cons_list;
        Cons* collect_000 = NULL;

        while (!(iter_000 == NIL)) {
          i = iter_000->value;
          iter_000 = iter_000->rest;
          if (!list->memberP(i)) {
            if (!((boolean)(collect_000))) {
              {
                collect_000 = cons(i, NIL);
                if (othersurvivors->the_cons_list == NIL) {
                  othersurvivors->the_cons_list = collect_000;
                }
                else {
                  add_cons_to_end_of_cons_list(othersurvivors->the_cons_list, collect_000);
                }
              }
            }
            else {
              {
                collect_000->rest = cons(i, NIL);
                collect_000 = collect_000->rest;
              }
            }
          }
        }
      }
      return (list->concatenate(othersurvivors, 0));
    }
  }
}

List* List::difference(List* otherlist) {
  { List* self = this;

    { List* list = self->copy();

      if (!((boolean)(otherlist))) {
        return (list);
      }
      { Object* i = NULL;
        Cons* iter_000 = otherlist->the_cons_list;

        while (!(iter_000 == NIL)) {
          i = iter_000->value;
          iter_000 = iter_000->rest;
          if (list->memberP(i)) {
            list->remove(i);
          }
        }
      }
      return (list);
    }
  }
}

List* List::subtract(List* otherlist) {
  { List* self = this;

    if (!((boolean)(otherlist))) {
      return (self);
    }
    { Object* i = NULL;
      Cons* iter_000 = otherlist->the_cons_list;

      while (!(iter_000 == NIL)) {
        i = iter_000->value;
        iter_000 = iter_000->rest;
        if (self->memberP(i)) {
          self->remove(i);
        }
      }
    }
    return (self);
  }
}

Abstract_Iterator* Property_List::allocate_iterator() {
  { Property_List* self = this;

    { Property_List_Iterator* iterator = new_property_list_iterator();

      iterator->plist_iterator_cursor = self->the_plist;
      iterator->plist_iterator_collection = self;
      iterator->first_iterationP = TRUE;
      return (iterator);
    }
  }
}

boolean Property_List_Iterator::nextP() {
  { Property_List_Iterator* self = this;

    if (self->first_iterationP) {
      self->first_iterationP = FALSE;
    }
    else {
      self->plist_iterator_cursor = self->plist_iterator_cursor->rest->rest;
    }
    { Cons* remainingplist = self->plist_iterator_cursor;

      if (!(remainingplist == NIL)) {
        self->key = remainingplist->value;
        self->value = remainingplist->rest->value;
        return (TRUE);
      }
      return (FALSE);
    }
  }
}

Object* Property_List_Iterator::value_setter(Object* value) {
  { Property_List_Iterator* self = this;

    self->plist_iterator_cursor->rest->value = value;
    self->value = value;
    return (value);
  }
}

Object* Property_List_Iterator::key_setter(Object* key) {
  { Property_List_Iterator* self = this;

    self->plist_iterator_cursor->value = key;
    self->key = key;
    return (key);
  }
}

Kv_Cons* kv_cons(Object* key, Object* value, Kv_Cons* rest) {
  // Create, fill-in, and return a new KV-CONS.
  { Kv_Cons* newkvc = new_kv_cons();

    newkvc->key = key;
    newkvc->value = value;
    newkvc->rest = rest;
    return (newkvc);
  }
}

void free_kv_cons(Kv_Cons* kvcons) {
  kvcons->free();
}

Object* Key_Value_List::lookup(Object* key) {
  { Key_Value_List* self = this;

    { Kv_Cons* cursor = self->the_kv_list;

      while (((boolean)(cursor))) {
        if (eqlP(cursor->key, key)) {
          return (cursor->value);
        }
        cursor = cursor->rest;
      }
      return (NULL);
    }
  }
}

int Key_Value_List::length() {
  { Key_Value_List* self = this;

    { int length = 0;
      Kv_Cons* cursor = self->the_kv_list;

      while (((boolean)(cursor))) {
        length = length + 1;
        cursor = cursor->rest;
      }
      return (length);
    }
  }
}

void Key_Value_List::insert_at(Object* key, Object* value) {
  // Insert the entry <`key', `value'> into the association
  // `self'. If a previous entry existed with key `key', that entry is replaced.
  { Key_Value_List* self = this;

    { Kv_Cons* cursor = self->the_kv_list;

      while (((boolean)(cursor))) {
        if (eqlP(cursor->key, key)) {
          cursor->value = value;
          return;
        }
        cursor = cursor->rest;
      }
      self->the_kv_list = kv_cons(key, value, self->the_kv_list);
    }
  }
}

Object* Key_Value_List::remove_at(Object* key) {
  // Remove the entry that matches the key `key'.
  // Return the value of the matching entry, or NULL if there is no matching entry.
  // Assumes that at most one entry matches `key'.
  { Key_Value_List* self = this;

    { Kv_Cons* cursor = self->the_kv_list;
      Kv_Cons* previouscursor = NULL;
      Object* value = NULL;

      if (((boolean)(cursor))) {
        if (eqlP(cursor->key, key)) {
          self->the_kv_list = cursor->rest;
          value = cursor->value;
          free_kv_cons(cursor);
          return (value);
        }
        while (((boolean)(cursor->rest))) {
          previouscursor = cursor;
          cursor = cursor->rest;
          if (eqlP(cursor->key, key)) {
            previouscursor->rest = cursor->rest;
            value = cursor->value;
            free_kv_cons(cursor);
            return (value);
          }
        }
      }
      return (NULL);
    }
  }
}

void Key_Value_List::insert_entry(Object* key, Object* value) {
  // Insert an entry <`key',`value'> to `self' unless an identical
  // entry already exists.  This can generate duplicate entries for `key'.
  { Key_Value_List* self = this;

    { boolean foundP_000 = FALSE;

      { Object* k = NULL;
        Object* v = NULL;
        Kv_Cons* iter_000 = self->the_kv_list;

        while (((boolean)(iter_000))) {
          k = iter_000->key;
          v = iter_000->value;
          iter_000 = iter_000->rest;
          if ((k == key) &&
              (v == value)) {
            foundP_000 = TRUE;
            break;
          }
        }
      }
      if (!(foundP_000)) {
        self->the_kv_list = kv_cons(key, value, self->the_kv_list);
      }
    }
  }
}

void Key_Value_List::remove_entry(Object* key, Object* value) {
  // Remove the entry that matches <`key',`value'>.
  // Assumes that more than one entry can match `key'.
  { Key_Value_List* self = this;

    { Kv_Cons* cursor = self->the_kv_list;
      Kv_Cons* previouscursor = NULL;

      if (((boolean)(cursor))) {
        if (eqlP(cursor->key, key) &&
            eqlP(cursor->value, value)) {
          self->the_kv_list = cursor->rest;
          free_kv_cons(cursor);
          return;
        }
        while (((boolean)(cursor->rest))) {
          previouscursor = cursor;
          cursor = cursor->rest;
          if (eqlP(cursor->key, key) &&
              eqlP(cursor->value, value)) {
            previouscursor->rest = cursor->rest;
            free_kv_cons(cursor);
            return;
          }
        }
      }
    }
  }
}

Object* Key_Value_List::pop() {
  // Remove and return the first element of the kv-list `self'.
  // Return NULL if the list is empty.
  { Key_Value_List* self = this;

    if (self->emptyP()) {
      return (NULL);
    }
    { Kv_Cons* cons = self->the_kv_list;
      Object* value = NULL;

      value = cons->value;
      self->the_kv_list = cons->rest;
      free_kv_cons(cons);
      return (value);
    }
  }
}

Key_Value_List* Key_Value_List::reverse() {
  // Destructively reverse the members of the list `self'.
  { Key_Value_List* self = this;

    { Kv_Cons* reversedlist = self->the_kv_list;
      Kv_Cons* remaininglist = NULL;
      Kv_Cons* temp = NULL;

      if (!((boolean)(reversedlist))) {
        return (self);
      }
      remaininglist = reversedlist->rest;
      if (!((boolean)(remaininglist))) {
        return (self);
      }
      reversedlist->rest = NULL;
      while (((boolean)(remaininglist))) {
        temp = remaininglist;
        remaininglist = remaininglist->rest;
        temp->rest = reversedlist;
        reversedlist = temp;
      }
      self->the_kv_list = reversedlist;
      return (self);
    }
  }
}

boolean Key_Value_List::emptyP() {
  { Key_Value_List* self = this;

    return (!((boolean)(self->the_kv_list)));
  }
}

boolean Key_Value_List::non_emptyP() {
  { Key_Value_List* self = this;

    return (((boolean)(self->the_kv_list)));
  }
}

Kv_Cons* copy_kv_cons_list(Kv_Cons* kvconslist) {
  // Return a copy of the cons list `consList'.
  if (!((boolean)(kvconslist))) {
    return (NULL);
  }
  { Kv_Cons* newkvconslist = new_kv_cons();
    Kv_Cons* nextkvcons = newkvconslist;
    Kv_Cons* copyfromkvcons = NULL;
    Kv_Cons* previouskvcons = NULL;

    nextkvcons->key = kvconslist->key;
    nextkvcons->value = kvconslist->value;
    copyfromkvcons = kvconslist->rest;
    while (((boolean)(copyfromkvcons))) {
      previouskvcons = nextkvcons;
      nextkvcons = new_kv_cons();
      previouskvcons->rest = nextkvcons;
      nextkvcons->key = copyfromkvcons->key;
      nextkvcons->value = copyfromkvcons->value;
      copyfromkvcons = copyfromkvcons->rest;
    }
    nextkvcons->rest = NULL;
    return (newkvconslist);
  }
}

Key_Value_List* Key_Value_List::copy() {
  // Return a copy of the kv-list `self'.  The kv-conses in
  // the copy are freshly allocated.
  { Key_Value_List* self = this;

    { Key_Value_List* copy = new_key_value_list();

      copy->the_kv_list = copy_kv_cons_list(self->the_kv_list);
      return (copy);
    }
  }
}

void Key_Value_List::clear() {
  // Make `self' an empty dictionary.
  { Key_Value_List* self = this;

    { Kv_Cons* next = self->the_kv_list;
      Kv_Cons* current = NULL;

      while (((boolean)(next))) {
        current = next;
        next = next->rest;
        free_kv_cons(current);
      }
      self->the_kv_list = NULL;
    }
  }
}

boolean Key_Value_List::object_equalP(Object* y) {
  // Return TRUE if `x' and `y' represent the same set of key/value pairs.
  { Key_Value_List* x = this;

    if (subtype_ofP(safe_primary_type(y), SGT_LISTS_STELLA_KEY_VALUE_LIST)) {
      { Object* y_000 = y;
        Key_Value_List* y = ((Key_Value_List*)(y_000));

        if (x->length() == y->length()) {
          { Object* key = NULL;
            Object* value = NULL;
            Kv_Cons* iter_000 = x->the_kv_list;

            while (((boolean)(iter_000))) {
              key = iter_000->key;
              value = iter_000->value;
              iter_000 = iter_000->rest;
              if (!equalP(value, y->lookup(key))) {
                return (FALSE);
              }
            }
          }
          return (TRUE);
        }
      }
    }
    else {
    }
    return (FALSE);
  }
}

Abstract_Iterator* Key_Value_List::allocate_iterator() {
  { Key_Value_List* self = this;

    { Kv_List_Iterator* iterator = new_kv_list_iterator();

      iterator->the_kv_list = self;
      iterator->kv_list_iterator_cursor = self->the_kv_list;
      iterator->first_iterationP = TRUE;
      return (iterator);
    }
  }
}

boolean Kv_List_Iterator::nextP() {
  { Kv_List_Iterator* self = this;

    if (self->first_iterationP) {
      self->first_iterationP = FALSE;
    }
    else {
      self->kv_list_iterator_cursor = self->kv_list_iterator_cursor->rest;
    }
    if (!((boolean)(self->kv_list_iterator_cursor))) {
      return (FALSE);
    }
    self->key = self->kv_list_iterator_cursor->key;
    self->value = self->kv_list_iterator_cursor->value;
    return (TRUE);
  }
}

Object* Kv_List_Iterator::value_setter(Object* value) {
  { Kv_List_Iterator* self = this;

    self->kv_list_iterator_cursor->value = value;
    self->value = value;
    return (value);
  }
}

Object* Kv_List_Iterator::key_setter(Object* key) {
  { Kv_List_Iterator* self = this;

    self->kv_list_iterator_cursor->key = key;
    self->key = key;
    return (key);
  }
}

boolean Sequence::memberP(Object* value) {
  // Return TRUE if `value' is a member of the sequence `self'.
  { Sequence* self = this;

    { Object* m = NULL;
      Iterator* iter_000 = ((Iterator*)(self->allocate_iterator()));

      while (iter_000->nextP()) {
        m = iter_000->value;
        if (eqlP(m, value)) {
          return (TRUE);
        }
      }
    }
    return (FALSE);
  }
}

List* list(int values, ...) {
  // Return a list containing `values', in order.
  { List* list = new_list();

    { Object* v = NULL;
      va_list iter_000;
      int iter_000_count = values;
      Cons* collect_000 = NULL;

      va_start(iter_000, values);
      while ((iter_000_count--) > 0) {
        v = va_arg(iter_000, Object*);
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(v, NIL);
            if (list->the_cons_list == NIL) {
              list->the_cons_list = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(list->the_cons_list, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(v, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
      va_end(iter_000);
    }
    return (list);
  }
}

Sequence* sequence(Surrogate* collectiontype, int values, ...) {
  // Return a sequence containing `values', in order.
  { Sequence* sequence = ((Sequence*)(create_object(collectiontype, 0)));

    { Object* i = NULL;
      va_list iter_000;
      int iter_000_count = values;

      va_start(iter_000, values);
      while ((iter_000_count--) > 0) {
        i = va_arg(iter_000, Object*);
        if (subtype_ofP(safe_primary_type(sequence), SGT_LISTS_STELLA_LIST)) {
          { Sequence* sequence_000 = sequence;
            List* sequence = ((List*)(sequence_000));

            sequence->insert_last(i);
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "sequence: Don't know how to `insert-last' into a " << "`" << collectiontype << "'";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
      va_end(iter_000);
    }
    return (sequence);
  }
}

Abstract_Dictionary* dictionary(Surrogate* collectiontype, int alternatingkeysandvalues, ...) {
  // Return a dictionary containing `values', in order.
  { Abstract_Dictionary* dictionary = ((Abstract_Dictionary*)(create_object(collectiontype, 0)));
    Object* key = NULL;
    Object* value = NULL;
    Cons* copy = NIL;
    Cons* cursor = NULL;

    { Object* item = NULL;
      va_list iter_000;
      int iter_000_count = alternatingkeysandvalues;
      Cons* collect_000 = NULL;

      va_start(iter_000, alternatingkeysandvalues);
      while ((iter_000_count--) > 0) {
        item = va_arg(iter_000, Object*);
        if (!((boolean)(collect_000))) {
          {
            collect_000 = cons(item, NIL);
            if (copy == NIL) {
              copy = collect_000;
            }
            else {
              add_cons_to_end_of_cons_list(copy, collect_000);
            }
          }
        }
        else {
          {
            collect_000->rest = cons(item, NIL);
            collect_000 = collect_000->rest;
          }
        }
      }
      va_end(iter_000);
    }
    cursor = copy;
    while (!(cursor == NIL)) {
      key = cursor->value;
      value = cursor->rest->value;
      { Surrogate* test_value_000 = safe_primary_type(dictionary);

        if (subtype_ofP(test_value_000, SGT_LISTS_STELLA_HASH_TABLE)) {
          { Abstract_Dictionary* dictionary_000 = dictionary;
            Hash_Table* dictionary = ((Hash_Table*)(dictionary_000));

            dictionary->insert_at(key, value);
          }
        }
        else if (subtype_ofP(test_value_000, SGT_LISTS_STELLA_KEY_VALUE_LIST)) {
          { Abstract_Dictionary* dictionary_001 = dictionary;
            Key_Value_List* dictionary = ((Key_Value_List*)(dictionary_001));

            dictionary->insert_at(key, value);
          }
        }
        else if (subtype_ofP(test_value_000, SGT_LISTS_STELLA_PROPERTY_LIST)) {
          { Abstract_Dictionary* dictionary_002 = dictionary;
            Property_List* dictionary = ((Property_List*)(dictionary_002));

            dictionary->insert_at(key, value);
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "dictionary: Can't create dictionaries of type " << "`" << collectiontype << "'";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
      cursor = cursor->rest->rest;
    }
    return (dictionary);
  }
}

List* List::listify() {
  // Return `self'.
  { List* self = this;

    return (self);
  }
}

List* Cons::listify() {
  // Return a list of elements in `self'.
  { Cons* self = this;

    { List* self_000 = new_list();

      self_000->the_cons_list = self;
      { List* value_000 = self_000;

        return (value_000);
      }
    }
  }
}

List* Key_Value_List::listify() {
  // Return a list of key-value pairs in `self'.
  { Key_Value_List* self = this;

    { List* result = new_list();

      { Object* k = NULL;
        Object* v = NULL;
        Kv_Cons* iter_000 = self->the_kv_list;
        Cons* collect_000 = NULL;

        while (((boolean)(iter_000))) {
          k = iter_000->key;
          v = iter_000->value;
          iter_000 = iter_000->rest;
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(cons(k, cons(v, NIL)), NIL);
              if (result->the_cons_list == NIL) {
                result->the_cons_list = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(result->the_cons_list, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(cons(k, cons(v, NIL)), NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
      return (result);
    }
  }
}

List* Vector::listify() {
  // Return a list of elements in `self'.
  { Vector* self = this;

    { List* result = new_list();

      { Object* v = NULL;
        Vector* vector_000 = self;
        int index_000 = 0;
        int length_000 = vector_000->length();
        Cons* collect_000 = NULL;

        while (index_000 < length_000) {
          v = (vector_000->the_array)[index_000];
          index_000 = index_000 + 1;
          if (!((boolean)(collect_000))) {
            {
              collect_000 = cons(v, NIL);
              if (result->the_cons_list == NIL) {
                result->the_cons_list = collect_000;
              }
              else {
                add_cons_to_end_of_cons_list(result->the_cons_list, collect_000);
              }
            }
          }
          else {
            {
              collect_000->rest = cons(v, NIL);
              collect_000 = collect_000->rest;
            }
          }
        }
      }
      return (result);
    }
  }
}

List* Iterator::listify() {
  // Return a list of elements generated by `self'.
  { Iterator* self = this;

    { List* list = new_list();

      { Object* value = NULL;
        Iterator* iter_000 = self;

        while (iter_000->nextP()) {
          value = iter_000->value;
          list->push(value);
        }
      }
      return (list->reverse());
    }
  }
}

void print_key_value_list(Key_Value_List* self, ostream* stream) {
  *(stream) << "(";
  { Kv_Cons* cursor = self->the_kv_list;
    int position = 1;

    while (((boolean)(cursor))) {
      if (position > 5) {
        *(stream) << " ...";
        break;
      }
      *(stream) << cursor;
      cursor = cursor->rest;
      position = position + 1;
    }
    *(stream) << ")";
  }
}

void print_cons_list(Cons* list, ostream* stream) {
  if (!(FALSE)) {
    *(stream) << "|i|";
  }
  if (list == NIL) {
    *(stream) << "()";
  }
  else {
    { int i = 0;
      int limit = 9;

      *(stream) << "(";
      *(stream) << list->value;
      { Object* item = NULL;
        Cons* iter_000 = list->rest;

        while (!(iter_000 == NIL)) {
          item = iter_000->value;
          iter_000 = iter_000->rest;
          *(stream) << " ";
          *(stream) << item;
          i = i + 1;
          if (i > limit) {
            break;
          }
        }
      }
      if (i <= limit) {
        *(stream) << ")";
      }
      else {
        *(stream) << " ...)";
      }
    }
  }
}

void help_startup_lists1() {
  {
    define_function_object("DEFINED-LIST?", "(DEFUN (DEFINED-LIST? BOOLEAN) ((SELF LIST)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE unless `self' is NULL or the `NIL-LIST'.\")", ((cpp_function_code)(&defined_listP)), NULL);
    define_function_object("NULL-LIST?", "(DEFUN (NULL-LIST? BOOLEAN) ((SELF LIST)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE iff `self' is NULL or the `NIL-LIST'.\")", ((cpp_function_code)(&null_listP)), NULL);
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF LIST)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if the list `self' has no members.\")", ((cpp_method_code)(&List::emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF LIST)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if the list `self' has at least one member.\")", ((cpp_method_code)(&List::non_emptyP)), ((cpp_method_code)(NULL)));
    define_function_object("TERMINATE-LIST?", "(DEFUN (TERMINATE-LIST? BOOLEAN) ((SELF LIST)))", ((cpp_function_code)(&terminate_listP)), NULL);
    define_method_object("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF LIST) (OBJECT (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Return TRUE iff `object' is a member of the list\n`self' (uses an `eql?' test).\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::memberP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (MEMB? BOOLEAN) ((SELF LIST) (OBJECT (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Return TRUE iff `object' is a member of the cons list\n`self' (uses an 'eq?' test).\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::membP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIRST (LIKE (ANY-VALUE SELF))) ((SELF LIST)) :DOCUMENTATION \"Return the first item in the list `self', or NULL if empty.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::first)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SECOND (LIKE (ANY-VALUE SELF))) ((SELF LIST)) :DOCUMENTATION \"Return the second item in the list `self', or NULL if empty.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::second)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (THIRD (LIKE (ANY-VALUE SELF))) ((SELF LIST)) :DOCUMENTATION \"Return the third item in the list `self', or NULL if empty.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::third)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FOURTH (LIKE (ANY-VALUE SELF))) ((SELF LIST)) :DOCUMENTATION \"Return the fourth item in the list `self', or NULL if empty.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::fourth)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIFTH (LIKE (ANY-VALUE SELF))) ((SELF LIST)) :DOCUMENTATION \"Return the fifth item in the list `self', or NULL if empty.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::fifth)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH (LIKE (ANY-VALUE SELF))) ((SELF LIST) (POSITION INTEGER)) :DOCUMENTATION \"Return the nth item in the list `self', or NULL if empty.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::nth)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LAST (LIKE (ANY-VALUE SELF))) ((SELF LIST)) :DOCUMENTATION \"Return the last element of `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::last)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REST (CONS OF (LIKE (ANY-VALUE SELF)))) ((SELF LIST)) :DOCUMENTATION \"Return a cons list of all but the first item in the list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::rest)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIRST-SETTER (LIKE (ANY-VALUE SELF))) ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&List::first_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SECOND-SETTER (LIKE (ANY-VALUE SELF))) ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&List::second_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (THIRD-SETTER (LIKE (ANY-VALUE SELF))) ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&List::third_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FOURTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&List::fourth_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (FIFTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&List::fifth_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF))) (POSITION INTEGER)) :PUBLIC? TRUE)", ((cpp_method_code)(&List::nth_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&List::length)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (POSITION INTEGER) ((SELF LIST) (OBJECT OBJECT) (START INTEGER)) :DOCUMENTATION \"Return the position of `object' within the list\n`self' (counting from zero); or return NULL if `object' does not occur within \n`self' (uses an `eql?' test).  If `start' was supplied as non-NULL, only \nconsider the sublist starting at `start', however, the returned position \nwill always be relative to the entire list.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::position)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Add `value' to the front of the list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::insert)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD PUSH ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Add `value' to the front of the list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::push)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-NEW ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Add `value' to the front of the list `self' unless its \nalready a member.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::insert_new)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-LAST ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Insert `value' as the last entry in the list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::insert_last)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE (LIKE SELF)) ((SELF LIST) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Destructively remove all entries in `self' that match `value'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::remove)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE-IF (LIKE SELF)) ((SELF LIST) (TEST? FUNCTION-CODE)) :DOCUMENTATION \"Destructively remove all members of the list `self' for which\n'test?' evaluates to TRUE.  `test' takes a single argument of type OBJECT and\nreturns TRUE or FALSE.  Returns `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::remove_if)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE-DUPLICATES (LIKE SELF)) ((SELF LIST)) :DOCUMENTATION \"Destructively remove duplicates from `self' and return the result.\nPreserves the original order of the remaining members.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::remove_duplicates)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (POP (LIKE (ANY-VALUE SELF))) ((SELF LIST)) :DOCUMENTATION \"Remove and return the first element in the list `self'.\nReturn NULL if the list is empty.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::pop)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REVERSE (LIKE SELF)) ((SELF LIST)) :DOCUMENTATION \"Reverse the members of `self' (in place).\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::reverse)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SUBSTITUTE (LIKE SELF)) ((SELF LIST) (INVALUE OBJECT) (OUTVALUE OBJECT)) :DOCUMENTATION \"Destructively replace each appearance of `outValue' by\n`inValue' in the list `self'.\")", ((cpp_method_code)(&List::substitute)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (CONCATENATE LIST) ((LIST1 LIST) (LIST2 LIST) |&REST| (OTHERLISTS LIST)) :DOCUMENTATION \"Copy `list2' and all `otherLists' onto the end of `list1'.\nThe operation is destructive wrt `list1', but leaves all other lists intact.\nThe two mandatory parameters allow us to optimize the common binary case by\nnot relying on the somewhat less efficient variable arguments mechanism.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::concatenate)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (PREPEND (LIKE SELF)) ((SELF LIST) (LIST2 LIST)) :DOCUMENTATION \"Copy `list2' onto the front of the list `self'.\nThe operation is destructive wrt `self', but leaves `list2' intact.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::prepend)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY (LIST OF (LIKE (ANY-VALUE SELF)))) ((SELF LIST)) :DOCUMENTATION \"Return a copy of the list `self'.  The conses in the copy are\nfreshly allocated.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::copy)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD CLEAR ((SELF LIST)) :DOCUMENTATION \"Make `self' an empty list.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::clear)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SORT (LIST OF (LIKE (ANY-VALUE SELF)))) ((SELF LIST) (PREDICATE FUNCTION-CODE)) :PUBLIC? TRUE :DOCUMENTATION \"Perform a stable, destructive sort of `self' according to\n`predicate', and return the result.  If `predicate' has a '<' semantics, the\nresult will be in ascending order.  If `predicate' is NULL, a suitable\n'<' predicate is chosen depending on the first element of `self', and it \nis assumed that all elements of `self' have the same type (supported\nelement types are GENERALIZED-SYMBOL, STRING, INTEGER, and FLOAT).\")", ((cpp_method_code)(&List::sort)), ((cpp_method_code)(NULL)));
    define_function_object("MAP-NULL-TO-NIL-LIST", "(DEFUN (MAP-NULL-TO-NIL-LIST LIST) ((SELF LIST)) :DOCUMENTATION \"Return NIL-LIST iff `self' is NULL or `self' otherwise.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (CHOOSE (NULL? SELF) NIL-LIST SELF)))", ((cpp_function_code)(&map_null_to_nil_list)), NULL);
    define_method_object("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X LIST) (Y OBJECT)) :DOCUMENTATION \"Return TRUE iff the lists `x' and `y' are structurally\nequivalent.  Uses `equal?' to test equality of elements.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::object_equalP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ALLOCATE-ITERATOR (LIST-ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF LIST)))", ((cpp_method_code)(&List::allocate_iterator)), ((cpp_method_code)(NULL)));
    define_function_object("TERMINATE-LIST-ITERATOR?", "(DEFUN (TERMINATE-LIST-ITERATOR? BOOLEAN) ((SELF LIST-ITERATOR)))", ((cpp_function_code)(&terminate_list_iteratorP)), NULL);
    define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF LIST-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&List_Iterator::nextP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF LIST-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&List_Iterator::emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (VALUE-SETTER OBJECT) ((SELF LIST-ITERATOR) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&List_Iterator::value_setter)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (BUT-LAST (ITERATOR OF (LIKE (ANY-VALUE SELF)))) ((SELF LIST)) :DOCUMENTATION \"Generate all but the last element of the list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::but_last)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE-DELETED-MEMBERS (LIKE SELF)) ((SELF LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&List::remove_deleted_members)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INITIALIZE-OBJECT ((SELF PROPERTY-LIST)))", ((cpp_method_code)(&Property_List::initialize_object)), ((cpp_method_code)(NULL)));
    define_function_object("SEARCH-PLIST", "(DEFUN (SEARCH-PLIST OBJECT) ((PLIST CONS) (KEY OBJECT)))", ((cpp_function_code)(&search_plist)), NULL);
    define_method_object("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF PROPERTY-LIST) (KEY (LIKE (ANY-KEY SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Property_List::lookup)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF PROPERTY-LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&Property_List::length)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-AT ((SELF PROPERTY-LIST) (KEY (LIKE (ANY-KEY SELF))) (VALUE (LIKE (ANY-VALUE SELF)))) :DOCUMENTATION \"Insert the entry <`key', `value'> into the property list `self'.\nIf a previous entry existed with key `key', that entry is replaced.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Property_List::insert_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REMOVE-AT OBJECT) ((SELF PROPERTY-LIST) (KEY (LIKE (ANY-KEY SELF)))) :DOCUMENTATION \"Remove the entry that matches the key `key'.  Return the\nvalue of the matching entry, or NULL if there is no matching entry.  Assumes that at\nmost one entry matches `key'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Property_List::remove_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF PROPERTY-LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&Property_List::emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF PROPERTY-LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&Property_List::non_emptyP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (COPY (LIKE SELF)) ((SELF PROPERTY-LIST)) :DOCUMENTATION \"Return a copy of the list `self'.  The conses in the copy are\nfreshly allocated.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Property_List::copy)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD CLEAR ((SELF PROPERTY-LIST)) :DOCUMENTATION \"Make `self' an empty property list.\")", ((cpp_method_code)(&Property_List::clear)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X PROPERTY-LIST) (Y OBJECT)) :DOCUMENTATION \"Return TRUE if `x' and `y' represent the same set of key/value pairs..\" :PUBLIC? TRUE)", ((cpp_method_code)(&Property_List::object_equalP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (EQUIVALENT-SETS? BOOLEAN) ((SELF LIST) (OTHERLIST LIST)))", ((cpp_method_code)(&List::equivalent_setsP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (INTERSECTION LIST) ((SELF LIST) (OTHERLIST LIST)))", ((cpp_method_code)(&List::intersection)), ((cpp_method_code)(NULL)));
  }
}

void startup_lists() {
  if (current_startup_time_phaseP(0)) {
    if (!(((boolean)(NIL_LIST)))) {
      NIL_LIST = new List();
    }
    NIL_LIST->the_cons_list = NIL;
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      SGT_LISTS_STELLA_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("LIST", NULL, 1)));
      SGT_LISTS_STELLA_PROPERTY_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("PROPERTY-LIST", NULL, 1)));
      SGT_LISTS_STELLA_KEY_VALUE_LIST = ((Surrogate*)(intern_rigid_symbol_wrt_module("KEY-VALUE-LIST", NULL, 1)));
      SGT_LISTS_STELLA_HASH_TABLE = ((Surrogate*)(intern_rigid_symbol_wrt_module("HASH-TABLE", NULL, 1)));
      SYM_LISTS_STELLA_STARTUP_LISTS = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-LISTS", NULL, 0)));
      SYM_LISTS_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_lists1();
      define_method_object("(DEFMETHOD (UNION LIST) ((SELF LIST) (OTHERLIST LIST)))", ((cpp_method_code)(&List::renamed_Union)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (DIFFERENCE LIST) ((SELF LIST) (OTHERLIST LIST)))", ((cpp_method_code)(&List::difference)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (SUBTRACT LIST) ((SELF LIST) (OTHERLIST LIST)))", ((cpp_method_code)(&List::subtract)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (ALLOCATE-ITERATOR (PROPERTY-LIST-ITERATOR OF (LIKE (ANY-KEY SELF)) (LIKE (ANY-VALUE SELF)))) ((SELF PROPERTY-LIST)))", ((cpp_method_code)(&Property_List::allocate_iterator)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF PROPERTY-LIST-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Property_List_Iterator::nextP)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (VALUE-SETTER (LIKE (ANY-VALUE SELF))) ((SELF PROPERTY-LIST-ITERATOR) (VALUE (LIKE (ANY-VALUE SELF)))))", ((cpp_method_code)(&Property_List_Iterator::value_setter)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (KEY-SETTER (LIKE (ANY-KEY SELF))) ((SELF PROPERTY-LIST-ITERATOR) (KEY (LIKE (ANY-KEY SELF)))))", ((cpp_method_code)(&Property_List_Iterator::key_setter)), ((cpp_method_code)(NULL)));
      define_function_object("KV-CONS", "(DEFUN (KV-CONS KV-CONS) ((KEY OBJECT) (VALUE OBJECT) (REST KV-CONS)) :DOCUMENTATION \"Create, fill-in, and return a new KV-CONS.\")", ((cpp_function_code)(&kv_cons)), NULL);
      define_function_object("FREE-KV-CONS", "(DEFUN FREE-KV-CONS ((KVCONS KV-CONS)))", ((cpp_function_code)(&free_kv_cons)), NULL);
      define_method_object("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF KEY-VALUE-LIST) (KEY (LIKE (ANY-KEY SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::lookup)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF KEY-VALUE-LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::length)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD INSERT-AT ((SELF KEY-VALUE-LIST) (KEY (LIKE (ANY-KEY SELF))) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Insert the entry <`key', `value'> into the association\n`self'. If a previous entry existed with key `key', that entry is replaced.\")", ((cpp_method_code)(&Key_Value_List::insert_at)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (REMOVE-AT OBJECT) ((SELF KEY-VALUE-LIST) (KEY (LIKE (ANY-KEY SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Remove the entry that matches the key `key'.\nReturn the value of the matching entry, or NULL if there is no matching entry.\nAssumes that at most one entry matches `key'.\")", ((cpp_method_code)(&Key_Value_List::remove_at)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD INSERT-ENTRY ((SELF KEY-VALUE-LIST) (KEY (LIKE (ANY-KEY SELF))) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Insert an entry <`key',`value'> to `self' unless an identical\nentry already exists.  This can generate duplicate entries for `key'.\")", ((cpp_method_code)(&Key_Value_List::insert_entry)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD REMOVE-ENTRY ((SELF KEY-VALUE-LIST) (KEY (LIKE (ANY-KEY SELF))) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE :DOCUMENTATION \"Remove the entry that matches <`key',`value'>.\nAssumes that more than one entry can match `key'.\")", ((cpp_method_code)(&Key_Value_List::remove_entry)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (POP (LIKE (ANY-VALUE SELF))) ((SELF KEY-VALUE-LIST)) :DOCUMENTATION \"Remove and return the first element of the kv-list `self'.\nReturn NULL if the list is empty.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::pop)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (REVERSE (LIKE SELF)) ((SELF KEY-VALUE-LIST)) :DOCUMENTATION \"Destructively reverse the members of the list `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::reverse)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (EMPTY? BOOLEAN) ((SELF KEY-VALUE-LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::emptyP)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (NON-EMPTY? BOOLEAN) ((SELF KEY-VALUE-LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::non_emptyP)), ((cpp_method_code)(NULL)));
      define_function_object("COPY-KV-CONS-LIST", "(DEFUN (COPY-KV-CONS-LIST KV-CONS) ((KVCONSLIST KV-CONS)) :DOCUMENTATION \"Return a copy of the cons list `consList'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&copy_kv_cons_list)), NULL);
      define_method_object("(DEFMETHOD (COPY (LIKE SELF)) ((SELF KEY-VALUE-LIST)) :DOCUMENTATION \"Return a copy of the kv-list `self'.  The kv-conses in\nthe copy are freshly allocated.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::copy)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD CLEAR ((SELF KEY-VALUE-LIST)) :DOCUMENTATION \"Make `self' an empty dictionary.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::clear)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X KEY-VALUE-LIST) (Y OBJECT)) :DOCUMENTATION \"Return TRUE if `x' and `y' represent the same set of key/value pairs.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::object_equalP)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (ALLOCATE-ITERATOR (KV-LIST-ITERATOR OF (LIKE (ANY-KEY SELF)) (LIKE (ANY-VALUE SELF)))) ((SELF KEY-VALUE-LIST)) :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::allocate_iterator)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (NEXT? BOOLEAN) ((SELF KV-LIST-ITERATOR)) :PUBLIC? TRUE)", ((cpp_method_code)(&Kv_List_Iterator::nextP)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (VALUE-SETTER (LIKE (ANY-VALUE SELF))) ((SELF KV-LIST-ITERATOR) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Kv_List_Iterator::value_setter)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (KEY-SETTER OBJECT) ((SELF KV-LIST-ITERATOR) (KEY OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Kv_List_Iterator::key_setter)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF SEQUENCE) (VALUE OBJECT)) :DOCUMENTATION \"Return TRUE if `value' is a member of the sequence `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Sequence::memberP)), ((cpp_method_code)(NULL)));
      define_function_object("LIST", "(DEFUN (LIST LIST) (|&REST| (VALUES OBJECT)) :DOCUMENTATION \"Return a list containing `values', in order.\" :PUBLIC? TRUE)", ((cpp_function_code)(&list)), NULL);
      define_function_object("SEQUENCE", "(DEFUN (SEQUENCE (SEQUENCE OF OBJECT)) ((COLLECTIONTYPE TYPE) |&REST| (VALUES OBJECT)) :DOCUMENTATION \"Return a sequence containing `values', in order.\" :PUBLIC? TRUE)", ((cpp_function_code)(&sequence)), NULL);
      define_function_object("DICTIONARY", "(DEFUN (DICTIONARY (ABSTRACT-DICTIONARY OF OBJECT OBJECT)) ((COLLECTIONTYPE TYPE) |&REST| (ALTERNATINGKEYSANDVALUES OBJECT)) :DOCUMENTATION \"Return a dictionary containing `values', in order.\" :PUBLIC? TRUE)", ((cpp_function_code)(&dictionary)), NULL);
      define_method_object("(DEFMETHOD (LISTIFY (LIST OF (LIKE (ANY-VALUE SELF)))) ((SELF LIST)) :DOCUMENTATION \"Return `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&List::listify)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (LISTIFY (LIST OF (LIKE (ANY-VALUE SELF)))) ((SELF CONS)) :DOCUMENTATION \"Return a list of elements in `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Cons::listify)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (LISTIFY (LIST OF (LIKE (ANY-VALUE SELF)))) ((SELF KEY-VALUE-LIST)) :DOCUMENTATION \"Return a list of key-value pairs in `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Key_Value_List::listify)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (LISTIFY (LIST OF (LIKE (ANY-VALUE SELF)))) ((SELF VECTOR)) :DOCUMENTATION \"Return a list of elements in `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Vector::listify)), ((cpp_method_code)(NULL)));
      define_method_object("(DEFMETHOD (LISTIFY (LIST OF (LIKE (ANY-VALUE SELF)))) ((SELF ITERATOR)) :DOCUMENTATION \"Return a list of elements generated by `self'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Iterator::listify)), ((cpp_method_code)(NULL)));
      define_function_object("PRINT-KEY-VALUE-LIST", "(DEFUN PRINT-KEY-VALUE-LIST ((SELF KEY-VALUE-LIST) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_key_value_list)), NULL);
      define_function_object("PRINT-CONS-LIST", "(DEFUN PRINT-CONS-LIST ((LIST CONS) (STREAM NATIVE-OUTPUT-STREAM)))", ((cpp_function_code)(&print_cons_list)), NULL);
      define_function_object("STARTUP-LISTS", "(DEFUN STARTUP-LISTS () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_lists)), NULL);
      { Method_Slot* function = lookup_function(SYM_LISTS_STELLA_STARTUP_LISTS);

        set_dynamic_slot_value(function->dynamic_slots, SYM_LISTS_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Lists"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL NIL-LIST LIST NULL :PUBLIC? TRUE)");
    }
  }
}

Surrogate* SGT_LISTS_STELLA_LIST = NULL;

Surrogate* SGT_LISTS_STELLA_PROPERTY_LIST = NULL;

Surrogate* SGT_LISTS_STELLA_KEY_VALUE_LIST = NULL;

Surrogate* SGT_LISTS_STELLA_HASH_TABLE = NULL;

Symbol* SYM_LISTS_STELLA_STARTUP_LISTS = NULL;

Symbol* SYM_LISTS_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
