// primal.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

boolean TRUE = 1;

boolean FALSE = 0;

double PI = 3.141592653589793;

int NULL_INTEGER = 1 << (sizeof (int) / sizeof (char) * 8 - 1);

short int NULL_SHORT_INTEGER = 1 << (sizeof (short int) / sizeof (char) * 8 - 1);

long int NULL_LONG_INTEGER = 1 << (sizeof (long int) / sizeof (char) * 8 - 1);

unsigned short int NULL_UNSIGNED_SHORT_INTEGER = (unsigned short int) -1;

unsigned long int NULL_UNSIGNED_LONG_INTEGER = (unsigned long int) -1;

double NULL_FLOAT = -HUGE_VAL /* IEEE infinity, defined in <math.h> */;

float NULL_SINGLE_FLOAT = -HUGE_VAL /* IEEE infinity, defined in <math.h> */;

double NULL_DOUBLE_FLOAT = -HUGE_VAL /* IEEE infinity, defined in <math.h> */;

char NULL_CHARACTER = '\0';

char NULL_BYTE = 255;

char NULL_OCTET = 255;

boolean eqlP(Object* x, Object* y) {
  // Return `true' if `x' and `y' are `eq?' or equivalent literals
  // such as strings that also might be wrapped in non-identical wrappers.  For
  // the case where `x' or `y' are plain literals such as strings or integers, the
  // STELLA translator substitutes the equality test appropriate for the particular
  // target language and does not actually call this function.  For cases where
  // `x' or `y' are known to be of type STANDARD-OBJECT, the STELLA translator
  // substitutes the faster `eq?' test inline.
  return (((!((boolean)(x))) ? (!((boolean)(y))) : ((x == y) ||
      x->object_eqlP(y))));
}

boolean equalP(Object* x, Object* y) {
  // Return `true' if `x' and `y' are `eql?' or considered equal
  // by a user-defined `object-equal?' method.  This implements a fully extensible
  // equality test similar to Java's `equals' method.
  return (((!((boolean)(x))) ? (!((boolean)(y))) : ((x == y) ||
      x->object_equalP(y))));
}

boolean Object::object_equalP(Object* y) {
  // Return `true' if `x' and `y' are `eq?'.
  { Object* x = this;

    return (x == y);
  }
}

boolean Wrapper::object_equalP(Object* y) {
  // Return `true' if `x' and `y' are literal wrappers whose
  // literals are considered `eql?'.
  { Wrapper* x = this;

    return (x->object_eqlP(y));
  }
}

boolean zeroP(int x) {
  // Return `true' if `x' is 0.
  return ((!x));
}

boolean plusP(int x) {
  // Return `true' if `x' is greater than 0.
  return ((x > 0));
}

boolean evenP(int x) {
  // Return `true' if `x' is an even number.
  return (!(x % 2));
}

boolean oddP(int x) {
  // Return `true' if `x' is an odd number.
  return ((x % 2));
}

int mod(int x, int modulo) {
  // Return the result of `x' mod `modulo'.
  return ((x % modulo));
}

int random_(int n) {
  // Generate a random integer in the interval [0..n-1].
  // `n' must be <= 2^15.
  if (n > 32768) {
    throw *new_stella_exception("random: Can only generate random numbers between 0 and (2^15)-1.");
  }
  { int rnum = abs(rand()) % n;

    return (rnum);
  }
}

void seed_random_number_generator() {
  // Seeds the random number generator with the current time
  srand(time(NULL));;
}

double sqrt_(double n) {
  // Return the square root of `n'.
  return (sqrt(n));
}

double cos_(double n) {
  // Return the cosine of `n' radians.
  return (cos(n));
}

double sin_(double n) {
  // Return the sine of `n' radians.
  return (sin(n));
}

double tan_(double n) {
  // Return the tangent of `n' radians.
  return (tan(n));
}

double log_(double n) {
  // Return the natural logarithm (base e) of `n'.
  return (log(n));
}

int min_(int x, int y) {
  // Return the minimum of `x' and `y'.  If either is NULL, return the other.
  if (x == NULL_INTEGER) {
    return (y);
  }
  if (y == NULL_INTEGER) {
    return (x);
  }
  return (((x < y) ? x : y));
}

int max_(int x, int y) {
  // Return the maximum of `x' and `y'.  If either is NULL, return the other.
  if (x == NULL_INTEGER) {
    return (y);
  }
  if (y == NULL_INTEGER) {
    return (x);
  }
  return (((x > y) ? x : y));
}

int integer_abs(int x) {
  // Return the absolute value of `x'.
  return (((x < 0) ? (0 - x) : x));
}

double float_abs(double x) {
  // Return the absolute value of `x'.
  return (((x < 0.0) ? (0.0 - x) : x));
}

int character_code(char character) {
  // Return the 8-bit ASCII code of `character' as an integer.
  return ((int)(unsigned char) character);
}

char code_character(int code) {
  // Return the character encoded by `code' (0 <= `code' <= 255).
  return ((char) code);
}

char character_downcase(char c) {
  // Return the lowercase character for `c'.
  return ((isupper(c) ? c + 'a' - 'A' : c));
}

char character_upcase(char c) {
  // Return the uppercase character for `c'.  If only the first character of
  // a sequence of characters is to be capitalized, character-capitalize should be used.
  return ((islower(c) ? c + 'A' - 'a' : c));
}

char character_capitalize(char c) {
  // Return the capitalized character for `c'.  This is generally the same
  // as the uppercase character, except for obscure non-English characters in Java.  It should
  // be used if only the first character of a sequence of characters is to be capitalized.
  return ((islower(c) ? c + 'A' - 'a' : c));
}

DEFINE_STELLA_SPECIAL(oTRANSIENTOBJECTSpo, boolean , FALSE);

boolean string_eqlP(char* x, char* y) {
  // Return true if `x' and `y' are equal strings including case for both null.
  if (x == NULL) {
    return (y == NULL);
  }
  else {
    return ((y != NULL) &&
        !strcmp(x, y));
  }
}

boolean string_equalP(char* x, char* y) {
  // Return true if `x' and `y' are equal strings ignoring case for both null.
  if (x == NULL) {
    return (y == NULL);
  }
  else {
    return ((y != NULL) &&
        !strcasecmp(x, y));
  }
}

int string_compare(char* x, char* y, boolean case_sensitiveP) {
  // Compare `x' and `y' lexicographically, and return -1, 0, 
  // or 1, depending on whether `x' is less than, equal, or greater than `y'.
  // If `case-sensitive?' is `true', then case does matter for the comparison
  if (case_sensitiveP) {
    return (strcmp(x, y));
  }
  else {
    return (strcasecmp(x, y));
  }
}

boolean stringL(char* x, char* y) {
  // Return TRUE if `x' is lexicographically < `y', considering case.
  return (string_compare(x, y, TRUE) < 0);
}

boolean stringLE(char* x, char* y) {
  // Return TRUE if `x' is lexicographically <= `y', considering case..
  return (string_compare(x, y, TRUE) <= 0);
}

boolean stringGE(char* x, char* y) {
  // Return TRUE if `x' is lexicographically >= `y', considering case..
  return (string_compare(x, y, TRUE) >= 0);
}

boolean stringG(char* x, char* y) {
  // Return TRUE if `x' is lexicographically > `y', considering case..
  return (string_compare(x, y, TRUE) > 0);
}

boolean string_lessP(char* x, char* y) {
  // Return TRUE if `x' is lexicographically < `y', ignoring case.
  return (string_compare(x, y, FALSE) < 0);
}

boolean string_less_equalP(char* x, char* y) {
  // Return TRUE if `x' is lexicographically <= `y', ignoring case.
  return (string_compare(x, y, FALSE) <= 0);
}

boolean string_greater_equalP(char* x, char* y) {
  // Return TRUE if `x' is lexicographically >= `y', ignoring case.
  return (string_compare(x, y, FALSE) >= 0);
}

boolean string_greaterP(char* x, char* y) {
  // Return TRUE if `x' is lexicographically > `y', ignoring case.
  return (string_compare(x, y, FALSE) > 0);
}

char* make_mutable_string(int size, char initchar) {
  // Return a new mutable string filled with `size' `initchar's.
  return (make_string(size, initchar));
}

char* make_raw_mutable_string(int size) {
  // Return a new uninitialized mutable string of `size'.
  { char* s = NULL;

    s = new (GC) char[size];
    return (s);
  }
}

char* string_concatenate(char* string1, char* string2, int otherstrings, ...) {
  // Return a new string representing the concatenation
  // of `string1', `string2', and `otherStrings'.  The two mandatory parameters
  // allow us to optimize the common binary case by not relying on the somewhat
  // less efficient variable arguments mechanism.
  if (otherstrings == 0) {
    return (string_concatenate(string1, string2));
  }
  { char* result = NULL;
    int length = strlen(string1) + strlen(string2);
    int index = 0;

    { char* string = NULL;
      va_list iter_000;
      int iter_000_count = otherstrings;

      va_start(iter_000, otherstrings);
      while ((iter_000_count--) > 0) {
        string = va_arg(iter_000, char*);
        length = length + strlen(string);
      }
      va_end(iter_000);
    }
    result = make_raw_mutable_string(length);
    { char ch = NULL_CHARACTER;
      char* vector_000 = string1;
      int index_000 = 0;
      int length_000 = strlen(vector_000);

      while (index_000 < length_000) {
        ch = vector_000[index_000];
        index_000 = index_000 + 1;
        result[index] = ch;
        index = index + 1;
      }
    }
    { char ch = NULL_CHARACTER;
      char* vector_001 = string2;
      int index_001 = 0;
      int length_001 = strlen(vector_001);

      while (index_001 < length_001) {
        ch = vector_001[index_001];
        index_001 = index_001 + 1;
        result[index] = ch;
        index = index + 1;
      }
    }
    { char* string = NULL;
      va_list iter_001;
      int iter_001_count = otherstrings;

      va_start(iter_001, otherstrings);
      while ((iter_001_count--) > 0) {
        string = va_arg(iter_001, char*);
        { char ch = NULL_CHARACTER;
          char* vector_002 = string;
          int index_002 = 0;
          int length_002 = strlen(vector_002);

          while (index_002 < length_002) {
            ch = vector_002[index_002];
            index_002 = index_002 + 1;
            result[index] = ch;
            index = index + 1;
          }
        }
      }
      va_end(iter_001);
    }
    return (result);
  }
}

char* replace_substrings(char* string, char* renamed_New, char* old) {
  // Replace all occurrences of `old' in `string' with `new'.
  { int stringlength = strlen(string);
    int oldlength = strlen(old);
    int newlength = strlen(renamed_New);
    int nofoccurrences = 0;
    int oldstart = 0;
    int cursor = 0;
    int resultcursor = 0;
    char* result = NULL;

    while ((oldstart = string_search(string, old, cursor)) != NULL_INTEGER) {
      nofoccurrences = nofoccurrences + 1;
      cursor = oldstart + oldlength;
    }
    if (nofoccurrences == 0) {
      return (string);
    }
    result = make_raw_mutable_string(stringlength + (nofoccurrences * (newlength - oldlength)));
    cursor = 0;
    while ((oldstart = string_search(string, old, cursor)) != NULL_INTEGER) {
      { int i = NULL_INTEGER;
        int iter_000 = cursor;
        int upper_bound_000 = oldstart - 1;

        while (iter_000 <= upper_bound_000) {
          i = iter_000;
          iter_000 = iter_000 + 1;
          result[resultcursor] = (string[i]);
          resultcursor = resultcursor + 1;
        }
      }
      { char renamed_Char = NULL_CHARACTER;
        char* vector_000 = renamed_New;
        int index_000 = 0;
        int length_000 = strlen(vector_000);

        while (index_000 < length_000) {
          renamed_Char = vector_000[index_000];
          index_000 = index_000 + 1;
          result[resultcursor] = renamed_Char;
          resultcursor = resultcursor + 1;
        }
      }
      cursor = oldstart + oldlength;
    }
    { int i = NULL_INTEGER;
      int iter_001 = cursor;
      int upper_bound_001 = stringlength - 1;

      while (iter_001 <= upper_bound_001) {
        i = iter_001;
        iter_001 = iter_001 + 1;
        result[resultcursor] = (string[i]);
        resultcursor = resultcursor + 1;
      }
    }
    return (result);
  }
}

int insert_string(char* source, int start, int end, char* target, int target_index, Keyword* case_conversion) {
  // Inserts characters from `source' begining at `start' and
  // ending at `end' into `target' starting at `target-index'.  If `end' is `null',
  // then the entire length of the string is used. The copy of characters is affected
  // by the `case-conversion' keyword which should be one of
  //    :UPCASE :DOWNCASE :CAPITALIZE :PRESERVE.
  // 
  // The final value of target-index is returned.
  if (end == NULL_INTEGER) {
    end = strlen(source) - 1;
  }
  if (end < start) {
    return (target_index);
  }
  if (case_conversion == KWD_PRIMAL_UPCASE) {
    { int j = NULL_INTEGER;
      int iter_000 = start;
      int upper_bound_000 = end;
      boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

      while (unboundedP_000 ||
          (iter_000 <= upper_bound_000)) {
        j = iter_000;
        iter_000 = iter_000 + 1;
        target[target_index] = (oCHARACTER_UPCASE_TABLEo[((int)(unsigned char) (source[j]))]);
        target_index = target_index + 1;
      }
    }
  }
  else if (case_conversion == KWD_PRIMAL_DOWNCASE) {
    { int j = NULL_INTEGER;
      int iter_001 = start;
      int upper_bound_001 = end;
      boolean unboundedP_001 = upper_bound_001 == NULL_INTEGER;

      while (unboundedP_001 ||
          (iter_001 <= upper_bound_001)) {
        j = iter_001;
        iter_001 = iter_001 + 1;
        target[target_index] = (oCHARACTER_DOWNCASE_TABLEo[((int)(unsigned char) (source[j]))]);
        target_index = target_index + 1;
      }
    }
  }
  else if (case_conversion == KWD_PRIMAL_CAPITALIZE) {
    target[target_index] = (oCHARACTER_UPCASE_TABLEo[((int)(unsigned char) (source[start]))]);
    target_index = target_index + 1;
    { int j = NULL_INTEGER;
      int iter_002 = start + 1;
      int upper_bound_002 = end;
      boolean unboundedP_002 = upper_bound_002 == NULL_INTEGER;

      while (unboundedP_002 ||
          (iter_002 <= upper_bound_002)) {
        j = iter_002;
        iter_002 = iter_002 + 1;
        target[target_index] = (oCHARACTER_DOWNCASE_TABLEo[((int)(unsigned char) (source[j]))]);
        target_index = target_index + 1;
      }
    }
  }
  else if (case_conversion == KWD_PRIMAL_PRESERVE) {
    { int j = NULL_INTEGER;
      int iter_003 = start;
      int upper_bound_003 = end;
      boolean unboundedP_003 = upper_bound_003 == NULL_INTEGER;

      while (unboundedP_003 ||
          (iter_003 <= upper_bound_003)) {
        j = iter_003;
        iter_003 = iter_003 + 1;
        target[target_index] = (source[j]);
        target_index = target_index + 1;
      }
    }
  }
  else {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "`" << case_conversion << "'" << " is not a valid case option";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  return (target_index);
}

char* character_to_string(char c) {
  return (make_string(1, c));
}

char* format_with_padding(char* input, int length, char padchar, Keyword* align, boolean truncateP) {
  // Formats `input' to be (at least) `length' long, using `padchar' to
  // fill if necessary.  `align' must be one of :LEFT, :RIGHT, :CENTER and will control
  // how `input' will be justified in the resulting string.  If `truncate?' is `true', then
  // then an overlength string will be truncated, using the opposite of `align' to pick
  // the truncation direction.
  { int len = strlen(input);

    if (len == length) {
      return (input);
    }
    else if (len > length) {
      if (truncateP) {
        if (align == KWD_PRIMAL_LEFT) {
          return (string_subsequence(input, len - length, len));
        }
        else if (align == KWD_PRIMAL_RIGHT) {
          return (string_subsequence(input, 0, len - length));
        }
        else if (align == KWD_PRIMAL_CENTER) {
          { int left = ((int)((len - length) / 2.0));

            return (string_subsequence(input, left, left + length));
          }
        }
        else {
          { Output_String_Stream* stream_000 = new_output_string_stream();

            *(stream_000->native_stream) << "`" << align << "'" << " is not a valid case option";
            throw *new_stella_exception(stream_000->the_string_reader());
          }
        }
      }
      else {
        return (input);
      }
    }
    else {
      if (align == KWD_PRIMAL_LEFT) {
        return (string_concatenate(input, make_string(length - len, padchar), 0));
      }
      else if (align == KWD_PRIMAL_RIGHT) {
        return (string_concatenate(make_string(length - len, padchar), input, 0));
      }
      else if (align == KWD_PRIMAL_CENTER) {
        { int left = ((int)((length - len) / 2.0));

          return (string_concatenate(make_string(left, padchar), input, 1, make_string((length - len) - left, padchar)));
        }
      }
      else {
        { Output_String_Stream* stream_001 = new_output_string_stream();

          *(stream_001->native_stream) << "`" << align << "'" << " is not a valid case option";
          throw *new_stella_exception(stream_001->the_string_reader());
        }
      }
    }
  }
}

char string_nth(char* self, int position) {
  // Return the character in `self' at `position'.
  return (self[position]);
}

char mutable_string_nth(char* self, int position) {
  // Return the character in `self' at `position'.
  return (self[position]);
}

char mutable_string_nth_setter(char* self, char ch, int position) {
  // Set the character in `self' at `position' to `ch'.
  return (self[position] = ch);
}

int string_length(char* self) {
  // Return the length of the string `self'.
  return (strlen(self));
}

int mutable_string_length(char* self) {
  // Return the length of the string `self'.
  return (strlen(self));
}

Object* unstringify(char* string) {
  return (read_s_expression_from_string(string));
}

# include "stella/cpp-lib/cpp-hashtable.hh"
void Abstract_Hash_Table::free_hash_table_values() {
  // Call free on each value in the hash table `self'.
  { Abstract_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      cout << "WARNING: free_hash_table_values not yet implemented" << endl;;
    }
  }
}

void Abstract_Hash_Table::initialize_hash_table() {
  // Insert a newly-created native hash table into `self'.
  { Abstract_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->the_stella_hash_table = make_stella_hash_table(50);
      return;
    }
    self->the_hash_table = (cpp_hash_table*)(new Native_EQL_Hash_Table<Object*, Object*>);
  }
}

Object* Hash_Table::lookup(Object* key) {
  { Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (stella_hash_table_lookup(self->the_stella_hash_table, key));
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      return (((Native_EQL_Hash_Table<Object*, Object*>*)native_table)->get(key));
    }
  }
}

void Hash_Table::insert_at(Object* key, Object* value) {
  { Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_insert_at(self->the_stella_hash_table, key, value);
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      (*((Native_EQL_Hash_Table<Object*, Object*>*)native_table))[key] = value;;
    }
  }
}

void Hash_Table::remove_at(Object* key) {
  { Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_remove_at(self->the_stella_hash_table, key);
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      ((Native_EQL_Hash_Table<Object*, Object*>*)native_table)->erase(key);;
    }
  }
}

void Integer_Hash_Table::initialize_hash_table() {
  // Insert a newly-created native hash table into `self'.
  { Integer_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->the_stella_hash_table = make_stella_hash_table(50);
      return;
    }
    self->the_hash_table = ((cpp_hash_table*)(new Native_EQL_Hash_Table<int, Object*>));
  }
}

Object* Integer_Hash_Table::lookup(int key) {
  { Integer_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (stella_hash_table_lookup(self->the_stella_hash_table, wrap_integer(key)));
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      return (((Native_EQL_Hash_Table<int, Object*>*)native_table)->get(key));
    }
  }
}

void Integer_Hash_Table::insert_at(int key, Object* value) {
  { Integer_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_insert_at(self->the_stella_hash_table, wrap_integer(key), value);
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      (*((Native_EQL_Hash_Table<int, Object*>*)native_table))[key] = value;;
    }
  }
}

void Integer_Hash_Table::remove_at(int key) {
  { Integer_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_remove_at(self->the_stella_hash_table, wrap_integer(key));
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      ((Native_EQL_Hash_Table<int, Object*>*)native_table)->erase(key);;
    }
  }
}

void Float_Hash_Table::initialize_hash_table() {
  // Insert a newly-created native hash table into `self'.
  { Float_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->the_stella_hash_table = make_stella_hash_table(50);
      return;
    }
    self->the_hash_table = (cpp_hash_table*)(new Native_EQL_Hash_Table<double, Object*>);
  }
}

Object* Float_Hash_Table::lookup(double key) {
  { Float_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (stella_hash_table_lookup(self->the_stella_hash_table, wrap_float(key)));
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      return (((Native_EQL_Hash_Table<double, Object*>*)native_table)->get(key));
    }
  }
}

void Float_Hash_Table::insert_at(double key, Object* value) {
  { Float_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_insert_at(self->the_stella_hash_table, wrap_float(key), value);
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      (*((Native_EQL_Hash_Table<double, Object*>*)native_table))[key] = value;;
    }
  }
}

void Float_Hash_Table::remove_at(double key) {
  { Float_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_remove_at(self->the_stella_hash_table, wrap_float(key));
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      ((Native_EQL_Hash_Table<double, Object*>*)native_table)->erase(key);;
    }
  }
}

void String_Hash_Table::initialize_hash_table() {
  // Insert a newly-created native hash table into `self'.
  { String_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->the_stella_hash_table = make_stella_hash_table(50);
      return;
    }
    self->the_hash_table = (cpp_hash_table*)(new Native_EQL_Hash_Table<char*, Object*>);
  }
}

Object* String_Hash_Table::lookup(char* key) {
  { String_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (stella_string_hash_table_lookup(self->the_stella_hash_table, key));
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      return (((Native_EQL_Hash_Table<char*, Object*>*)native_table)->get(key));
    }
  }
}

void String_Hash_Table::insert_at(char* key, Object* value) {
  { String_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_insert_at(self->the_stella_hash_table, wrap_string(key), value);
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      (*((Native_EQL_Hash_Table<char*, Object*>*)native_table))[key] = value;;
    }
  }
}

void String_Hash_Table::remove_at(char* key) {
  { String_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_remove_at(self->the_stella_hash_table, wrap_string(key));
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      ((Native_EQL_Hash_Table<char*, Object*>*)native_table)->erase(key);;
    }
  }
}

void String_To_Integer_Hash_Table::initialize_hash_table() {
  // Insert a newly-created native hash table into `self'.
  { String_To_Integer_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      self->the_stella_hash_table = make_stella_hash_table(50);
      return;
    }
    self->the_hash_table = (cpp_hash_table*)(new Native_EQL_Hash_Table<char*, int>(NULL_INTEGER));
  }
}

int String_To_Integer_Hash_Table::lookup(char* key) {
  { String_To_Integer_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      return (unwrap_integer(((Integer_Wrapper*)(stella_string_hash_table_lookup(self->the_stella_hash_table, key)))));
    }
    { cpp_hash_table* native_table = self->the_hash_table;
      int result = NULL_INTEGER;

      result = ((Native_EQL_Hash_Table<char*, int>*)native_table)->get(key);
      return (result);
    }
  }
}

void String_To_Integer_Hash_Table::insert_at(char* key, int value) {
  { String_To_Integer_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_insert_at(self->the_stella_hash_table, wrap_string(key), wrap_integer(value));
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      (*((Native_EQL_Hash_Table<char*, int>*)native_table))[key] = value;;
    }
  }
}

void String_To_Integer_Hash_Table::remove_at(char* key) {
  { String_To_Integer_Hash_Table* self = this;

    if (oUSE_STELLA_HASH_TABLESpo) {
      stella_hash_table_remove_at(self->the_stella_hash_table, wrap_string(key));
      return;
    }
    { cpp_hash_table* native_table = self->the_hash_table;

      ((Native_EQL_Hash_Table<char*, int>*)native_table)->erase(key);;
    }
  }
}

// A table of 256 26-bit random numbers that can be used to
// hash sequences of bytes.  Each bit-column in the table has an approximately
// even number of 0's and 1's.
int* oHASH_BYTE_RANDOM_TABLEo = NULL;

int object_hash_code(Object* self) {
  // Return a hash code for `self'.  Two objects that are `eq?'
  // are guaranteed to generate the same hash code.  Two objects that are not `eq?'
  // do not necessarily generate different hash codes.  Similar to `hash-code' but
  // always hashes on the address of `self' even if it is a wrapper.
  return ((size_t)self);
}

int Object::hash_code() {
  // Return a hash code for `self'.  Two objects that are `eql?'
  // are guaranteed to generate the same hash code.  Two objects that are not `eql?'
  // do not necessarily generate different hash codes.
  { Object* self = this;

    return ((size_t)self);
  }
}

int Standard_Object::hash_code() {
  { Standard_Object* self = this;

    return ((size_t)self);
  }
}

int Wrapper::hash_code() {
  { Wrapper* self = this;

    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "WRAPPER.hash-code: Don't know how to hash on " << "`" << self << "'";
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
}

int String_Wrapper::hash_code() {
  { String_Wrapper* self = this;

    return (hash_string(self->wrapper_value, 0));
  }
}

int Integer_Wrapper::hash_code() {
  { Integer_Wrapper* self = this;

    return (self->wrapper_value);
  }
}

int Float_Wrapper::hash_code() {
  { Float_Wrapper* self = this;

    return ((size_t)(self->wrapper_value));
  }
}

int Character_Wrapper::hash_code() {
  { Character_Wrapper* self = this;

    return (((oHASH_BYTE_RANDOM_TABLEo[(int)(unsigned char) (self->wrapper_value)]) ^ 33485726));
  }
}

int Boolean_Wrapper::hash_code() {
  { Boolean_Wrapper* self = this;

    return ((self->wrapper_value ? 27333705 : 31891526));
  }
}

int string_hash_code(char* self) {
  return (hash_string(self, 0));
}

int integer_hash_code(int self) {
  return (self);
}

int float_hash_code(double self) {
  return ((size_t)self);
}

int character_hash_code(char self) {
  return (((oHASH_BYTE_RANDOM_TABLEo[(int)(unsigned char) self]) ^ 33485726));
}

int hash_string(char* string, int seedcode) {
  // Generate a hash-code for `string' and return it.
  // Two strings that are equal but not eq will generate the same code.
  // The hash-code is based on `seedCode' which usually will be 0.  However,
  // `seedCode' can also be used to supply the result of a previous hash
  // operation to achieve hashing on sequences of strings without actually
  // having to concatenate them.
  { int code = seedcode;
    int cursor = strlen(string);

    if (cursor == 0) {
      return (4303803);
    }
    else {
      cursor = cursor - 1;
    }
    for (;;) {
      code = (code ^ (oHASH_BYTE_RANDOM_TABLEo[(int)(unsigned char) (string[cursor])]));
      if (cursor == 0) {
        break;
      }
      else {
        cursor = cursor - 1;
      }
      if ((code & 1) == 0) {
        code = (code >> 1);
      }
      else {
        code = (((code >> 1)) | 67108864);
      }
    }
    return (code);
  }
}

// List of prime numbers approximately growing by a factor of 2
// that are suitable to be used as hash table sizes.
List* oHASH_TABLE_SIZE_PRIMESo = NULL;

int pick_hash_table_size_prime(int minsize) {
  // Return a hash table prime of at least `minSize'.
  { Integer_Wrapper* prime = NULL;
    Cons* iter_000 = oHASH_TABLE_SIZE_PRIMESo->the_cons_list;

    while (!(iter_000 == NIL)) {
      prime = ((Integer_Wrapper*)(iter_000->value));
      iter_000 = iter_000->rest;
      if (prime->wrapper_value >= minsize) {
        return (prime->wrapper_value);
      }
    }
  }
  throw *new_stella_exception("pick-hash-table-size-prime: minimum size is too large");
}

void Vector::initialize_vector() {
  { Vector* self = this;

    { int size = self->array_size;

      self->the_array = new (GC) Object*[size];
      { Object** array = self->the_array;

        { int i = NULL_INTEGER;
          int iter_000 = 0;
          int upper_bound_000 = size - 1;

          while (iter_000 <= upper_bound_000) {
            i = iter_000;
            iter_000 = iter_000 + 1;
            array[i] = NULL;
          }
        }
      }
    }
  }
}

void resize_vector(Vector* self, int size) {
  // Change the size of `self' to `size'.  If `size' is smaller
  // than the current size of `self' the vector will be truncated.  Otherwise, 
  // the internal array of `self' will be grown to `size' and unused elements
  // will be initialized to NULL.
  { int old_size = self->array_size;
    int i = old_size;
    Object** old_array = self->the_array;
    Object** new_array = old_array;

    if (size == old_size) {
      return;
    }
    new_array = new (GC) Object*[size];
    if (size < old_size) {
      old_size = size;
      i = old_size;
    }
    while ((i = i - 1) >= 0) {
      new_array[i] = (old_array[i]);
    }
    { int i = NULL_INTEGER;
      int iter_000 = old_size;
      int upper_bound_000 = size - 1;

      while (iter_000 <= upper_bound_000) {
        i = iter_000;
        iter_000 = iter_000 + 1;
        new_array[i] = NULL;
      }
    }
    self->the_array = new_array;
    self->array_size = size;
  }
}

Object* native_vector_nth(Object** self, int position) {
  // Return the element in `self' at `position'.
  return (self[position]);
}

Object* native_vector_nth_setter(Object** self, Object* value, int position) {
  // Set the element in `self' at `position' to `value'.
  return (self[position] = value);
}

void unmake(Object* self) {
}

void copy_file(char* fromfile, char* tofile) {
  // Copy file `fromFile' to file `toFile', clobbering
  // any data already in `toFile'.
  if (!probe_fileP(fromfile)) {
    { Output_String_Stream* stream_000 = new_output_string_stream();

      *(stream_000->native_stream) << "Can't copy non-existent file: " << "`" << fromfile << "'" << endl;
      throw *new_stella_exception(stream_000->the_string_reader());
    }
  }
  { Input_File_Stream* from = new_input_file_stream(fromfile);
    Output_File_Stream* to = new_output_file_stream(tofile);

    { char* line = NULL;
      Line_Iterator* iter_000 = lines(from);

      while (iter_000->nextP()) {
        line = iter_000->value;
        *(to->native_stream) << line << endl;
      }
    }
    to->free();
  }
}

int lognot(int arg) {
  return ((~ arg));
}

int logand(int arg1, int arg2) {
  return ((arg1 & arg2));
}

int logor(int arg1, int arg2) {
  return ((arg1 | arg2));
}

int logxor(int arg1, int arg2) {
  return ((arg1 ^ arg2));
}

int shift_left(int arg, int count) {
  return ((arg << count));
}

int shift_right(int arg, int count) {
  return ((arg >> count));
}

Object* stellify(Object* self) {
  // Convert a Lisp object into a STELLA object.
  { Object* result = self;

    return (result);
  }
}

boolean stella_objectP(Object* self) {
  // Return TRUE if `self' is a member of the STELLA class `OBJECT'.
  { boolean resultP = TRUE;

    return (resultP);
  }
}

boolean running_as_lispP() {
  // Return TRUE if the executable code is a Common Lisp application.
  { boolean resultP = FALSE;

    return (resultP);
  }
}

void sleep_(double seconds) {
  // The program will sleep for the indicated number of seconds.
  // Fractional values are allowed, but the results are implementation dependent:
  // Common Lisp uses the fractions natively, Java with a resolution of 0.001,
  // and C++ can only use integral values.
  sleep((unsigned int)seconds);
}

char* exception_message(exception* e) {
  // Accesses the error message of the exception `e'.
  return ((char*)e->what());
}

void print_exception_context(exception* e, Output_Stream* stream) {
  // Prints a system dependent information about the context of the specified
  // exception.  For example, in Java it prints a stack trace.  In Lisp, it is vendor dependent.
  { ostream* s = stream->native_stream;

    *(stream->native_stream) << "No exception context available" << endl;
  }
}

char* make_process_lock() {
  return (NULL);
}

void help_startup_primal1() {
  {
    SYM_PRIMAL_STELLA_FLOOR = ((Symbol*)(intern_rigid_symbol_wrt_module("FLOOR", NULL, 0)));
    KWD_PRIMAL_CPP = ((Keyword*)(intern_rigid_symbol_wrt_module("CPP", NULL, 2)));
    KWD_PRIMAL_FUNCTION = ((Keyword*)(intern_rigid_symbol_wrt_module("FUNCTION", NULL, 2)));
    SYM_PRIMAL_STELLA_RANDOM = ((Symbol*)(intern_rigid_symbol_wrt_module("RANDOM", NULL, 0)));
    SYM_PRIMAL_STELLA_SQRT = ((Symbol*)(intern_rigid_symbol_wrt_module("SQRT", NULL, 0)));
    SYM_PRIMAL_STELLA_COS = ((Symbol*)(intern_rigid_symbol_wrt_module("COS", NULL, 0)));
    SYM_PRIMAL_STELLA_SIN = ((Symbol*)(intern_rigid_symbol_wrt_module("SIN", NULL, 0)));
    SYM_PRIMAL_STELLA_TAN = ((Symbol*)(intern_rigid_symbol_wrt_module("TAN", NULL, 0)));
    SYM_PRIMAL_STELLA_LOG = ((Symbol*)(intern_rigid_symbol_wrt_module("LOG", NULL, 0)));
    SYM_PRIMAL_STELLA_MIN = ((Symbol*)(intern_rigid_symbol_wrt_module("MIN", NULL, 0)));
    SYM_PRIMAL_STELLA_MAX = ((Symbol*)(intern_rigid_symbol_wrt_module("MAX", NULL, 0)));
    KWD_PRIMAL_UPCASE = ((Keyword*)(intern_rigid_symbol_wrt_module("UPCASE", NULL, 2)));
    KWD_PRIMAL_DOWNCASE = ((Keyword*)(intern_rigid_symbol_wrt_module("DOWNCASE", NULL, 2)));
    KWD_PRIMAL_CAPITALIZE = ((Keyword*)(intern_rigid_symbol_wrt_module("CAPITALIZE", NULL, 2)));
    KWD_PRIMAL_PRESERVE = ((Keyword*)(intern_rigid_symbol_wrt_module("PRESERVE", NULL, 2)));
    KWD_PRIMAL_LEFT = ((Keyword*)(intern_rigid_symbol_wrt_module("LEFT", NULL, 2)));
    KWD_PRIMAL_RIGHT = ((Keyword*)(intern_rigid_symbol_wrt_module("RIGHT", NULL, 2)));
    KWD_PRIMAL_CENTER = ((Keyword*)(intern_rigid_symbol_wrt_module("CENTER", NULL, 2)));
    SYM_PRIMAL_STELLA_HASH_CODE = ((Symbol*)(intern_rigid_symbol_wrt_module("HASH-CODE", NULL, 0)));
    KWD_PRIMAL_JAVA = ((Keyword*)(intern_rigid_symbol_wrt_module("JAVA", NULL, 2)));
    SYM_PRIMAL_STELLA_SLEEP = ((Symbol*)(intern_rigid_symbol_wrt_module("SLEEP", NULL, 0)));
    SYM_PRIMAL_STELLA_STARTUP_PRIMAL = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-PRIMAL", NULL, 0)));
    SYM_PRIMAL_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
  }
}

void help_startup_primal2() {
  {
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST UNKNOWN)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST OBJECT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST SECOND-CLASS-OBJECT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST NATIVE-VECTOR)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST STRING)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST MUTABLE-STRING)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST CHARACTER)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST CODE)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST INTEGER)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NULL? BOOLEAN) ((TEST FLOAT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST UNKNOWN)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST OBJECT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST SECOND-CLASS-OBJECT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST NATIVE-VECTOR)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST STRING)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST MUTABLE-STRING)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST CHARACTER)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST CODE)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST INTEGER)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (DEFINED? BOOLEAN) ((TEST FLOAT)) :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("EQ?", "(DEFUN (EQ? BOOLEAN) ((X UNKNOWN) (Y UNKNOWN)) :DOCUMENTATION \"Return `true' if `x' and `y' are literally the same object (or\nsimple number).  Analogue to the Common Lisp EQL and C++ and Java's ==.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    define_function_object("=", "(DEFUN (= BOOLEAN) ((X NUMBER) (Y NUMBER)) :DOCUMENTATION \"Return `true' if `x' and `y' are numbers of exactly the same magnitude.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    define_function_object("EQL?", "(DEFUN (EQL? BOOLEAN) ((X OBJECT) (Y OBJECT)) :DOCUMENTATION \"Return `true' if `x' and `y' are `eq?' or equivalent literals\nsuch as strings that also might be wrapped in non-identical wrappers.  For\nthe case where `x' or `y' are plain literals such as strings or integers, the\nSTELLA translator substitutes the equality test appropriate for the particular\ntarget language and does not actually call this function.  For cases where\n`x' or `y' are known to be of type STANDARD-OBJECT, the STELLA translator\nsubstitutes the faster `eq?' test inline.\" :PUBLIC? TRUE)", ((cpp_function_code)(&eqlP)), NULL);
    define_function_object("EQUAL?", "(DEFUN (EQUAL? BOOLEAN) ((X OBJECT) (Y OBJECT)) :DOCUMENTATION \"Return `true' if `x' and `y' are `eql?' or considered equal\nby a user-defined `object-equal?' method.  This implements a fully extensible\nequality test similar to Java's `equals' method.\" :PUBLIC? TRUE)", ((cpp_function_code)(&equalP)), NULL);
    define_method_object("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X OBJECT) (Y OBJECT)) :DOCUMENTATION \"Return `true' if `x' and `y' are `eq?'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Object::object_equalP)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (OBJECT-EQUAL? BOOLEAN) ((X WRAPPER) (Y OBJECT)) :DOCUMENTATION \"Return `true' if `x' and `y' are literal wrappers whose\nliterals are considered `eql?'.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Wrapper::object_equalP)), ((cpp_method_code)(NULL)));
    define_function_object(">", "(DEFUN (> BOOLEAN) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    define_function_object(">=", "(DEFUN (>= BOOLEAN) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    define_function_object("<", "(DEFUN (< BOOLEAN) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    define_function_object("<=", "(DEFUN (<= BOOLEAN) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    define_function_object("+", "(DEFUN (+ NUMBER) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    define_function_object("-", "(DEFUN (- NUMBER) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    define_function_object("*", "(DEFUN (* NUMBER) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    define_function_object("/", "(DEFUN (/ NUMBER) ((X NUMBER) (Y NUMBER)) :NATIVE? TRUE)", NULL, NULL);
    define_function_object("ZERO?", "(DEFUN (ZERO? BOOLEAN) ((X INTEGER)) :DOCUMENTATION \"Return `true' if `x' is 0.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:= X 0) :CPP \"(!x)\" :JAVA \"(x == 0)\")))", ((cpp_function_code)(&zeroP)), NULL);
    define_function_object("PLUS?", "(DEFUN (PLUS? BOOLEAN) ((X INTEGER)) :DOCUMENTATION \"Return `true' if `x' is greater than 0.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:> X 0) :CPP \"(x > 0)\" :JAVA \"(x > 0)\")))", ((cpp_function_code)(&plusP)), NULL);
    define_function_object("EVEN?", "(DEFUN (EVEN? BOOLEAN) ((X INTEGER)) :DOCUMENTATION \"Return `true' if `x' is an even number.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:NOT (CL:LOGBITP 0 X)) :CPP \"!(x % 2)\" :JAVA \"((x % 2) == 0)\")))", ((cpp_function_code)(&evenP)), NULL);
    define_function_object("ODD?", "(DEFUN (ODD? BOOLEAN) ((X INTEGER)) :DOCUMENTATION \"Return `true' if `x' is an odd number.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGBITP 0 X) :CPP \"(x % 2)\" :JAVA \"((x % 2) == 1)\")))", ((cpp_function_code)(&oddP)), NULL);
    define_function_object("MOD", "(DEFUN (MOD INTEGER) ((X INTEGER) (MODULO INTEGER)) :DOCUMENTATION \"Return the result of `x' mod `modulo'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:MOD X MODULO) :CPP \"(x % modulo)\" :JAVA \"(x % modulo)\")))", ((cpp_function_code)(&mod)), NULL);
    define_function_object("CEILING", "(DEFUN (CEILING INTEGER) ((N NUMBER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("FLOOR", "(DEFUN (FLOOR INTEGER) ((N NUMBER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("ROUND", "(DEFUN (ROUND INTEGER) ((N NUMBER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("EXP", "(DEFUN (EXP FLOAT) ((X FLOAT)) :PUBLIC? TRUE :NATIVE? TRUE :DOCUMENTATION \"The natural exponentiation function e^x.\")", NULL, NULL);
    define_function_object("RANDOM", "(DEFUN (RANDOM INTEGER) ((N INTEGER)) :PUBLIC? TRUE :DOCUMENTATION \"Generate a random integer in the interval [0..n-1].\n`n' must be <= 2^15.\")", ((cpp_function_code)(&random_)), NULL);
    define_function_object("SEED-RANDOM-NUMBER-GENERATOR", "(DEFUN SEED-RANDOM-NUMBER-GENERATOR () :PUBLIC? TRUE :DOCUMENTATION \"Seeds the random number generator with the current time\")", ((cpp_function_code)(&seed_random_number_generator)), NULL);
    define_function_object("SQRT", "(DEFUN (SQRT FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the square root of `n'.\" (RETURN (VERBATIM :COMMON-LISP (CL:SQRT N) :CPP \"sqrt(n)\" :JAVA \"Math.sqrt(n)\")))", ((cpp_function_code)(&sqrt_)), NULL);
    define_function_object("COS", "(DEFUN (COS FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the cosine of `n' radians.\" (RETURN (VERBATIM :COMMON-LISP (CL:COS N) :CPP \"cos(n)\" :JAVA \"Math.cos(n)\")))", ((cpp_function_code)(&cos_)), NULL);
    define_function_object("SIN", "(DEFUN (SIN FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the sine of `n' radians.\" (RETURN (VERBATIM :COMMON-LISP (CL:SIN N) :CPP \"sin(n)\" :JAVA \"Math.sin(n)\")))", ((cpp_function_code)(&sin_)), NULL);
    define_function_object("TAN", "(DEFUN (TAN FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the tangent of `n' radians.\" (RETURN (VERBATIM :COMMON-LISP (CL:TAN N) :CPP \"tan(n)\" :JAVA \"Math.tan(n)\")))", ((cpp_function_code)(&tan_)), NULL);
    define_function_object("LOG", "(DEFUN (LOG FLOAT) ((N FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE :DOCUMENTATION \"Return the natural logarithm (base e) of `n'.\" (RETURN (VERBATIM :COMMON-LISP (CL:LOG N) :CPP \"log(n)\" :JAVA \"Math.log(n)\")))", ((cpp_function_code)(&log_)), NULL);
    define_function_object("MIN", "(DEFUN (MIN INTEGER) ((X INTEGER) (Y INTEGER)) :DOCUMENTATION \"Return the minimum of `x' and `y'.  If either is NULL, return the other.\" :PUBLIC? TRUE)", ((cpp_function_code)(&min_)), NULL);
    define_function_object("MAX", "(DEFUN (MAX INTEGER) ((X INTEGER) (Y INTEGER)) :DOCUMENTATION \"Return the maximum of `x' and `y'.  If either is NULL, return the other.\" :PUBLIC? TRUE)", ((cpp_function_code)(&max_)), NULL);
    define_method_object("(DEFMETHOD (ABS INTEGER) ((X INTEGER)) :DOCUMENTATION \"Return the absolute value of `x'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (CHOOSE (< X 0) (- 0 X) X)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (ABS FLOAT) ((X FLOAT)) :DOCUMENTATION \"Return the absolute value of `x'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (CHOOSE (< X 0.0) (- 0.0 X) X)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("CHARACTER-CODE", "(DEFUN (CHARACTER-CODE INTEGER) ((CHARACTER CHARACTER)) :DOCUMENTATION \"Return the 8-bit ASCII code of `character' as an integer.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CHAR-CODE CHARACTER) :CPP \"(int)(unsigned char) character\" :JAVA \"(int) character\")))", ((cpp_function_code)(&character_code)), NULL);
    define_function_object("CODE-CHARACTER", "(DEFUN (CODE-CHARACTER CHARACTER) ((CODE INTEGER)) :DOCUMENTATION \"Return the character encoded by `code' (0 <= `code' <= 255).\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CODE-CHAR CODE) :CPP \"(char) code\" :JAVA \"(char) code\")))", ((cpp_function_code)(&code_character)), NULL);
    define_function_object("CHARACTER-DOWNCASE", "(DEFUN (CHARACTER-DOWNCASE CHARACTER) ((C CHARACTER)) :DOCUMENTATION \"Return the lowercase character for `c'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CHAR-DOWNCASE C) :CPP \"(isupper(c) ? c + 'a' - 'A' : c)\" :JAVA \"Character.toLowerCase(c)\")))", ((cpp_function_code)(&character_downcase)), NULL);
    define_function_object("CHARACTER-UPCASE", "(DEFUN (CHARACTER-UPCASE CHARACTER) ((C CHARACTER)) :DOCUMENTATION \"Return the uppercase character for `c'.  If only the first character of\na sequence of characters is to be capitalized, character-capitalize should be used.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CHAR-UPCASE C) :CPP \"(islower(c) ? c + 'A' - 'a' : c)\" :JAVA \"Character.toUpperCase(c)\")))", ((cpp_function_code)(&character_upcase)), NULL);
    define_function_object("CHARACTER-CAPITALIZE", "(DEFUN (CHARACTER-CAPITALIZE CHARACTER) ((C CHARACTER)) :DOCUMENTATION \"Return the capitalized character for `c'.  This is generally the same\nas the uppercase character, except for obscure non-English characters in Java.  It should\nbe used if only the first character of a sequence of characters is to be capitalized.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:CHAR-UPCASE C) :CPP \"(islower(c) ? c + 'A' - 'a' : c)\" :JAVA \"Character.toTitleCase(c)\")))", ((cpp_function_code)(&character_capitalize)), NULL);
    define_function_object("STRING-EQL?", "(DEFUN (STRING-EQL? BOOLEAN) ((X STRING) (Y STRING)) :DOCUMENTATION \"Return true if `x' and `y' are equal strings including case for both null.\" :PUBLIC? TRUE)", ((cpp_function_code)(&string_eqlP)), NULL);
  }
}

void help_startup_primal3() {
  {
    define_function_object("STRING-EQUAL?", "(DEFUN (STRING-EQUAL? BOOLEAN) ((X STRING) (Y STRING)) :DOCUMENTATION \"Return true if `x' and `y' are equal strings ignoring case for both null.\" :PUBLIC? TRUE)", ((cpp_function_code)(&string_equalP)), NULL);
    define_function_object("STRING-COMPARE", "(DEFUN (STRING-COMPARE INTEGER) ((X STRING) (Y STRING) (CASE-SENSITIVE? BOOLEAN)) :PUBLIC? TRUE :DOCUMENTATION \"Compare `x' and `y' lexicographically, and return -1, 0, \nor 1, depending on whether `x' is less than, equal, or greater than `y'.\nIf `case-sensitive?' is `true', then case does matter for the comparison\")", ((cpp_function_code)(&string_compare)), NULL);
    define_function_object("STRING<", "(DEFUN (STRING< BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically < `y', considering case.\")", ((cpp_function_code)(&stringL)), NULL);
    define_function_object("STRING<=", "(DEFUN (STRING<= BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically <= `y', considering case..\")", ((cpp_function_code)(&stringLE)), NULL);
    define_function_object("STRING>=", "(DEFUN (STRING>= BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically >= `y', considering case..\")", ((cpp_function_code)(&stringGE)), NULL);
    define_function_object("STRING>", "(DEFUN (STRING> BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically > `y', considering case..\")", ((cpp_function_code)(&stringG)), NULL);
    define_function_object("STRING-LESS?", "(DEFUN (STRING-LESS? BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically < `y', ignoring case.\")", ((cpp_function_code)(&string_lessP)), NULL);
    define_function_object("STRING-LESS-EQUAL?", "(DEFUN (STRING-LESS-EQUAL? BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically <= `y', ignoring case.\")", ((cpp_function_code)(&string_less_equalP)), NULL);
    define_function_object("STRING-GREATER-EQUAL?", "(DEFUN (STRING-GREATER-EQUAL? BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically >= `y', ignoring case.\")", ((cpp_function_code)(&string_greater_equalP)), NULL);
    define_function_object("STRING-GREATER?", "(DEFUN (STRING-GREATER? BOOLEAN) ((X STRING) (Y STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return TRUE if `x' is lexicographically > `y', ignoring case.\")", ((cpp_function_code)(&string_greaterP)), NULL);
    define_function_object("MAKE-STRING", "(DEFUN (MAKE-STRING STRING) ((SIZE INTEGER) (INITCHAR CHARACTER)) :DOCUMENTATION \"Return a new string filled with `size' `initchar's.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    define_function_object("MAKE-MUTABLE-STRING", "(DEFUN (MAKE-MUTABLE-STRING MUTABLE-STRING) ((SIZE INTEGER) (INITCHAR CHARACTER)) :DOCUMENTATION \"Return a new mutable string filled with `size' `initchar's.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:MAKE-STRING SIZE :INITIAL-ELEMENT INITCHAR) :CPP \"make_string(size, initchar)\" :JAVA \"#$(STELLAROOT).javalib.Native.makeMutableString(size, initchar)\")))", ((cpp_function_code)(&make_mutable_string)), NULL);
    define_function_object("MAKE-RAW-MUTABLE-STRING", "(DEFUN (MAKE-RAW-MUTABLE-STRING MUTABLE-STRING) ((SIZE INTEGER)) :DOCUMENTATION \"Return a new uninitialized mutable string of `size'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (LET ((S MUTABLE-STRING NULL)) (VERBATIM :COMMON-LISP (SETQ S (CL:MAKE-STRING SIZE)) :CPP \"s = new (GC) char[size]\" :JAVA \"s = new StringBuffer(size); s.setLength(size)\") (RETURN S)))", ((cpp_function_code)(&make_raw_mutable_string)), NULL);
    define_function_object("STRING-CONCATENATE", "(DEFUN (STRING-CONCATENATE STRING) ((STRING1 STRING) (STRING2 STRING)) :DOCUMENTATION \"Return a new string representing the concatenation of\n`string1' and `string2'.\" :NATIVE? TRUE)", NULL, NULL);
    define_method_object("(DEFMETHOD (CONCATENATE STRING) ((STRING1 STRING) (STRING2 STRING) |&REST| (OTHERSTRINGS STRING)) :PUBLIC? TRUE :DOCUMENTATION \"Return a new string representing the concatenation\nof `string1', `string2', and `otherStrings'.  The two mandatory parameters\nallow us to optimize the common binary case by not relying on the somewhat\nless efficient variable arguments mechanism.\")", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("STRING-UPCASE", "(DEFUN (STRING-UPCASE STRING) ((STRING STRING)) :DOCUMENTATION \"Return an upper-case copy of `string'.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("STRING-DOWNCASE", "(DEFUN (STRING-DOWNCASE STRING) ((STRING STRING)) :DOCUMENTATION \"Return a lower-case copy of `string'.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("STRING-CAPITALIZE", "(DEFUN (STRING-CAPITALIZE STRING) ((STRING STRING)) :DOCUMENTATION \"Return a capitalized version of `string'.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_method_object("(DEFMETHOD (COPY STRING) ((STRING STRING)) :DOCUMENTATION \"Return a copy of `string'.\" :NATIVE? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SUBSTITUTE MUTABLE-STRING) ((SELF MUTABLE-STRING) (NEWCHAR CHARACTER) (OLDCHAR CHARACTER)) :DOCUMENTATION \"Substitute all occurences of `oldChar' with `newChar'\nin the string `self'.\" :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (SUBSTITUTE STRING) ((SELF STRING) (NEWCHAR CHARACTER) (OLDCHAR CHARACTER)) :DOCUMENTATION \"Substitute all occurences of `oldChar' with `newChar'\nin the string `self'.\" :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("REPLACE-SUBSTRINGS", "(DEFUN (REPLACE-SUBSTRINGS STRING) ((STRING STRING) (NEW STRING) (OLD STRING)) :DOCUMENTATION \"Replace all occurrences of `old' in `string' with `new'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&replace_substrings)), NULL);
    define_function_object("INSERT-STRING", "(DEFUN (INSERT-STRING INTEGER) ((SOURCE STRING) (START INTEGER) (END INTEGER) (TARGET MUTABLE-STRING) (TARGET-INDEX INTEGER) (CASE-CONVERSION KEYWORD)) :DOCUMENTATION \"Inserts characters from `source' begining at `start' and\nending at `end' into `target' starting at `target-index'.  If `end' is `null',\nthen the entire length of the string is used. The copy of characters is affected\nby the `case-conversion' keyword which should be one of\n   :UPCASE :DOWNCASE :CAPITALIZE :PRESERVE.\n\nThe final value of target-index is returned.\")", ((cpp_function_code)(&insert_string)), NULL);
    define_function_object("INTEGER-TO-STRING", "(DEFUN (INTEGER-TO-STRING STRING) ((I INTEGER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("FLOAT-TO-STRING", "(DEFUN (FLOAT-TO-STRING STRING) ((F FLOAT)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("CHARACTER-TO-STRING", "(DEFUN (CHARACTER-TO-STRING STRING) ((C CHARACTER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (MAKE-STRING 1 C)))", ((cpp_function_code)(&character_to_string)), NULL);
    define_function_object("FORMAT-FLOAT", "(DEFUN (FORMAT-FLOAT STRING) ((F FLOAT) (NDECIMALS INTEGER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("STRING-TO-INTEGER", "(DEFUN (STRING-TO-INTEGER INTEGER) ((STRING STRING)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("STRING-TO-FLOAT", "(DEFUN (STRING-TO-FLOAT FLOAT) ((STRING STRING)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("FORMAT-WITH-PADDING", "(DEFUN (FORMAT-WITH-PADDING STRING) ((INPUT STRING) (LENGTH INTEGER) (PADCHAR CHARACTER) (ALIGN KEYWORD) (TRUNCATE? BOOLEAN)) :PUBLIC? TRUE :DOCUMENTATION \"Formats `input' to be (at least) `length' long, using `padchar' to\nfill if necessary.  `align' must be one of :LEFT, :RIGHT, :CENTER and will control\nhow `input' will be justified in the resulting string.  If `truncate?' is `true', then\nthen an overlength string will be truncated, using the opposite of `align' to pick\nthe truncation direction.\")", ((cpp_function_code)(&format_with_padding)), NULL);
    define_method_object("(DEFMETHOD (MEMBER? BOOLEAN) ((SELF STRING) (CHAR CHARACTER)) :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH CHARACTER) ((SELF STRING) (POSITION INTEGER)) :DOCUMENTATION \"Return the character in `self' at `position'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:AREF (CL:THE CL:SIMPLE-STRING SELF) (CL:THE CL:FIXNUM POSITION)) :CPP \"self[position]\" :JAVA \"self.charAt(position)\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH CHARACTER) ((SELF MUTABLE-STRING) (POSITION INTEGER)) :DOCUMENTATION \"Return the character in `self' at `position'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:AREF (CL:THE CL:SIMPLE-STRING SELF) (CL:THE CL:FIXNUM POSITION)) :CPP \"self[position]\" :JAVA \"self.charAt(position)\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH-SETTER CHARACTER) ((SELF MUTABLE-STRING) (CH CHARACTER) (POSITION INTEGER)) :DOCUMENTATION \"Set the character in `self' at `position' to `ch'.\" :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (SETF (CL:AREF (CL:THE CL:SIMPLE-STRING SELF) (CL:THE CL:FIXNUM POSITION)) (CL:THE CL:CHARACTER CH)) :CPP \"self[position] = ch\" :JAVA \"#$(STELLAROOT).javalib.Native.mutableString_nthSetter(self, ch, position)\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (REST STRING) ((SELF STRING)) :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF STRING)) :DOCUMENTATION \"Return the length of the string `self'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LENGTH SELF) :CPP \"strlen(self)\" :JAVA \"self.length()\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LENGTH INTEGER) ((SELF MUTABLE-STRING)) :DOCUMENTATION \"Return the length of the string `self'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LENGTH SELF) :CPP \"strlen(self)\" :JAVA \"self.length()\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (POSITION INTEGER) ((STRING STRING) (CHARACTER CHARACTER) (START INTEGER)) :DOCUMENTATION \"Return the position of `character' within `string' (counting\nfrom zero); or return NULL if `character' does not occur within `string'.\nIf `start' was supplied as non-NULL, only consider the substring starting\nat `start', however, the returned position will always be relative to the\nentire string.\" :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("STRING-SEARCH", "(DEFUN (STRING-SEARCH INTEGER) ((STRING STRING) (SUBSTRING STRING) (START INTEGER)) :DOCUMENTATION \"Return start position of the left-most occurrence of\n`substring' in `string', beginning from `start'.  Return NULL if it is not\na substring.\" :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_method_object("(DEFMETHOD (SUBSEQUENCE STRING) ((STRING STRING) (START INTEGER) (END INTEGER)) :DOCUMENTATION \"Return a substring of `string' beginning at position `start'\nand ending up to but not including position `end', counting from zero.  An\n`end' value of NULL stands for the rest of the string.\" :NATIVE? TRUE :PUBLIC? TRUE)", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("STRINGIFY", "(DEFUN (STRINGIFY STRING) ((EXPRESSION OBJECT)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("UNSTRINGIFY", "(DEFUN (UNSTRINGIFY OBJECT) ((STRING STRING)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (READ-S-EXPRESSION-FROM-STRING STRING)))", ((cpp_function_code)(&unstringify)), NULL);
    define_method_object("(DEFMETHOD FREE-HASH-TABLE-VALUES ((SELF ABSTRACT-HASH-TABLE)) :DOCUMENTATION \"Call free on each value in the hash table `self'.\")", ((cpp_method_code)(&Abstract_Hash_Table::free_hash_table_values)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF ABSTRACT-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&Abstract_Hash_Table::initialize_hash_table)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Hash_Table::lookup)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-AT ((SELF HASH-TABLE) (KEY (LIKE (ANY-KEY SELF))) (VALUE (LIKE (ANY-VALUE SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Hash_Table::insert_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD REMOVE-AT ((SELF HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))) :PUBLIC? TRUE)", ((cpp_method_code)(&Hash_Table::remove_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF INTEGER-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&Integer_Hash_Table::initialize_hash_table)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF INTEGER-HASH-TABLE) (KEY INTEGER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Integer_Hash_Table::lookup)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-AT ((SELF INTEGER-HASH-TABLE) (KEY INTEGER) (VALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Integer_Hash_Table::insert_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD REMOVE-AT ((SELF INTEGER-HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))))", ((cpp_method_code)(&Integer_Hash_Table::remove_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF FLOAT-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&Float_Hash_Table::initialize_hash_table)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF FLOAT-HASH-TABLE) (KEY FLOAT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Float_Hash_Table::lookup)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-AT ((SELF FLOAT-HASH-TABLE) (KEY FLOAT) (VALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&Float_Hash_Table::insert_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD REMOVE-AT ((SELF FLOAT-HASH-TABLE) (KEY (LIKE (ANY-KEY SELF)))))", ((cpp_method_code)(&Float_Hash_Table::remove_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF STRING-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&String_Hash_Table::initialize_hash_table)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (LOOKUP (LIKE (ANY-VALUE SELF))) ((SELF STRING-HASH-TABLE) (KEY STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(&String_Hash_Table::lookup)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-AT ((SELF STRING-HASH-TABLE) (KEY STRING) (VALUE OBJECT)) :PUBLIC? TRUE)", ((cpp_method_code)(&String_Hash_Table::insert_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD REMOVE-AT ((SELF STRING-HASH-TABLE) (KEY STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(&String_Hash_Table::remove_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INITIALIZE-HASH-TABLE ((SELF STRING-TO-INTEGER-HASH-TABLE)) :DOCUMENTATION \"Insert a newly-created native hash table into `self'.\")", ((cpp_method_code)(&String_To_Integer_Hash_Table::initialize_hash_table)), ((cpp_method_code)(NULL)));
  }
}

void help_startup_primal4() {
  {
    define_method_object("(DEFMETHOD (LOOKUP INTEGER) ((SELF STRING-TO-INTEGER-HASH-TABLE) (KEY STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(&String_To_Integer_Hash_Table::lookup)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD INSERT-AT ((SELF STRING-TO-INTEGER-HASH-TABLE) (KEY STRING) (VALUE INTEGER)) :PUBLIC? TRUE)", ((cpp_method_code)(&String_To_Integer_Hash_Table::insert_at)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD REMOVE-AT ((SELF STRING-TO-INTEGER-HASH-TABLE) (KEY STRING)) :PUBLIC? TRUE)", ((cpp_method_code)(&String_To_Integer_Hash_Table::remove_at)), ((cpp_method_code)(NULL)));
    define_function_object("OBJECT-HASH-CODE", "(DEFUN (OBJECT-HASH-CODE INTEGER) ((SELF OBJECT)) :DOCUMENTATION \"Return a hash code for `self'.  Two objects that are `eq?'\nare guaranteed to generate the same hash code.  Two objects that are not `eq?'\ndo not necessarily generate different hash codes.  Similar to `hash-code' but\nalways hashes on the address of `self' even if it is a wrapper.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP \"(STELLA::%%OBJECT-HASH-CODE SELF)\" :CPP \"(size_t)self\" :JAVA \"self.hashCode()\")))", ((cpp_function_code)(&object_hash_code)), NULL);
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF OBJECT)) :DOCUMENTATION \"Return a hash code for `self'.  Two objects that are `eql?'\nare guaranteed to generate the same hash code.  Two objects that are not `eql?'\ndo not necessarily generate different hash codes.\" :PUBLIC? TRUE)", ((cpp_method_code)(&Object::hash_code)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF STANDARD-OBJECT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (OBJECT-HASH-CODE SELF)))", ((cpp_method_code)(&Standard_Object::hash_code)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF WRAPPER)) :PUBLIC? TRUE)", ((cpp_method_code)(&Wrapper::hash_code)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF STRING-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (HASH-CODE (WRAPPER-VALUE SELF))))", ((cpp_method_code)(&String_Wrapper::hash_code)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF INTEGER-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (HASH-CODE (WRAPPER-VALUE SELF))))", ((cpp_method_code)(&Integer_Wrapper::hash_code)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF FLOAT-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (HASH-CODE (WRAPPER-VALUE SELF))))", ((cpp_method_code)(&Float_Wrapper::hash_code)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF CHARACTER-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (HASH-CODE (WRAPPER-VALUE SELF))))", ((cpp_method_code)(&Character_Wrapper::hash_code)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF BOOLEAN-WRAPPER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (CHOOSE (WRAPPER-VALUE SELF) 27333705 31891526)))", ((cpp_method_code)(&Boolean_Wrapper::hash_code)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF STRING)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:SXHASH SELF) :JAVA \"self.hashCode()\" :OTHERWISE (HASH-STRING SELF 0))))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF INTEGER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN SELF))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF FLOAT)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:SXHASH SELF) :CPP \"(size_t)self\" :JAVA \"new Float(self).hashCode()\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (HASH-CODE INTEGER) ((SELF CHARACTER)) :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (LOGXOR (AREF *HASH-BYTE-RANDOM-TABLE* (CHARACTER-CODE SELF)) 33485726)))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("HASH-STRING", "(DEFUN (HASH-STRING INTEGER) ((STRING STRING) (SEEDCODE INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE :DOCUMENTATION \"Generate a hash-code for `string' and return it.\nTwo strings that are equal but not eq will generate the same code.\nThe hash-code is based on `seedCode' which usually will be 0.  However,\n`seedCode' can also be used to supply the result of a previous hash\noperation to achieve hashing on sequences of strings without actually\nhaving to concatenate them.\" (LET ((CODE SEEDCODE) (CURSOR (LENGTH STRING))) (IF (= CURSOR 0) (RETURN 4303803) (-- CURSOR)) (LOOP (SETQ CODE (LOGXOR CODE (AREF *HASH-BYTE-RANDOM-TABLE* (CHARACTER-CODE (NTH STRING CURSOR))))) (IF (= CURSOR 0) (BREAK) (-- CURSOR)) (IF (= (LOGAND CODE 1) 0) (SETQ CODE (SHIFT-RIGHT CODE 1)) (SETQ CODE (LOGOR (SHIFT-RIGHT CODE 1) 67108864)))) (RETURN CODE)))", ((cpp_function_code)(&hash_string)), NULL);
    define_function_object("PICK-HASH-TABLE-SIZE-PRIME", "(DEFUN (PICK-HASH-TABLE-SIZE-PRIME INTEGER) ((MINSIZE INTEGER)) :DOCUMENTATION \"Return a hash table prime of at least `minSize'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&pick_hash_table_size_prime)), NULL);
    define_method_object("(DEFMETHOD INITIALIZE-VECTOR ((SELF VECTOR)))", ((cpp_method_code)(&Vector::initialize_vector)), ((cpp_method_code)(NULL)));
    define_function_object("RESIZE-VECTOR", "(DEFUN RESIZE-VECTOR ((SELF VECTOR) (SIZE INTEGER)) :DOCUMENTATION \"Change the size of `self' to `size'.  If `size' is smaller\nthan the current size of `self' the vector will be truncated.  Otherwise, \nthe internal array of `self' will be grown to `size' and unused elements\nwill be initialized to NULL.\" :PUBLIC? TRUE)", ((cpp_function_code)(&resize_vector)), NULL);
    define_method_object("(DEFMETHOD (NTH (LIKE (ANY-VALUE SELF))) ((SELF NATIVE-VECTOR) (POSITION INTEGER)) :DOCUMENTATION \"Return the element in `self' at `position'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:AREF SELF POSITION) :CPP \"self[position]\" :JAVA \"self[position]\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_method_object("(DEFMETHOD (NTH-SETTER (LIKE (ANY-VALUE SELF))) ((SELF NATIVE-VECTOR) (VALUE (LIKE (ANY-VALUE SELF))) (POSITION INTEGER)) :DOCUMENTATION \"Set the element in `self' at `position' to `value'.\" :PUBLIC? TRUE :GLOBALLY-INLINE? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:SETF (CL:AREF SELF POSITION) VALUE) :CPP \"self[position] = value\" :JAVA \"self[position] = value\")))", ((cpp_method_code)(NULL)), ((cpp_method_code)(NULL)));
    define_function_object("UNMAKE", "(DEFUN UNMAKE ((SELF OBJECT)))", ((cpp_function_code)(&unmake)), NULL);
    define_function_object("PROBE-FILE?", "(DEFUN (PROBE-FILE? BOOLEAN) ((FILENAME FILE-NAME)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("FILE-WRITE-DATE", "(DEFUN (FILE-WRITE-DATE CALENDAR-DATE) ((FILENAME FILE-NAME)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("FILE-LENGTH", "(DEFUN (FILE-LENGTH INTEGER) ((FILENAME FILE-NAME)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("DELETE-FILE", "(DEFUN DELETE-FILE ((FILENAME FILE-NAME)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("COPY-FILE", "(DEFUN COPY-FILE ((FROMFILE FILE-NAME) (TOFILE FILE-NAME)) :DOCUMENTATION \"Copy file `fromFile' to file `toFile', clobbering\nany data already in `toFile'.\" :PUBLIC? TRUE)", ((cpp_function_code)(&copy_file)), NULL);
    define_function_object("LOGNOT", "(DEFUN (LOGNOT INTEGER) ((ARG INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGNOT ARG) :CPP \"(~ arg)\" :JAVA \"(~ arg)\")))", ((cpp_function_code)(&lognot)), NULL);
    define_function_object("LOGAND", "(DEFUN (LOGAND INTEGER) ((ARG1 INTEGER) (ARG2 INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGAND ARG1 ARG2) :CPP \"(arg1 & arg2)\" :JAVA \"(arg1 & arg2)\")))", ((cpp_function_code)(&logand)), NULL);
    define_function_object("LOGOR", "(DEFUN (LOGOR INTEGER) ((ARG1 INTEGER) (ARG2 INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGIOR ARG1 ARG2) :CPP \"(arg1 | arg2)\" :JAVA \"(arg1 | arg2)\")))", ((cpp_function_code)(&logor)), NULL);
    define_function_object("LOGXOR", "(DEFUN (LOGXOR INTEGER) ((ARG1 INTEGER) (ARG2 INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:LOGXOR ARG1 ARG2) :CPP \"(arg1 ^ arg2)\" :JAVA \"(arg1 ^ arg2)\")))", ((cpp_function_code)(&logxor)), NULL);
    define_function_object("INTEGER-LENGTH", "(DEFUN (INTEGER-LENGTH INTEGER) ((ARG INTEGER)) :NATIVE? TRUE :PUBLIC? TRUE)", NULL, NULL);
    define_function_object("SHIFT-LEFT", "(DEFUN (SHIFT-LEFT INTEGER) ((ARG INTEGER) (COUNT INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:ASH ARG COUNT) :CPP \"(arg << count)\" :JAVA \"(arg << count)\")))", ((cpp_function_code)(&shift_left)), NULL);
    define_function_object("SHIFT-RIGHT", "(DEFUN (SHIFT-RIGHT INTEGER) ((ARG INTEGER) (COUNT INTEGER)) :GLOBALLY-INLINE? TRUE :PUBLIC? TRUE (RETURN (VERBATIM :COMMON-LISP (CL:ASH ARG (CL:THE CL:FIXNUM (CL:- COUNT))) :CPP \"(arg >> count)\" :JAVA \"(arg >> count)\")))", ((cpp_function_code)(&shift_right)), NULL);
    define_function_object("STELLIFY", "(DEFUN (STELLIFY OBJECT) ((SELF OBJECT)) :DOCUMENTATION \"Convert a Lisp object into a STELLA object.\" :PUBLIC? TRUE)", ((cpp_function_code)(&stellify)), NULL);
    define_function_object("STELLA-OBJECT?", "(DEFUN (STELLA-OBJECT? BOOLEAN) ((SELF OBJECT)) :DOCUMENTATION \"Return TRUE if `self' is a member of the STELLA class `OBJECT'.\")", ((cpp_function_code)(&stella_objectP)), NULL);
    define_function_object("RUNNING-AS-LISP?", "(DEFUN (RUNNING-AS-LISP? BOOLEAN) () :DOCUMENTATION \"Return TRUE if the executable code is a Common Lisp application.\")", ((cpp_function_code)(&running_as_lispP)), NULL);
    define_function_object("GET-TICKTOCK", "(DEFUN (GET-TICKTOCK TICKTOCK) () :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    define_function_object("TICKTOCK-DIFFERENCE", "(DEFUN (TICKTOCK-DIFFERENCE FLOAT) ((T1 TICKTOCK) (T2 TICKTOCK)) :DOCUMENTATION \"The difference in two TICKTOCK time values in seconds where\n`t1' is the earlier time.  The resolution is implementation dependent but will\nnormally be some fractional value of a second.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    define_function_object("TICKTOCK-RESOLUTION", "(DEFUN (TICKTOCK-RESOLUTION FLOAT) () :DOCUMENTATION \"The minimum theoretically detectable resolution of the\ndifference in two TICKTOCK time values in seconds.  This\nresolution is implementation dependent.  It may also not\nbe realizable in practice, since the timing grain size may\nbe larger than this resolution.\" :PUBLIC? TRUE :NATIVE? TRUE)", NULL, NULL);
    define_function_object("SLEEP", "(DEFUN SLEEP ((SECONDS FLOAT)) :DOCUMENTATION \"The program will sleep for the indicated number of seconds.\nFractional values are allowed, but the results are implementation dependent:\nCommon Lisp uses the fractions natively, Java with a resolution of 0.001,\nand C++ can only use integral values.\")", ((cpp_function_code)(&sleep_)), NULL);
    define_function_object("EXCEPTION-MESSAGE", "(DEFUN (EXCEPTION-MESSAGE STRING) ((E NATIVE-EXCEPTION)) :PUBLIC? TRUE :DOCUMENTATION \"Accesses the error message of the exception `e'.\")", ((cpp_function_code)(&exception_message)), NULL);
    define_function_object("PRINT-EXCEPTION-CONTEXT", "(DEFUN PRINT-EXCEPTION-CONTEXT ((E NATIVE-EXCEPTION) (STREAM OUTPUT-STREAM)) :PUBLIC? TRUE :DOCUMENTATION \"Prints a system dependent information about the context of the specified\nexception.  For example, in Java it prints a stack trace.  In Lisp, it is vendor dependent.\")", ((cpp_function_code)(&print_exception_context)), NULL);
    define_function_object("MAKE-PROCESS-LOCK", "(DEFUN (MAKE-PROCESS-LOCK PROCESS-LOCK-OBJECT) ())", ((cpp_function_code)(&make_process_lock)), NULL);
    define_function_object("STARTUP-PRIMAL", "(DEFUN STARTUP-PRIMAL () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_primal)), NULL);
    { Method_Slot* function = lookup_function(SYM_PRIMAL_STELLA_STARTUP_PRIMAL);

      set_dynamic_slot_value(function->dynamic_slots, SYM_PRIMAL_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Primal"), NULL_STRING_WRAPPER);
    }
  }
}

void startup_primal() {
  if (current_startup_time_phaseP(0)) {
    
#ifndef HAVE_SGI_HASH_MAPS
  oUSE_STELLA_HASH_TABLESpo = TRUE;
#endif
;
    oHASH_BYTE_RANDOM_TABLEo = new (GC)(int[256]);
    { int i = NULL_INTEGER;
      int iter_007 = 0;
      int upper_bound_008 = 255;

      while (iter_007 <= upper_bound_008) {
        i = iter_007;
        iter_007 = iter_007 + 1;
        oHASH_BYTE_RANDOM_TABLEo[i] = 0;
      }
    }
    { int bit = NULL_INTEGER;
      int iter_009 = 0;
      int upper_bound_010 = 25;

      while (iter_009 <= upper_bound_010) {
        bit = iter_009;
        iter_009 = iter_009 + 1;
        { int i = NULL_INTEGER;
          int iter_011 = 0;
          int upper_bound_012 = 255;

          while (iter_011 <= upper_bound_012) {
            i = iter_011;
            iter_011 = iter_011 + 1;
            oHASH_BYTE_RANDOM_TABLEo[i] = ((oHASH_BYTE_RANDOM_TABLEo[i]) | (((random_(2)) << bit)));
          }
        }
      }
    }
    oHASH_TABLE_SIZE_PRIMESo = list(25, wrap_integer(29), wrap_integer(53), wrap_integer(97), wrap_integer(193), wrap_integer(389), wrap_integer(769), wrap_integer(1543), wrap_integer(3079), wrap_integer(6151), wrap_integer(12289), wrap_integer(24593), wrap_integer(49157), wrap_integer(98317), wrap_integer(196613), wrap_integer(393241), wrap_integer(786433), wrap_integer(1572869), wrap_integer(3145739), wrap_integer(6291469), wrap_integer(12582917), wrap_integer(25165843), wrap_integer(50331653), wrap_integer(100663319), wrap_integer(201326611), wrap_integer(402653189));
  }
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      help_startup_primal1();
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      help_startup_primal2();
      help_startup_primal3();
      help_startup_primal4();
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT TRUE BOOLEAN (VERBATIM :COMMON-LISP CL:T :CPP \"1\" :JAVA \"true\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT FALSE BOOLEAN (VERBATIM :COMMON-LISP CL:NIL :CPP \"0\" :JAVA \"false\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT PI FLOAT (VERBATIM :COMMON-LISP \"(cl:float CL:PI 0.0d0)\" :CPP \"3.141592653589793\" :JAVA \"java.lang.Math.PI\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(defconstant NULL UNKNOWN NULL)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-INTEGER INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"1 << (sizeof (int) / sizeof (char) * 8 - 1)\" :JAVA \"Integer.MIN_VALUE\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-SHORT-INTEGER SHORT-INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"1 << (sizeof (short int) / sizeof (char) * 8 - 1)\" :JAVA \"Short.MIN_VALUE\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-LONG-INTEGER LONG-INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"1 << (sizeof (long int) / sizeof (char) * 8 - 1)\" :JAVA \"Long.MIN_VALUE\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-UNSIGNED-SHORT-INTEGER UNSIGNED-SHORT-INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"(unsigned short int) -1\" :JAVA \"Short.MIN_VALUE\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-UNSIGNED-LONG-INTEGER UNSIGNED-LONG-INTEGER (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-FIXNUM :CPP \"(unsigned long int) -1\" :JAVA \"Long.MIN_VALUE\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-FLOAT FLOAT (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-DOUBLE-FLOAT :CPP \"-HUGE_VAL /* IEEE infinity, defined in <math.h> */\" :JAVA \"Double.NEGATIVE_INFINITY; /* IEEE Infinity */\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-SINGLE-FLOAT SINGLE-FLOAT (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-SINGLE-FLOAT :CPP \"-HUGE_VAL /* IEEE infinity, defined in <math.h> */\" :JAVA \"Float.NEGATIVE_INFINITY; /* IEEE Infinity */\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-DOUBLE-FLOAT DOUBLE-FLOAT (VERBATIM :COMMON-LISP CL:MOST-NEGATIVE-DOUBLE-FLOAT :CPP \"-HUGE_VAL /* IEEE infinity, defined in <math.h> */\" :JAVA \"Double.NEGATIVE_INFINITY; /* IEEE Infinity */\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-CHARACTER CHARACTER (VERBATIM :COMMON-LISP (CL:CODE-CHAR 0) :CPP \"'\\\\0'\" :JAVA \"Character.MIN_VALUE\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-BYTE BYTE (VERBATIM :COMMON-LISP \"255\" :CPP \"255\" :JAVA \"-128\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFCONSTANT NULL-OCTET OCTET (VERBATIM :COMMON-LISP \"255\" :CPP \"255\" :JAVA \"-128\") :PUBLIC? TRUE)");
      register_native_name(SYM_PRIMAL_STELLA_FLOOR, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      register_native_name(SYM_PRIMAL_STELLA_RANDOM, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      register_native_name(SYM_PRIMAL_STELLA_SQRT, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      register_native_name(SYM_PRIMAL_STELLA_COS, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      register_native_name(SYM_PRIMAL_STELLA_SIN, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      register_native_name(SYM_PRIMAL_STELLA_TAN, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      register_native_name(SYM_PRIMAL_STELLA_LOG, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      register_native_name(SYM_PRIMAL_STELLA_MIN, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      register_native_name(SYM_PRIMAL_STELLA_MAX, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
      define_stella_global_variable_from_stringified_source("(DEFSPECIAL *TRANSIENTOBJECTS?* BOOLEAN FALSE :PUBLIC? TRUE)");
      register_native_name(SYM_PRIMAL_STELLA_HASH_CODE, KWD_PRIMAL_JAVA, KWD_PRIMAL_FUNCTION);
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *HASH-BYTE-RANDOM-TABLE* (ARRAY (256) OF INTEGER) NULL :DOCUMENTATION \"A table of 256 26-bit random numbers that can be used to\nhash sequences of bytes.  Each bit-column in the table has an approximately\neven number of 0's and 1's.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *HASH-TABLE-SIZE-PRIMES* (LIST OF INTEGER-WRAPPER) NULL :DOCUMENTATION \"List of prime numbers approximately growing by a factor of 2\nthat are suitable to be used as hash table sizes.\" :PUBLIC? TRUE)");
      register_native_name(SYM_PRIMAL_STELLA_SLEEP, KWD_PRIMAL_CPP, KWD_PRIMAL_FUNCTION);
    }
  }
}

Symbol* SYM_PRIMAL_STELLA_FLOOR = NULL;

Keyword* KWD_PRIMAL_CPP = NULL;

Keyword* KWD_PRIMAL_FUNCTION = NULL;

Symbol* SYM_PRIMAL_STELLA_RANDOM = NULL;

Symbol* SYM_PRIMAL_STELLA_SQRT = NULL;

Symbol* SYM_PRIMAL_STELLA_COS = NULL;

Symbol* SYM_PRIMAL_STELLA_SIN = NULL;

Symbol* SYM_PRIMAL_STELLA_TAN = NULL;

Symbol* SYM_PRIMAL_STELLA_LOG = NULL;

Symbol* SYM_PRIMAL_STELLA_MIN = NULL;

Symbol* SYM_PRIMAL_STELLA_MAX = NULL;

Keyword* KWD_PRIMAL_UPCASE = NULL;

Keyword* KWD_PRIMAL_DOWNCASE = NULL;

Keyword* KWD_PRIMAL_CAPITALIZE = NULL;

Keyword* KWD_PRIMAL_PRESERVE = NULL;

Keyword* KWD_PRIMAL_LEFT = NULL;

Keyword* KWD_PRIMAL_RIGHT = NULL;

Keyword* KWD_PRIMAL_CENTER = NULL;

Symbol* SYM_PRIMAL_STELLA_HASH_CODE = NULL;

Keyword* KWD_PRIMAL_JAVA = NULL;

Symbol* SYM_PRIMAL_STELLA_SLEEP = NULL;

Symbol* SYM_PRIMAL_STELLA_STARTUP_PRIMAL = NULL;

Symbol* SYM_PRIMAL_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella
