// startup.cc

/*--------------------------- BEGIN LICENSE BLOCK ---------------------------+
|                                                                            |
| Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          |
|                                                                            |
| The contents of this file are subject to the Mozilla Public License        |
| Version 1.1 (the "License"); you may not use this file except in           |
| compliance with the License. You may obtain a copy of the License at       |
| http://www.mozilla.org/MPL/                                                |
|                                                                            |
| Software distributed under the License is distributed on an "AS IS" basis, |
| WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   |
| for the specific language governing rights and limitations under the       |
| License.                                                                   |
|                                                                            |
| The Original Code is the STELLA Programming Language.                      |
|                                                                            |
| The Initial Developer of the Original Code is                              |
| UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          |
| 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               |
|                                                                            |
| Portions created by the Initial Developer are Copyright (C) 1996-2001      |
| the Initial Developer. All Rights Reserved.                                |
|                                                                            |
| Contributor(s):                                                            |
|                                                                            |
| Alternatively, the contents of this file may be used under the terms of    |
| either the GNU General Public License Version 2 or later (the "GPL"), or   |
| the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   |
| in which case the provisions of the GPL or the LGPL are applicable instead |
| of those above. If you wish to allow use of your version of this file only |
| under the terms of either the GPL or the LGPL, and not to allow others to  |
| use your version of this file under the terms of the MPL, indicate your    |
| decision by deleting the provisions above and replace them with the notice |
| and other provisions required by the GPL or the LGPL. If you do not delete |
| the provisions above, a recipient may use your version of this file under  |
| the terms of any one of the MPL, the GPL or the LGPL.                      |
|                                                                            |
+---------------------------- END LICENSE BLOCK ----------------------------*/

#include "stella/stella-system.hh"

namespace stella {

int oSTELLA_MAJOR_VERSION_NUMBERo = 3;

int oSTELLA_MINOR_VERSION_NUMBERo = 0;

char* oSTELLA_RELEASE_STATEo = ".beta";

int oSTELLA_PATCH_LEVELo = 0;

char* oSTELLA_VERSION_STRINGo = NULL;

// List of phases that can be legally used as an optional
// phase argument to a `startup-time-progn' form.  The corresponding code
// will be executed during the execution of a startup-time-code function only
// if the position of the keyword in the list corresponds to the current value of
// `*STARTUP-TIME-PHASE*', or if phasing of startup-time code is disabled.
List* oSTARTUP_TIME_PHASESo = NULL;

// The current phase during 'phased startup'.
// The value has to correspond to the position of one of the keywords
// in `*STARTUP-TIME-PHASES*'.  999 means no phasing at all.
int oSTARTUP_TIME_PHASEo = 999;

boolean current_startup_time_phaseP(int phase) {
  return ((oSTARTUP_TIME_PHASEo == 999) ||
      (phase == oSTARTUP_TIME_PHASEo));
}

int encode_startup_time_phase(Keyword* phase) {
  { int startuptimephaseasnumber = 0;

    if (!(oSTARTUP_TIME_PHASESo->memberP(phase))) {
      phase = KWD_STARTUP_FINAL;
    }
    { Keyword* ph = NULL;
      Cons* iter_000 = oSTARTUP_TIME_PHASESo->the_cons_list;

      while (!(iter_000 == NIL)) {
        ph = ((Keyword*)(iter_000->value));
        iter_000 = iter_000->rest;
        if (ph == phase) {
          return (startuptimephaseasnumber);
        }
        else {
          startuptimephaseasnumber = startuptimephaseasnumber + 1;
        }
      }
    }
    throw *new_stella_exception("`encode-startup-time-phase' should never get this far!");
  }
}

void startup_kernel(boolean verboseP) {
  { int phase = NULL_INTEGER;
    int iter_000 = 0;
    int upper_bound_000 = 9;
    boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

    while (unboundedP_000 ||
        (iter_000 <= upper_bound_000)) {
      phase = iter_000;
      iter_000 = iter_000 + 1;
      oSTARTUP_TIME_PHASEo = phase;
      if (verboseP) {
        if (phase == 0) {
        }
        else if (phase == 2) {
          cout << "Running kernel startup code..." << endl;
          cout << "Initializing symbol tables..." << endl;
        }
        else if (phase == 3) {
          cout << "Initializing quoted constants..." << endl;
        }
        else if (phase == 4) {
          cout << "Initializing global variables..." << endl;
        }
        else if (phase == 5) {
          cout << "Creating class objects..." << endl;
        }
        else if (phase == 6) {
          cout << "Finalizing classes..." << endl;
        }
        else if (phase == 7) {
          cout << "Creating method objects..." << endl;
        }
        else if (phase == 8) {
          cout << "Finalizing methods..." << endl;
        }
        else if (phase == 9) {
          cout << "Running non-phased startup code..." << endl;
        }
      }
      if (phase == 6) {
        finalize_classes();
        oCLASS_HIERARCHY_BOOTEDpo = TRUE;
        continue;
      }
      if (phase == 8) {
        finalize_slots();
        cleanup_unfinalized_classes();
        continue;
      }
      startup_startup();
      startup_cpp_primal();
;
      startup_primal();
      startup_type_predicates();
      startup_conses();
      startup_hierarchy();
      startup_taxonomies();
      startup_lists();
      startup_collections();
      startup_iterators();
      startup_literals();
      startup_symbols();
      startup_classes();
      startup_methods();
      startup_defclass();
      startup_stella_in();
      startup_foreach();
      startup_walk();
      startup_dynamic_slots();
      startup_dynamic_literal_slots();
      startup_systems();
      startup_cl_translate();
      startup_macros();
      startup_memoize();
      startup_streams();
      startup_describe();
      startup_demons();
      startup_more_demons();
      startup_name_utility();
      startup_modules();
      startup_contexts();
      startup_read();
      startup_date_time();
      startup_date_time_parser();
    }
  }
}

void startup_cpp_translator() {
  startup_cpp_translate();
  startup_cpp_class_out();
  startup_cpp_output();
  startup_cpp_translate_file();
}

void startup_java_translator() {
  startup_java_translate();
  startup_java_class_out();
  startup_java_output();
  startup_java_translate_file();
}

void startup_idl_translator() {
  startup_idl_translate();
  startup_idl_class_out();
  startup_idl_output();
  startup_idl_translate_file();
}

void startup(boolean verboseP) {
  startup_kernel(verboseP);
  if (verboseP) {
    cout << "Starting up translators..." << endl;
  }
  { int phase = NULL_INTEGER;
    int iter_000 = 0;
    int upper_bound_000 = 9;
    boolean unboundedP_000 = upper_bound_000 == NULL_INTEGER;

    while (unboundedP_000 ||
        (iter_000 <= upper_bound_000)) {
      phase = iter_000;
      iter_000 = iter_000 + 1;
      oSTARTUP_TIME_PHASEo = phase;
      startup_translate_file();
      startup_cl_translate_file();
      startup_cpp_translator();
      startup_java_translator();
      startup_idl_translator();
    }
  }
  oSTARTUP_TIME_PHASEo = 999;
  sweep_transients();
  if (verboseP) {
    cout << "Welcome to " << "`" << oSTELLA_VERSION_STRINGo << "'" << endl;
  }
}

int main() {
  // Main STELLA entry point for testing.
  cout << "Welcome to STELLA!" << endl;
  startup(TRUE);
  cout << "Bye!" << endl;
  return (1);
}

void startup_startup() {
  { 
    BIND_STELLA_SPECIAL(oMODULEo, Module*, oSTELLA_MODULEo);
    BIND_STELLA_SPECIAL(oCONTEXTo, Context*, oMODULEo.get());
    if (current_startup_time_phaseP(2)) {
      KWD_STARTUP_EARLY_INITS = ((Keyword*)(intern_rigid_symbol_wrt_module("EARLY-INITS", NULL, 2)));
      KWD_STARTUP_MODULES = ((Keyword*)(intern_rigid_symbol_wrt_module("MODULES", NULL, 2)));
      KWD_STARTUP_SYMBOLS = ((Keyword*)(intern_rigid_symbol_wrt_module("SYMBOLS", NULL, 2)));
      KWD_STARTUP_QUOTED_CONSTANTS = ((Keyword*)(intern_rigid_symbol_wrt_module("QUOTED-CONSTANTS", NULL, 2)));
      KWD_STARTUP_GLOBALS = ((Keyword*)(intern_rigid_symbol_wrt_module("GLOBALS", NULL, 2)));
      KWD_STARTUP_CLASSES = ((Keyword*)(intern_rigid_symbol_wrt_module("CLASSES", NULL, 2)));
      KWD_STARTUP_FINALIZE_CLASSES = ((Keyword*)(intern_rigid_symbol_wrt_module("FINALIZE-CLASSES", NULL, 2)));
      KWD_STARTUP_METHODS = ((Keyword*)(intern_rigid_symbol_wrt_module("METHODS", NULL, 2)));
      KWD_STARTUP_FINALIZE_METHODS = ((Keyword*)(intern_rigid_symbol_wrt_module("FINALIZE-METHODS", NULL, 2)));
      KWD_STARTUP_FINAL = ((Keyword*)(intern_rigid_symbol_wrt_module("FINAL", NULL, 2)));
      SYM_STARTUP_STELLA_STARTUP_STARTUP = ((Symbol*)(intern_rigid_symbol_wrt_module("STARTUP-STARTUP", NULL, 0)));
      SYM_STARTUP_STELLA_METHOD_STARTUP_CLASSNAME = ((Symbol*)(intern_rigid_symbol_wrt_module("METHOD-STARTUP-CLASSNAME", NULL, 0)));
    }
    if (current_startup_time_phaseP(4)) {
      oSTELLA_VERSION_STRINGo = string_concatenate("STELLA ", integer_to_string(oSTELLA_MAJOR_VERSION_NUMBERo), 6, ".", integer_to_string(oSTELLA_MINOR_VERSION_NUMBERo), oSTELLA_RELEASE_STATEo, " (patch-level ", integer_to_string(oSTELLA_PATCH_LEVELo), ")");
      oSTARTUP_TIME_PHASESo = ((List*)(list(10, KWD_STARTUP_EARLY_INITS, KWD_STARTUP_MODULES, KWD_STARTUP_SYMBOLS, KWD_STARTUP_QUOTED_CONSTANTS, KWD_STARTUP_GLOBALS, KWD_STARTUP_CLASSES, KWD_STARTUP_FINALIZE_CLASSES, KWD_STARTUP_METHODS, KWD_STARTUP_FINALIZE_METHODS, KWD_STARTUP_FINAL)));
    }
    if (current_startup_time_phaseP(6)) {
      finalize_classes();
    }
    if (current_startup_time_phaseP(7)) {
      define_function_object("CURRENT-STARTUP-TIME-PHASE?", "(DEFUN (CURRENT-STARTUP-TIME-PHASE? BOOLEAN) ((PHASE INTEGER)) :PUBLIC? TRUE)", ((cpp_function_code)(&current_startup_time_phaseP)), NULL);
      define_function_object("ENCODE-STARTUP-TIME-PHASE", "(DEFUN (ENCODE-STARTUP-TIME-PHASE INTEGER) ((PHASE KEYWORD)))", ((cpp_function_code)(&encode_startup_time_phase)), NULL);
      define_function_object("STARTUP-KERNEL", "(DEFUN STARTUP-KERNEL ((VERBOSE? BOOLEAN)))", ((cpp_function_code)(&startup_kernel)), NULL);
      define_function_object("STARTUP-CPP-TRANSLATOR", "(DEFUN STARTUP-CPP-TRANSLATOR ())", ((cpp_function_code)(&startup_cpp_translator)), NULL);
      define_function_object("STARTUP-JAVA-TRANSLATOR", "(DEFUN STARTUP-JAVA-TRANSLATOR ())", ((cpp_function_code)(&startup_java_translator)), NULL);
      define_function_object("STARTUP-IDL-TRANSLATOR", "(DEFUN STARTUP-IDL-TRANSLATOR ())", ((cpp_function_code)(&startup_idl_translator)), NULL);
      define_function_object("STARTUP", "(DEFUN STARTUP ((VERBOSE? BOOLEAN)) :PUBLIC? TRUE)", ((cpp_function_code)(&startup)), NULL);
      define_function_object("MAIN", "(DEFUN (MAIN INTEGER) () :PUBLIC? TRUE :DOCUMENTATION \"Main STELLA entry point for testing.\")", NULL, NULL);
      define_function_object("STARTUP-STARTUP", "(DEFUN STARTUP-STARTUP () :PUBLIC? TRUE)", ((cpp_function_code)(&startup_startup)), NULL);
      { Method_Slot* function = lookup_function(SYM_STARTUP_STELLA_STARTUP_STARTUP);

        set_dynamic_slot_value(function->dynamic_slots, SYM_STARTUP_STELLA_METHOD_STARTUP_CLASSNAME, wrap_string("Startup-Startup"), NULL_STRING_WRAPPER);
      }
    }
    if (current_startup_time_phaseP(8)) {
      finalize_slots();
      cleanup_unfinalized_classes();
    }
    if (current_startup_time_phaseP(9)) {
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STELLA-MAJOR-VERSION-NUMBER* INTEGER 3 :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STELLA-MINOR-VERSION-NUMBER* INTEGER 0 :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STELLA-RELEASE-STATE* STRING \".beta\" :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STELLA-PATCH-LEVEL* INTEGER 0 :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STELLA-VERSION-STRING* STRING (CONCATENATE \"STELLA \" (INTEGER-TO-STRING *STELLA-MAJOR-VERSION-NUMBER*) \".\" (INTEGER-TO-STRING *STELLA-MINOR-VERSION-NUMBER*) *STELLA-RELEASE-STATE* \" (patch-level \" (INTEGER-TO-STRING *STELLA-PATCH-LEVEL*) \")\") :PUBLIC? TRUE)");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STARTUP-TIME-PHASES* (LIST OF KEYWORD) (CAST (LIST :EARLY-INITS :MODULES :SYMBOLS :QUOTED-CONSTANTS :GLOBALS :CLASSES :FINALIZE-CLASSES :METHODS :FINALIZE-METHODS :FINAL) (LIST OF KEYWORD)) :DOCUMENTATION \"List of phases that can be legally used as an optional\nphase argument to a `startup-time-progn' form.  The corresponding code\nwill be executed during the execution of a startup-time-code function only\nif the position of the keyword in the list corresponds to the current value of\n`*STARTUP-TIME-PHASE*', or if phasing of startup-time code is disabled.\")");
      define_stella_global_variable_from_stringified_source("(DEFGLOBAL *STARTUP-TIME-PHASE* INTEGER 999 :PUBLIC? TRUE :DOCUMENTATION \"The current phase during 'phased startup'.\nThe value has to correspond to the position of one of the keywords\nin `*STARTUP-TIME-PHASES*'.  999 means no phasing at all.\")");
    }
  }
}

Keyword* KWD_STARTUP_EARLY_INITS = NULL;

Keyword* KWD_STARTUP_MODULES = NULL;

Keyword* KWD_STARTUP_SYMBOLS = NULL;

Keyword* KWD_STARTUP_QUOTED_CONSTANTS = NULL;

Keyword* KWD_STARTUP_GLOBALS = NULL;

Keyword* KWD_STARTUP_CLASSES = NULL;

Keyword* KWD_STARTUP_FINALIZE_CLASSES = NULL;

Keyword* KWD_STARTUP_METHODS = NULL;

Keyword* KWD_STARTUP_FINALIZE_METHODS = NULL;

Keyword* KWD_STARTUP_FINAL = NULL;

Symbol* SYM_STARTUP_STELLA_STARTUP_STARTUP = NULL;

Symbol* SYM_STARTUP_STELLA_METHOD_STARTUP_CLASSNAME = NULL;

} // end of namespace stella


using namespace stella;

int main() {
  // Main STELLA entry point for testing.
  cout << "Welcome to STELLA!" << endl;
  startup(TRUE);
  cout << "Bye!" << endl;
  return (1);
}

