;;; -*- Mode: Lisp; Package: CL-USER; Syntax: COMMON-LISP; Base: 10 -*-

;;;;;;;;;;;;;;;;;;;;;;;;;;;; BEGIN LICENSE BLOCK ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                            ;
; Version: MPL 1.1/GPL 2.0/LGPL 2.1                                          ;
;                                                                            ;
; The contents of this file are subject to the Mozilla Public License        ;
; Version 1.1 (the "License"); you may not use this file except in           ;
; compliance with the License. You may obtain a copy of the License at       ;
; http://www.mozilla.org/MPL/                                                ;
;                                                                            ;
; Software distributed under the License is distributed on an "AS IS" basis, ;
; WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License   ;
; for the specific language governing rights and limitations under the       ;
; License.                                                                   ;
;                                                                            ;
; The Original Code is the STELLA Programming Language.                      ;
;                                                                            ;
; The Initial Developer of the Original Code is                              ;
; UNIVERSITY OF SOUTHERN CALIFORNIA, INFORMATION SCIENCES INSTITUTE          ;
; 4676 Admiralty Way, Marina Del Rey, California 90292, U.S.A.               ;
;                                                                            ;
; Portions created by the Initial Developer are Copyright (C) 1996-2001      ;
; the Initial Developer. All Rights Reserved.                                ;
;                                                                            ;
; Contributor(s):                                                            ;
;                                                                            ;
; Alternatively, the contents of this file may be used under the terms of    ;
; either the GNU General Public License Version 2 or later (the "GPL"), or   ;
; the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),   ;
; in which case the provisions of the GPL or the LGPL are applicable instead ;
; of those above. If you wish to allow use of your version of this file only ;
; under the terms of either the GPL or the LGPL, and not to allow others to  ;
; use your version of this file under the terms of the MPL, indicate your    ;
; decision by deleting the provisions above and replace them with the notice ;
; and other provisions required by the GPL or the LGPL. If you do not delete ;
; the provisions above, a recipient may use your version of this file under  ;
; the terms of any one of the MPL, the GPL or the LGPL.                      ;
;                                                                            ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;; END LICENSE BLOCK ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;; Version: translations.lisp,v 1.14 2001/10/12 01:47:34 hans Exp

;;; Pathname translations for PowerLoom and related systems.

(in-package "CL-USER")


(defparameter *powerloom-root-directory*  
  (namestring (truename (make-pathname :host (pathname-host *load-pathname*)
				       :directory (pathname-directory *load-pathname*)))))

(defparameter *powerloom-binary-directory*
    (format nil "~Abin/~A/"
            *powerloom-root-directory*
            (or (some #'(lambda (entry)
                          (when (find (first entry) *features*)
                            (second entry)))
                      '((:ALLEGRO-V4.1 "acl4.1")
                        (:ALLEGRO-V4.2 "acl4.2")
                        (:ALLEGRO-V4.3 "acl4.3")
                        (:ALLEGRO-V5.0 "acl5.0")
                        (:ALLEGRO-V5.0.1 "acl5.0")
                        (:ALLEGRO-V6.0 "acl6.0")
                        (:ALLEGRO      "aclx.x")
                        (:LCL4.1       "lcl4.1")
                        (:LCL4.0       "lcl4.0")
                        (:LCL          "lclx.x")
                        (:MCL          "mcl")
                        (:LISPWORKS4.0 "lw4.0")
                        (:LISPWORKS4.1 "lw4.1")
                        ))
                "lisp")))

(let* ((directorySeparator
        (char *powerloom-root-directory*
              (1- (length *powerloom-root-directory*))))
       ;; try to figure out dynamically whether the current Lisp/OS
       ;;    combination wants a two or three-argument physical pattern,
       ;;    since that behavior seems to be somewhat unpredictable:
       (physicalPattern
        (or (ignore-errors
             (progn (setf (logical-pathname-translations "PL")
                      `(("sources;**;*.*.*"
                         ,(format nil "sources~a*.*.*" directorySeparator))))
                    (translate-logical-pathname "PL:sources;foo.ste")
                    "*.*.*"))
            "*.*"))
       (sourcesTranslation
        (substitute directorySeparator #\/
                    (format nil "~Asources/**/~a"
                            *powerloom-root-directory*
                            physicalPattern)))
       (binariesTranslation
	(substitute directorySeparator #\/
                    (format nil "~a**/~a"
                            *powerloom-binary-directory*
                            physicalPattern)))
       (nativesTranslation
        (substitute directorySeparator #\/
                    (format nil "~anative/**/~a"
                            *powerloom-root-directory*
                            physicalPattern))))
  (setf (logical-pathname-translations "PL")
    `(("sources;**;*.*.*" ,sourcesTranslation)
      ("binaries;**;*.*.*" ,binariesTranslation)
      ("natives;**;*.*.*" ,nativesTranslation))))
